@@ -202,6 +202,8 @@ int
 netsnmp_register_handler(netsnmp_handler_registration *reginfo)
 {
     netsnmp_mib_handler *handler;
+    int flags = 0;
+
     if (reginfo == NULL) {
         snmp_log(LOG_ERR, "netsnmp_register_handler() called illegally\n");
         netsnmp_assert(reginfo != NULL);
@@ -245,13 +247,18 @@ netsnmp_register_handler(netsnmp_handler_registration *reginfo)
                                netsnmp_get_bulk_to_next_handler());
     }
 
+    for (handler = reginfo->handler; handler; handler = handler->next) {
+        if (handler->flags & MIB_HANDLER_INSTANCE)
+            flags = FULLY_QUALIFIED_INSTANCE;
+    }
+
     return netsnmp_register_mib(reginfo->handlerName,
                                 NULL, 0, 0,
                                 reginfo->rootoid, reginfo->rootoid_len,
                                 reginfo->priority,
                                 reginfo->range_subid,
                                 reginfo->range_ubound, NULL,
-                                reginfo->contextName, reginfo->timeout, 0,
+                                reginfo->contextName, reginfo->timeout, flags,
                                 reginfo, 1);
 }
 
@@ -572,6 +579,10 @@ netsnmp_handler_free(netsnmp_mib_handler *handler)
          *  defined. About 30 functions down the stack, starting
          *  in clear_context() -> clear_subtree()
          */
+        if ((handler->myvoid != NULL) && (handler->data_free != NULL))
+        {
+            handler->data_free(handler->myvoid);
+        }
         SNMP_FREE(handler->handler_name);
         SNMP_FREE(handler);
     }
@@ -593,6 +604,7 @@ netsnmp_handler_dup(netsnmp_mib_handler *handler)
 
     if (h != NULL) {
         h->myvoid = handler->myvoid;
+        h->data_free = handler->data_free;
 
         if (handler->next != NULL) {
             h->next = netsnmp_handler_dup(handler->next);
@@ -617,6 +629,7 @@ netsnmp_handler_registration_free(netsnmp_handler_registration *reginfo)
         SNMP_FREE(reginfo->handlerName);
         SNMP_FREE(reginfo->contextName);
         SNMP_FREE(reginfo->rootoid);
+        reginfo->rootoid_len = 0;
         SNMP_FREE(reginfo);
     }
 }
@@ -798,7 +811,7 @@ netsnmp_request_remove_list_data(netsnmp_request_info *request,
  *         returned if request is NULL or request->parent_data is NULL or
  *         request->parent_data object is not found.
  */
-NETSNMP_INLINE void    *
+void    *
 netsnmp_request_get_list_data(netsnmp_request_info *request,
                               const char *name)
 {
