@@ -57,15 +57,12 @@
 #endif
 #include <net-snmp/utilities.h>
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
-
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/agent_trap.h>
 #include <net-snmp/agent/snmp_agent.h>
 #include <net-snmp/agent/agent_callbacks.h>
 
+#include <net-snmp/agent/agent_module_config.h>
 #include <net-snmp/agent/mib_module_config.h>
 
 #ifdef USING_AGENTX_PROTOCOL_MODULE
@@ -83,8 +80,8 @@ struct trap_sink *sinks = NULL;
 
 extern struct timeval starttime;
 
-oid             objid_enterprisetrap[] = { NOTIFICATION_MIB };
-oid             trap_version_id[] = { SYSTEM_MIB };
+oid             objid_enterprisetrap[] = { NETSNMP_NOTIFICATION_MIB };
+oid             trap_version_id[] = { NETSNMP_SYSTEM_MIB };
 int             enterprisetrap_len;
 int             trap_version_id_len;
 
@@ -111,7 +108,7 @@ oid             agentaddr_oid[] = { SNMPV2_COMM_OBJS_PREFIX, 3, 0 };
 size_t          agentaddr_oid_len;
 oid             community_oid[] = { SNMPV2_COMM_OBJS_PREFIX, 4, 0 };
 size_t          community_oid_len;
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
 char           *snmp_trapcommunity = NULL;
 #endif
 
@@ -228,26 +225,15 @@ remove_trap_session(netsnmp_session * ss)
     return 0;
 }
 
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
-int
-create_trap_session(char *sink, u_short sinkport,
-                    char *com, int version, int pdutype)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
+static int
+create_trap_session2(const char *sink, const char* sinkport,
+		     char *com, int version, int pdutype)
 {
+    netsnmp_transport *t;
     netsnmp_session session, *sesp;
-    char           *peername = NULL;
-    int             len;
-
-    len = strlen(sink) + 4 + 32;
-    if ((peername = malloc(len)) == NULL) {
-        return 0;
-    } else if (NULL != strchr(sink,':')) {
-        snprintf(peername, len, "%s", sink);
-    } else {
-        snprintf(peername, len, "udp:%s:%hu", sink, sinkport);
-    }
 
     memset(&session, 0, sizeof(netsnmp_session));
-    session.peername = peername;
     session.version = version;
     if (com) {
         session.community = (u_char *) com;
@@ -269,44 +255,62 @@ create_trap_session(char *sink, u_short sinkport,
                                        NETSNMP_DS_LIB_CLIENT_ADDR)) && 
         ((0 == strcmp("localhost",sink)) || (0 == strcmp("127.0.0.1",sink))))
         session.localname = "localhost";
-    sesp = snmp_open(&session);
-    free(peername);
 
-    if (sesp) {
-        return add_trap_session(sesp, pdutype,
-                                (pdutype == SNMP_MSG_INFORM), version);
+    t = netsnmp_tdomain_transport_full("snmptrap", sink, 0, NULL, sinkport);
+    if (t != NULL) {
+	sesp = snmp_add(&session, t, NULL, NULL);
+
+	if (sesp) {
+	    return add_trap_session(sesp, pdutype,
+				    (pdutype == SNMP_MSG_INFORM), version);
+	}
     }
-
     /*
      * diagnose snmp_open errors with the input netsnmp_session pointer 
      */
     snmp_sess_perror("snmpd: create_trap_session", &session);
     return 0;
 }
+
+int
+create_trap_session(char *sink, u_short sinkport,
+		    char *com, int version, int pdutype)
+{
+    char buf[sizeof(sinkport) * 3 + 2];
+    if (sinkport != 0) {
+	sprintf(buf, ":%hu", sinkport);
+	snmp_log(LOG_NOTICE,
+		 "Using a separate port number is deprecated, please correct "
+		 "the sink specification instead");
+    }
+    return create_trap_session2(sink, sinkport ? buf : NULL, com, version,
+				pdutype);
+}
+
 #endif /* support for community based SNMP */
 
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
 static int
-create_v1_trap_session(char *sink, u_short sinkport, char *com)
+create_v1_trap_session(char *sink, const char *sinkport, char *com)
 {
-    return create_trap_session(sink, sinkport, com,
-                               SNMP_VERSION_1, SNMP_MSG_TRAP);
+    return create_trap_session2(sink, sinkport, com,
+				SNMP_VERSION_1, SNMP_MSG_TRAP);
 }
 #endif
 
-#ifndef DISABLE_SNMPV2C
+#ifndef NETSNMP_DISABLE_SNMPV2C
 static int
-create_v2_trap_session(char *sink, u_short sinkport, char *com)
+create_v2_trap_session(const char *sink, const char *sinkport, char *com)
 {
-    return create_trap_session(sink, sinkport, com,
-                               SNMP_VERSION_2c, SNMP_MSG_TRAP2);
+    return create_trap_session2(sink, sinkport, com,
+				SNMP_VERSION_2c, SNMP_MSG_TRAP2);
 }
 
 static int
-create_v2_inform_session(char *sink, u_short sinkport, char *com)
+create_v2_inform_session(const char *sink, const char *sinkport, char *com)
 {
-    return create_trap_session(sink, sinkport, com,
-                               SNMP_VERSION_2c, SNMP_MSG_INFORM);
+    return create_trap_session2(sink, sinkport, com,
+				SNMP_VERSION_2c, SNMP_MSG_INFORM);
 }
 #endif
 
@@ -328,20 +332,6 @@ snmpd_free_trapsinks(void)
 	 *******************/
 
 
-netsnmp_variable_list*
-find_varbind_in_list( netsnmp_variable_list *vblist,
-                      oid *name, size_t len)
-{
-    netsnmp_variable_list *v;
-
-    for (v=vblist; v; v=v->next_variable)
-        if (!snmp_oid_compare(v->name, v->name_length,
-                              name,    len))
-            return v;
-
-    return NULL;
-}
-
 netsnmp_pdu*
 convert_v2pdu_to_v1( netsnmp_pdu* template_v2pdu )
 {
@@ -391,6 +381,23 @@ convert_v2pdu_to_v1( netsnmp_pdu* template_v2pdu )
         return NULL;
     }
 
+    /*
+     * Check the v2 varbind list for any varbinds
+     *  that are not valid in an SNMPv1 trap.
+     *  This basically means Counter64 values.
+     *
+     * RFC 2089 said to omit such varbinds from the list.
+     * RFC 2576/3584 say to drop the trap completely.
+     */
+    for (var = vblist->next_variable; var; var = var->next_variable) {
+        if ( var->type == ASN_COUNTER64 ) {
+            snmp_log(LOG_WARNING,
+                     "send_trap: v1 traps can't carry Counter64 varbinds\n");
+            snmp_free_pdu(template_v1pdu);
+            return NULL;
+        }
+    }
+
     /*
      * Set the generic & specific trap types,
      *    and the enterprise field from the v2 varbind list.
@@ -424,6 +431,12 @@ convert_v2pdu_to_v1( netsnmp_pdu* template_v2pdu )
          *   into enterprise and specific trap
          */
         len = vblist->val_len / sizeof(oid);
+        if ( len <= 2 ) {
+            snmp_log(LOG_WARNING,
+                     "send_trap: v2 trapOID too short (%d)\n", len);
+            snmp_free_pdu(template_v1pdu);
+            return NULL;
+        }
         template_v1pdu->trap_type     = SNMP_TRAP_ENTERPRISESPECIFIC;
         template_v1pdu->specific_type = vblist->val.objid[len - 1];
         len--;
@@ -557,8 +570,7 @@ convert_v1pdu_to_v2( netsnmp_pdu* template_v1pdu )
     var = find_varbind_in_list( template_v2pdu->variables,
                                 snmptrapenterprise_oid,
                                 snmptrapenterprise_oid_len);
-    if (!var && 
-        template_v1pdu->trap_type != SNMP_TRAP_ENTERPRISESPECIFIC) {
+    if (!var) {
         if (!snmp_varlist_add_variable( &(template_v2pdu->variables),
                  snmptrapenterprise_oid, snmptrapenterprise_oid_len,
                  ASN_OBJECT_ID,
@@ -724,8 +736,6 @@ netsnmp_send_traps(int trap, int specific,
         if (!template_v1pdu) {
             snmp_log(LOG_WARNING,
                      "send_trap: failed to convert v2->v1 template PDU\n");
-            snmp_free_pdu(template_v2pdu);
-            return -1;
         }
 
     } else {
@@ -764,26 +774,26 @@ netsnmp_send_traps(int trap, int specific,
         if (!template_v2pdu) {
             snmp_log(LOG_WARNING,
                      "send_trap: failed to convert v1->v2 template PDU\n");
-            snmp_free_pdu(template_v1pdu);
-            return -1;
         }
     }
 
     /*
      * Check whether we're ignoring authFail traps
      */
-    if (template_v1pdu->trap_type == SNMP_TRAP_AUTHFAIL &&
+    if (template_v1pdu) {
+      if (template_v1pdu->trap_type == SNMP_TRAP_AUTHFAIL &&
         snmp_enableauthentraps == SNMP_AUTHENTICATED_TRAPS_DISABLED) {
         snmp_free_pdu(template_v1pdu);
         snmp_free_pdu(template_v2pdu);
         return 0;
-    }
+      }
 
     /*
      * Ensure that the v1 trap PDU includes the local IP address
      */
-     pdu_in_addr_t = (in_addr_t *) template_v1pdu->agent_addr;
-    *pdu_in_addr_t = get_myaddr();
+       pdu_in_addr_t = (in_addr_t *) template_v1pdu->agent_addr;
+      *pdu_in_addr_t = get_myaddr();
+    }
 
 
     /*
@@ -792,20 +802,26 @@ netsnmp_send_traps(int trap, int specific,
      *   providing an appropriately formatted PDU in each case
      */
     for (sink = sinks; sink; sink = sink->next) {
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
         if (sink->version == SNMP_VERSION_1) {
+          if (template_v1pdu) {
             send_trap_to_sess(sink->sesp, template_v1pdu);
+          }
         } else {
 #endif
+          if (template_v2pdu) {
             template_v2pdu->command = sink->pdutype;
             send_trap_to_sess(sink->sesp, template_v2pdu);
-#ifndef DISABLE_SNMPV1
+          }
+#ifndef NETSNMP_DISABLE_SNMPV1
         }
 #endif
     }
-    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
+    if (template_v1pdu)
+        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                         SNMPD_CALLBACK_SEND_TRAP1, template_v1pdu);
-    snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
+    if (template_v2pdu)
+        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                         SNMPD_CALLBACK_SEND_TRAP2, template_v2pdu);
     snmp_free_pdu(template_v1pdu);
     snmp_free_pdu(template_v2pdu);
@@ -873,6 +889,9 @@ send_trap_to_sess(netsnmp_session * sess, netsnmp_pdu *template_pdu)
 {
     netsnmp_pdu    *pdu;
     int            result;
+    char           tmp[SPRINT_MAX_LEN];
+    int            len;
+
 
     if (!sess || !template_pdu)
         return;
@@ -880,10 +899,13 @@ send_trap_to_sess(netsnmp_session * sess, netsnmp_pdu *template_pdu)
     DEBUGMSGTL(("trap", "sending trap type=%d, version=%d\n",
                 template_pdu->command, sess->version));
 
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
     if (sess->version == SNMP_VERSION_1 &&
         (template_pdu->command != SNMP_MSG_TRAP))
         return;                 /* Skip v1 sinks for v2 only traps */
+    if (sess->version != SNMP_VERSION_1 &&
+        (template_pdu->command == SNMP_MSG_TRAP))
+        return;                 /* Skip v2+ sinks for v1 only traps */
 #endif
     template_pdu->version = sess->version;
     pdu = snmp_clone_pdu(template_pdu);
@@ -898,6 +920,14 @@ send_trap_to_sess(netsnmp_session * sess, netsnmp_pdu *template_pdu)
             snmp_async_send(sess, pdu, &handle_inform_response, NULL);
         
     } else {
+        if ((sess->version == SNMP_VERSION_3) &&
+                (pdu->command == SNMP_MSG_TRAP2) &&
+                (pdu->securityEngineIDLen == 0)) {
+            len = snmpv3_get_engineID(tmp, sizeof(tmp));
+            memdup(&pdu->securityEngineID, tmp, len);
+            pdu->securityEngineIDLen = len;
+        }
+
         result = snmp_send(sess, pdu);
     }
 
@@ -928,10 +958,10 @@ send_trap_vars(int trap, int specific, netsnmp_variable_list * vars)
  *
  * This function eventually calls send_enterprise_trap_vars.  If the
  * trap type is not set to SNMP_TRAP_ENTERPRISESPECIFIC the enterprise 
- * and enterprise_length paramater is set to the pre defined SYSTEM_MIB 
+ * and enterprise_length paramater is set to the pre defined NETSNMP_SYSTEM_MIB 
  * oid and length respectively.  If the trap type is set to 
  * SNMP_TRAP_ENTERPRISESPECIFIC the enterprise and enterprise_length 
- * parameters are set to the pre-defined NOTIFICATION_MIB oid and length 
+ * parameters are set to the pre-defined NETSNMP_NOTIFICATION_MIB oid and length 
  * respectively.
  *
  * @param trap is the generic trap type.
@@ -958,10 +988,10 @@ send_easy_trap(int trap, int specific)
  *
  * This function eventually calls send_enterprise_trap_vars.  If the
  * trap type is not set to SNMP_TRAP_ENTERPRISESPECIFIC the enterprise 
- * and enterprise_length paramater is set to the pre defined SYSTEM_MIB 
+ * and enterprise_length paramater is set to the pre defined NETSNMP_SYSTEM_MIB 
  * oid and length respectively.  If the trap type is set to 
  * SNMP_TRAP_ENTERPRISESPECIFIC the enterprise and enterprise_length 
- * parameters are set to the pre-defined NOTIFICATION_MIB oid and length 
+ * parameters are set to the pre-defined NETSNMP_NOTIFICATION_MIB oid and length 
  * respectively.
  *
  * @param vars is used to supply list of variable bindings to form an SNMPv2 
@@ -1041,13 +1071,12 @@ snmpd_parse_config_authtrap(const char *token, char *cptr)
     }
 }
 
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
 void
 snmpd_parse_config_trapsink(const char *token, char *cptr)
 {
     char            tmpbuf[1024];
     char           *sp, *cp, *pp = NULL;
-    int             sinkport;
     char            *st;
 
     if (!snmp_trapcommunity)
@@ -1056,17 +1085,9 @@ snmpd_parse_config_trapsink(const char *token, char *cptr)
     cp = strtok_r(NULL, " \t\n", &st);
     if (cp)
         pp = strtok_r(NULL, " \t\n", &st);
-    if (cp && pp) {
-        sinkport = atoi(pp);
-        if ((sinkport < 1) || (sinkport > 0xffff)) {
-            config_perror("trapsink port out of range");
-            sinkport = SNMP_TRAP_PORT;
-        }
-    } else {
-        sinkport = SNMP_TRAP_PORT;
-    }
-    if (create_v1_trap_session(sp, (u_short)sinkport,
-                               cp ? cp : snmp_trapcommunity) == 0) {
+    if (pp)
+	config_pwarn("The separate port argument to trapsink is deprecated");
+    if (create_v1_trap_session(sp, pp, cp ? cp : snmp_trapcommunity) == 0) {
         snprintf(tmpbuf, sizeof(tmpbuf), "cannot create trapsink: %s", cptr);
         tmpbuf[sizeof(tmpbuf)-1] = '\0';
         config_perror(tmpbuf);
@@ -1074,7 +1095,7 @@ snmpd_parse_config_trapsink(const char *token, char *cptr)
 }
 #endif
 
-#ifndef DISABLE_SNMPV2C
+#ifndef NETSNMP_DISABLE_SNMPV2C
 void
 snmpd_parse_config_trap2sink(const char *word, char *cptr)
 {
@@ -1089,17 +1110,9 @@ snmpd_parse_config_trap2sink(const char *word, char *cptr)
     cp = strtok_r(NULL, " \t\n", &st);
     if (cp)
         pp = strtok_r(NULL, " \t\n", &st);
-    if (cp && pp) {
-        sinkport = atoi(pp);
-        if ((sinkport < 1) || (sinkport > 0xffff)) {
-            config_perror("trapsink port out of range");
-            sinkport = SNMP_TRAP_PORT;
-        }
-    } else {
-        sinkport = SNMP_TRAP_PORT;
-    }
-    if (create_v2_trap_session(sp, (u_short)sinkport,
-                               cp ? cp : snmp_trapcommunity) == 0) {
+    if (pp)
+	config_pwarn("The separate port argument to trapsink2 is deprecated");
+    if (create_v2_trap_session(sp, pp, cp ? cp : snmp_trapcommunity) == 0) {
         snprintf(tmpbuf, sizeof(tmpbuf), "cannot create trap2sink: %s", cptr);
         tmpbuf[sizeof(tmpbuf)-1] = '\0';
         config_perror(tmpbuf);
@@ -1120,17 +1133,9 @@ snmpd_parse_config_informsink(const char *word, char *cptr)
     cp = strtok_r(NULL, " \t\n", &st);
     if (cp)
         pp = strtok_r(NULL, " \t\n", &st);
-    if (cp && pp) {
-        sinkport = atoi(pp);
-        if ((sinkport < 1) || (sinkport > 0xffff)) {
-            config_perror("trapsink port out of range");
-            sinkport = SNMP_TRAP_PORT;
-        }
-    } else {
-        sinkport = SNMP_TRAP_PORT;
-    }
-    if (create_v2_inform_session(sp, (u_short)sinkport,
-                                 cp ? cp : snmp_trapcommunity) == 0) {
+    if (pp)
+	config_pwarn("The separate port argument to informsink is deprecated");
+    if (create_v2_inform_session(sp, pp, cp ? cp : snmp_trapcommunity) == 0) {
         snprintf(tmpbuf, sizeof(tmpbuf), "cannot create informsink: %s", cptr);
         tmpbuf[sizeof(tmpbuf)-1] = '\0';
         config_perror(tmpbuf);
@@ -1171,6 +1176,7 @@ snmpd_parse_config_trapsess(const char *word, char *cptr)
     char           *argv[MAX_ARGS], *cp = cptr, tmp[SPRINT_MAX_LEN];
     int             argn, arg;
     netsnmp_session session, *ss;
+    size_t          len;
 
     /*
      * inform or trap?  default to trap 
@@ -1187,8 +1193,10 @@ snmpd_parse_config_trapsess(const char *word, char *cptr)
     }
 
     arg = snmp_parse_args(argn, argv, &session, "C:", trapOptProc);
-    ss = snmp_open(&session);
 
+    ss = snmp_add(&session,
+		  netsnmp_transport_open_client("snmptrap", session.peername),
+		  NULL, NULL);
     for (; argn > 0; argn--) {
         free(argv[argn - 1]);
     }
@@ -1200,19 +1208,31 @@ snmpd_parse_config_trapsess(const char *word, char *cptr)
         return;
     }
 
-#ifndef DISABLE_SNMPV1
+    /*
+     * If this is an SNMPv3 TRAP session, then the agent is
+     *   the authoritative engine, so set the engineID accordingly
+     */
+    if (ss->version == SNMP_VERSION_3 &&
+        traptype != SNMP_MSG_INFORM   &&
+        ss->securityEngineIDLen == 0) {
+            len = snmpv3_get_engineID( tmp, sizeof(tmp));
+            memdup(&ss->securityEngineID, tmp, len);
+            ss->securityEngineIDLen = len;
+    }
+
+#ifndef NETSNMP_DISABLE_SNMPV1
     if (ss->version == SNMP_VERSION_1) {
         add_trap_session(ss, SNMP_MSG_TRAP, 0, SNMP_VERSION_1);
     } else {
 #endif
         add_trap_session(ss, traptype, (traptype == SNMP_MSG_INFORM),
                          ss->version);
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
     }
 #endif
 }
 
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
 void
 snmpd_parse_config_trapcommunity(const char *word, char *cptr)
 {
