@@ -560,7 +560,7 @@ table_helper_handler(netsnmp_mib_handler *handler,
                             tbl_req_info->colnum,
                             tbl_req_info->number_indexes));
                 for (vb = tbl_req_info->indexes, count = 0;
-                     vb && count < tbl_info->number_indexes;
+                     vb && count < tbl_req_info->number_indexes;
                      count++, vb = vb->next_variable) {
                     out_len = 0;
                     if (sprint_realloc_by_type(&buf, &buf_len, &out_len, 1,
@@ -837,10 +837,7 @@ netsnmp_table_build_oid_from_index(netsnmp_handler_registration *reginfo,
     memcpy(&tmpoid[len], table_info->index_oid,
            table_info->index_oid_len * sizeof(oid));
     len += table_info->index_oid_len;
-    if (var->name && var->name != var->name_loc)
-        SNMP_FREE(var->name);
-    snmp_clone_mem((void **) &var->name, tmpoid, len * sizeof(oid));
-    var->name_length = len;
+    snmp_set_var_objid( var, tmpoid, len );
 
     return SNMPERR_SUCCESS;
 }
@@ -909,11 +906,21 @@ netsnmp_check_getnext_reply(netsnmp_request_info *request,
             /*
              * the new result must be better than the old 
              */
+#ifdef ONLY_WORKS_WITH_ONE_VARBIND
             if (!*outvar)
                 *outvar = snmp_clone_varbind(newvar);
 	    else
+                /* 
+                 * TODO: walk the full varbind list, setting
+                 *       *all* the values - not just the first.
+                 */
                 snmp_set_var_typed_value(*outvar, newvar->type,
 				newvar->val.string, newvar->val_len);
+#else  /* Interim replacement approach - less efficient, but it works! */
+            if (*outvar)
+                snmp_free_varbind(*outvar);
+            *outvar = snmp_clone_varbind(newvar);
+#endif
             snmp_set_var_objid(*outvar, myname, myname_len);
 
             return 1;
