@@ -13,14 +13,13 @@
 #include <net-snmp/agent/table_data.h>
 #include <net-snmp/agent/read_only.h>
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
-
 /** @defgroup table_data table_data
  *  Helps you implement a table with datamatted storage.
  *  @ingroup table
  *
+ *  This helper is obsolete.  If you are writing a new module, please
+ *  consider using the table_tdata helper instead.
+ *
  *  This helper helps you implement a table where all the indexes are
  *  expected to be stored within the agent itself and not in some
  *  external storage location.  It can be used to store a list of
@@ -33,7 +32,13 @@
  *  @{
  */
 
-/**
+/* ==================================
+ *
+ * Table Data API: Table maintenance
+ *
+ * ================================== */
+
+/*
  * generates the index portion of an table oid from a varlist.
  */
 void
@@ -42,6 +47,83 @@ netsnmp_table_data_generate_index_oid(netsnmp_table_row *row)
     build_oid(&row->index_oid, &row->index_oid_len, NULL, 0, row->indexes);
 }
 
+/** creates and returns a pointer to table data set */
+netsnmp_table_data *
+netsnmp_create_table_data(const char *name)
+{
+    netsnmp_table_data *table = SNMP_MALLOC_TYPEDEF(netsnmp_table_data);
+    if (name && table)
+        table->name = strdup(name);
+    return table;
+}
+
+/** creates and returns a pointer to table data set */
+netsnmp_table_row *
+netsnmp_create_table_data_row(void)
+{
+    netsnmp_table_row *row = SNMP_MALLOC_TYPEDEF(netsnmp_table_row);
+    return row;
+}
+
+/** clones a data row. DOES NOT CLONE THE CONTAINED DATA. */
+netsnmp_table_row *
+netsnmp_table_data_clone_row(netsnmp_table_row *row)
+{
+    netsnmp_table_row *newrow = NULL;
+    if (!row)
+        return NULL;
+
+    memdup((u_char **) & newrow, (u_char *) row,
+           sizeof(netsnmp_table_row));
+    if (!newrow)
+        return NULL;
+
+    if (row->indexes) {
+        newrow->indexes = snmp_clone_varbind(newrow->indexes);
+        if (!newrow->indexes) {
+            free (newrow);
+            return NULL;
+        }
+    }
+
+    if (row->index_oid) {
+        memdup((u_char **) & newrow->index_oid,
+               (u_char *) row->index_oid,
+               row->index_oid_len * sizeof(oid));
+        if (!newrow->index_oid) {
+            free (newrow);
+            return NULL;
+        }
+    }
+
+    return newrow;
+}
+
+/** deletes a row's memory.
+ *  returns the void data that it doesn't know how to delete. */
+void           *
+netsnmp_table_data_delete_row(netsnmp_table_row *row)
+{
+    void           *data;
+
+    if (!row)
+        return NULL;
+
+    /*
+     * free the memory we can 
+     */
+    if (row->indexes)
+        snmp_free_varbind(row->indexes);
+    SNMP_FREE(row->index_oid);
+    data = row->data;
+    free(row);
+
+    /*
+     * return the void * pointer 
+     */
+    return data;
+}
+
 /**
  * Adds a row of data to a given table (stored in proper lexographical order).
  *
@@ -146,6 +228,16 @@ netsnmp_table_data_add_row(netsnmp_table_data *table,
     return SNMPERR_SUCCESS;
 }
 
+/** swaps out origrow with newrow.  This does *not* delete/free anything! */
+NETSNMP_INLINE void
+netsnmp_table_data_replace_row(netsnmp_table_data *table,
+                               netsnmp_table_row *origrow,
+                               netsnmp_table_row *newrow)
+{
+    netsnmp_table_data_remove_row(table, origrow);
+    netsnmp_table_data_add_row(table, newrow);
+}
+
 /**
  * removes a row of data to a given table and returns it (no free's called)
  *
@@ -172,31 +264,6 @@ netsnmp_table_data_remove_row(netsnmp_table_data *table,
     return row;
 }
 
-/** deletes a row's memory.
- *  returns the void data that it doesn't know how to delete. */
-void           *
-netsnmp_table_data_delete_row(netsnmp_table_row *row)
-{
-    void           *data;
-
-    if (!row)
-        return NULL;
-
-    /*
-     * free the memory we can 
-     */
-    if (row->indexes)
-        snmp_free_varbind(row->indexes);
-    SNMP_FREE(row->index_oid);
-    data = row->data;
-    free(row);
-
-    /*
-     * return the void * pointer 
-     */
-    return data;
-}
-
 /**
  * removes and frees a row of data to a given table and returns the void *
  *
@@ -217,67 +284,92 @@ netsnmp_table_data_remove_and_delete_row(netsnmp_table_data *table,
     return netsnmp_table_data_delete_row(row);
 }
 
-/** swaps out origrow with newrow.  This does *not* delete/free anything! */
-NETSNMP_INLINE void
-netsnmp_table_data_replace_row(netsnmp_table_data *table,
-                               netsnmp_table_row *origrow,
-                               netsnmp_table_row *newrow)
+    /* =====================================
+     * Generic API - mostly renamed wrappers
+     * ===================================== */
+
+netsnmp_table_data *
+netsnmp_table_data_create_table(const char *name, long flags)
 {
-    netsnmp_table_data_remove_row(table, origrow);
-    netsnmp_table_data_add_row(table, newrow);
+    return netsnmp_create_table_data( name );
 }
 
-/** finds the data in "datalist" stored at "indexes" */
-netsnmp_table_row *
-netsnmp_table_data_get(netsnmp_table_data *table,
-                       netsnmp_variable_list * indexes)
+void
+netsnmp_table_data_delete_table( netsnmp_table_data *table )
 {
-    oid             searchfor[MAX_OID_LEN];
-    size_t          searchfor_len = MAX_OID_LEN;
+    netsnmp_table_row *row, *nextrow;
 
-    build_oid_noalloc(searchfor, MAX_OID_LEN, &searchfor_len, NULL, 0,
-                      indexes);
-    return netsnmp_table_data_get_from_oid(table, searchfor,
-                                           searchfor_len);
-}
-
-/** finds the data in "datalist" stored at the searchfor oid */
-netsnmp_table_row *
-netsnmp_table_data_get_from_oid(netsnmp_table_data *table,
-                                oid * searchfor, size_t searchfor_len)
-{
-    netsnmp_table_row *row;
     if (!table)
-        return NULL;
+        return;
 
-    for (row = table->first_row; row != NULL; row = row->next) {
-        if (row->index_oid &&
-            snmp_oid_compare(searchfor, searchfor_len,
-                             row->index_oid, row->index_oid_len) == 0)
-            return row;
+    for (row = table->first_row; row; row=nextrow) {
+        nextrow   = row->next;
+        row->next = NULL;
+        netsnmp_table_data_delete_row(row);
+        /* Can't delete table-specific entry memory */
     }
-    return NULL;
+    table->first_row = NULL;
+
+    if (table->name) {
+        SNMP_FREE(table->name);
+        table->name = NULL;
+    }
+    SNMP_FREE(table);
+    return;
 }
 
-/** returns the first row in the table */
 netsnmp_table_row *
-netsnmp_table_data_get_first_row(netsnmp_table_data *table)
+netsnmp_table_data_create_row( void* entry )
 {
-    if (!table)
-        return NULL;
-    return table->first_row;
+    netsnmp_table_row *row = SNMP_MALLOC_TYPEDEF(netsnmp_table_row);
+    if (row)
+        row->data = entry;
+    return row;
 }
 
-/** returns the next row in the table */
-netsnmp_table_row *
-netsnmp_table_data_get_next_row(netsnmp_table_data *table,
-                                netsnmp_table_row  *row)
+    /* netsnmp_table_data_clone_row() defined above */
+
+int
+netsnmp_table_data_copy_row( netsnmp_table_row  *old_row,
+                             netsnmp_table_row  *new_row )
 {
-    if (!row)
-        return NULL;
-    return row->next;
+    if (!old_row || !new_row)
+        return -1;
+
+    memcpy(new_row, old_row, sizeof(netsnmp_table_row));
+
+    if (old_row->indexes)
+        new_row->indexes = snmp_clone_varbind(old_row->indexes);
+    if (old_row->index_oid)
+        memdup((u_char **) & new_row->index_oid,
+               (u_char *)    old_row->index_oid,
+               old_row->index_oid_len * sizeof(oid));
+    /* XXX - Doesn't copy table-specific row structure */
+    return 0;
 }
 
+    /*
+     * netsnmp_table_data_delete_row()
+     * netsnmp_table_data_add_row()
+     * netsnmp_table_data_replace_row()
+     * netsnmp_table_data_remove_row()
+     *     all defined above
+     */
+
+void *
+netsnmp_table_data_remove_delete_row(netsnmp_table_data *table,
+                                     netsnmp_table_row *row)
+{
+    return netsnmp_table_data_remove_and_delete_row(table, row);
+}
+
+
+/* ==================================
+ *
+ * Table Data API: MIB maintenance
+ *
+ * ================================== */
+
 /** Creates a table_data handler and returns it */
 netsnmp_mib_handler *
 netsnmp_get_table_data_handler(netsnmp_table_data *table)
@@ -314,17 +406,15 @@ netsnmp_register_table_data(netsnmp_handler_registration *reginfo,
 /** registers a handler as a read-only data table
  *  If table_info != NULL, it registers it as a normal table too. */
 int
-netsnmp_register_read_only_table_data(netsnmp_handler_registration
-                                      *reginfo, netsnmp_table_data *table,
-                                      netsnmp_table_registration_info
-                                      *table_info)
+netsnmp_register_read_only_table_data(netsnmp_handler_registration *reginfo,
+                                      netsnmp_table_data *table,
+                                      netsnmp_table_registration_info *table_info)
 {
     netsnmp_inject_handler(reginfo, netsnmp_get_read_only_handler());
     return netsnmp_register_table_data(reginfo, table, table_info);
 }
 
-
-/**
+/*
  * The helper handler that takes care of passing a specific row of
  * data down to the lower handler(s).  It sets request->processed if
  * the request should not be handled.
@@ -335,7 +425,6 @@ netsnmp_table_data_helper_handler(netsnmp_mib_handler *handler,
                                   netsnmp_agent_request_info *reqinfo,
                                   netsnmp_request_info *requests)
 {
-
     netsnmp_table_data *table = (netsnmp_table_data *) handler->myvoid;
     netsnmp_request_info *request;
     int             valid_request = 0;
@@ -559,22 +648,33 @@ netsnmp_table_data_helper_handler(netsnmp_mib_handler *handler,
         return SNMP_ERR_NOERROR;
 }
 
-/** creates and returns a pointer to table data set */
+/** extracts the table being accessed passed from the table_data helper */
 netsnmp_table_data *
-netsnmp_create_table_data(const char *name)
+netsnmp_extract_table(netsnmp_request_info *request)
 {
-    netsnmp_table_data *table = SNMP_MALLOC_TYPEDEF(netsnmp_table_data);
-    if (name && table)
-        table->name = strdup(name);
-    return table;
+    return (netsnmp_table_data *)
+                netsnmp_request_get_list_data(request, TABLE_DATA_TABLE);
 }
 
-/** creates and returns a pointer to table data set */
+/** extracts the row being accessed passed from the table_data helper */
 netsnmp_table_row *
-netsnmp_create_table_data_row(void)
+netsnmp_extract_table_row(netsnmp_request_info *request)
 {
-    netsnmp_table_row *row = SNMP_MALLOC_TYPEDEF(netsnmp_table_row);
-    return row;
+    return (netsnmp_table_row *) netsnmp_request_get_list_data(request,
+                                                               TABLE_DATA_ROW);
+}
+
+/** extracts the data from the row being accessed passed from the
+ * table_data helper */
+void           *
+netsnmp_extract_table_row_data(netsnmp_request_info *request)
+{
+    netsnmp_table_row *row;
+    row = (netsnmp_table_row *) netsnmp_extract_table_row(request);
+    if (row)
+        return row->data;
+    else
+        return NULL;
 }
 
 /** inserts a newly created table_data row into a request */
@@ -642,36 +742,7 @@ netsnmp_insert_table_row(netsnmp_request_info *request,
     }
 }
 
-/** extracts the row being accessed passed from the table_data helper */
-netsnmp_table_row *
-netsnmp_extract_table_row(netsnmp_request_info *request)
-{
-    return (netsnmp_table_row *) netsnmp_request_get_list_data(request,
-                                                               TABLE_DATA_ROW);
-}
-
-/** extracts the table being accessed passed from the table_data helper */
-netsnmp_table_data *
-netsnmp_extract_table(netsnmp_request_info *request)
-{
-    return (netsnmp_table_data *) netsnmp_request_get_list_data(request,
-                                                               TABLE_DATA_TABLE);
-}
-
-/** extracts the data from the row being accessed passed from the
- * table_data helper */
-void           *
-netsnmp_extract_table_row_data(netsnmp_request_info *request)
-{
-    netsnmp_table_row *row;
-    row = (netsnmp_table_row *) netsnmp_extract_table_row(request);
-    if (row)
-        return row->data;
-    else
-        return NULL;
-}
-
-/** builds a result given a row, a varbind to set and the data */
+/* builds a result given a row, a varbind to set and the data */
 int
 netsnmp_table_data_build_result(netsnmp_handler_registration *reginfo,
                                 netsnmp_agent_request_info *reqinfo,
@@ -705,34 +776,63 @@ netsnmp_table_data_build_result(netsnmp_handler_registration *reginfo,
     return SNMPERR_SUCCESS;     /* WWWXXX: check for bounds */
 }
 
-/** clones a data row. DOES NOT CLONE THE CONTAINED DATA. */
+
+/* ==================================
+ *
+ * Table Data API: Row operations
+ *     (table-independent rows)
+ *
+ * ================================== */
+
+/** returns the first row in the table */
 netsnmp_table_row *
-netsnmp_table_data_clone_row(netsnmp_table_row *row)
+netsnmp_table_data_get_first_row(netsnmp_table_data *table)
+{
+    if (!table)
+        return NULL;
+    return table->first_row;
+}
+
+/** returns the next row in the table */
+netsnmp_table_row *
+netsnmp_table_data_get_next_row(netsnmp_table_data *table,
+                                netsnmp_table_row  *row)
 {
-    netsnmp_table_row *newrow = NULL;
     if (!row)
         return NULL;
+    return row->next;
+}
 
-    memdup((u_char **) & newrow, (u_char *) row,
-           sizeof(netsnmp_table_row));
-    if (!newrow)
+/** finds the data in "datalist" stored at "indexes" */
+netsnmp_table_row *
+netsnmp_table_data_get(netsnmp_table_data *table,
+                       netsnmp_variable_list * indexes)
+{
+    oid             searchfor[MAX_OID_LEN];
+    size_t          searchfor_len = MAX_OID_LEN;
+
+    build_oid_noalloc(searchfor, MAX_OID_LEN, &searchfor_len, NULL, 0,
+                      indexes);
+    return netsnmp_table_data_get_from_oid(table, searchfor,
+                                           searchfor_len);
+}
+
+/** finds the data in "datalist" stored at the searchfor oid */
+netsnmp_table_row *
+netsnmp_table_data_get_from_oid(netsnmp_table_data *table,
+                                oid * searchfor, size_t searchfor_len)
+{
+    netsnmp_table_row *row;
+    if (!table)
         return NULL;
 
-    if (row->indexes) {
-        newrow->indexes = snmp_clone_varbind(newrow->indexes);
-        if (!newrow->indexes)
-            return NULL;
+    for (row = table->first_row; row != NULL; row = row->next) {
+        if (row->index_oid &&
+            snmp_oid_compare(searchfor, searchfor_len,
+                             row->index_oid, row->index_oid_len) == 0)
+            return row;
     }
-
-    if (row->index_oid) {
-        memdup((u_char **) & newrow->index_oid,
-               (u_char *) row->index_oid,
-               row->index_oid_len * sizeof(oid));
-        if (!newrow->index_oid)
-            return NULL;
-    }
-
-    return newrow;
+    return NULL;
 }
 
 int
@@ -747,5 +847,163 @@ netsnmp_table_data_num_rows(netsnmp_table_data *table)
     }
     return i;
 }
+
+    /* =====================================
+     * Generic API - mostly renamed wrappers
+     * ===================================== */
+
+netsnmp_table_row *
+netsnmp_table_data_row_first(netsnmp_table_data *table)
+{
+    return netsnmp_table_data_get_first_row(table);
+}
+
+netsnmp_table_row *
+netsnmp_table_data_row_get(  netsnmp_table_data *table,
+                             netsnmp_table_row  *row)
+{
+    if (!table || !row)
+        return NULL;
+    return netsnmp_table_data_get_from_oid(table, row->index_oid,
+                                                  row->index_oid_len);
+}
+
+netsnmp_table_row *
+netsnmp_table_data_row_next( netsnmp_table_data *table,
+                             netsnmp_table_row  *row)
+{
+    return netsnmp_table_data_get_next_row(table, row);
+}
+
+netsnmp_table_row *
+netsnmp_table_data_row_get_byoid( netsnmp_table_data *table,
+                                  oid *instance, size_t len)
+{
+    return netsnmp_table_data_get_from_oid(table, instance, len);
+}
+
+netsnmp_table_row *
+netsnmp_table_data_row_next_byoid(netsnmp_table_data *table,
+                                  oid *instance, size_t len)
+{
+    netsnmp_table_row *row;
+
+    if (!table || !instance)
+        return NULL;
+    
+    for (row = table->first_row; row; row = row->next) {
+        if (snmp_oid_compare(row->index_oid,
+                             row->index_oid_len,
+                             instance, len) > 0)
+            return row;
+    }
+    return NULL;
+}
+
+netsnmp_table_row *
+netsnmp_table_data_row_get_byidx( netsnmp_table_data    *table,
+                                  netsnmp_variable_list *indexes)
+{
+    return netsnmp_table_data_get(table, indexes);
+}
+
+netsnmp_table_row *
+netsnmp_table_data_row_next_byidx(netsnmp_table_data    *table,
+                                  netsnmp_variable_list *indexes)
+{
+    oid    instance[MAX_OID_LEN];
+    size_t len    = MAX_OID_LEN;
+
+    if (!table || !indexes)
+        return NULL;
+
+    build_oid_noalloc(instance, MAX_OID_LEN, &len, NULL, 0, indexes);
+    return netsnmp_table_data_row_next_byoid(table, instance, len);
+}
+
+int
+netsnmp_table_data_row_count(netsnmp_table_data *table)
+{
+    return netsnmp_table_data_num_rows(table);
+}
+
+
+/* ==================================
+ *
+ * Table Data API: Row operations
+ *     (table-specific rows)
+ *
+ * ================================== */
+
+void *
+netsnmp_table_data_entry_first(netsnmp_table_data *table)
+{
+    netsnmp_table_row *row =
+        netsnmp_table_data_get_first_row(table);
+    return (row ? row->data : NULL );
+}
+
+void *
+netsnmp_table_data_entry_get(  netsnmp_table_data *table,
+                               netsnmp_table_row  *row)
+{
+    return (row ? row->data : NULL );
+}
+
+void *
+netsnmp_table_data_entry_next( netsnmp_table_data *table,
+                               netsnmp_table_row  *row)
+{
+    row =
+        netsnmp_table_data_row_next(table, row);
+    return (row ? row->data : NULL );
+}
+
+void *
+netsnmp_table_data_entry_get_byidx( netsnmp_table_data    *table,
+                                    netsnmp_variable_list *indexes)
+{
+    netsnmp_table_row *row =
+        netsnmp_table_data_row_get_byidx(table, indexes);
+    return (row ? row->data : NULL );
+}
+
+void *
+netsnmp_table_data_entry_next_byidx(netsnmp_table_data    *table,
+                                    netsnmp_variable_list *indexes)
+{
+    netsnmp_table_row *row =
+        netsnmp_table_data_row_next_byidx(table, indexes);
+    return (row ? row->data : NULL );
+}
+
+void *
+netsnmp_table_data_entry_get_byoid( netsnmp_table_data *table,
+                                    oid *instance, size_t len)
+{
+    netsnmp_table_row *row =
+        netsnmp_table_data_row_get_byoid(table, instance, len);
+    return (row ? row->data : NULL );
+}
+
+void *
+netsnmp_table_data_entry_next_byoid(netsnmp_table_data *table,
+                                    oid *instance, size_t len)
+{
+    netsnmp_table_row *row =
+        netsnmp_table_data_row_next_byoid(table, instance, len);
+    return (row ? row->data : NULL );
+}
+
+    /* =====================================
+     * Generic API - mostly renamed wrappers
+     * ===================================== */
+
+/* ==================================
+ *
+ * Table Data API: Index operations
+ *
+ * ================================== */
+
 /** @} 
  */
