@@ -36,9 +36,6 @@ typedef struct newrow_stash_s {
  *  Helps you implement a table with automatted storage.
  *  @ingroup table_data
  *
- *  This helper is obsolete.  If you are writing a new module, please
- *  consider using the table_dataset2 helper instead.
- *
  *  This handler helps you implement a table where all the data is
  *  expected to be stored within the agent itself and not in some
  *  external storage location.  It handles all MIB requests including
@@ -60,11 +57,11 @@ typedef struct newrow_stash_s {
 
 void
 netsnmp_init_table_dataset(void) {
-#ifndef DISABLE_MIB_LOADING
+#ifndef NETSNMP_DISABLE_MIB_LOADING
     register_app_config_handler("table",
                                 netsnmp_config_parse_table_set, NULL,
                                 "tableoid");
-#endif /* DISABLE_MIB_LOADING */
+#endif /* NETSNMP_DISABLE_MIB_LOADING */
     register_app_config_handler("add_row", netsnmp_config_parse_add_row,
                                 NULL, "table_name indexes... values...");
 }
@@ -563,7 +560,7 @@ netsnmp_table_data_set_helper_handler(netsnmp_mib_handler *handler,
             data = (netsnmp_table_data_set_storage *) row->data;
 
         if (!row || !table_info || !data) {
-            if (!MODE_IS_SET(reqinfo->mode)) {
+            if (!MODE_IS_SET(reqinfo->mode) || !table_info) {
                 netsnmp_set_request_error(reqinfo, request,
                                           SNMP_NOSUCHINSTANCE);
                 continue;
@@ -761,6 +758,7 @@ netsnmp_table_data_set_helper_handler(netsnmp_mib_handler *handler,
                                                       newrow, row);
                     netsnmp_table_dataset_delete_row(newrow);
                 }
+                newrow = NULL;
             }
             break;
 
@@ -768,10 +766,28 @@ netsnmp_table_data_set_helper_handler(netsnmp_mib_handler *handler,
             if (newrowstash->state != STATE_COMMIT) {
                 newrowstash->state = STATE_COMMIT;
                 if (!newrowstash->created) {
+		    netsnmp_request_info       *req;
                     netsnmp_table_dataset_delete_row(row);
+
+		    /* Walk the request list to update the reference to the old row w/ th new one */
+    		    for (req = requests; req; req=req->next) {
+        
+		    	/*
+         			* For requests that have the old row values,
+         			* so add the newly-created row information.
+         	    	*/
+        	    	if ((netsnmp_table_row *) netsnmp_extract_table_row(req) == row) {
+	    			netsnmp_request_remove_list_data(req, TABLE_DATA_ROW);
+            			netsnmp_request_add_list_data(req,
+                		    netsnmp_create_data_list(TABLE_DATA_ROW, newrow, NULL));
+        	    	}
+    		    }
+
+		    row = NULL;
                 }
                 if (newrowstash->deleted) {
                     netsnmp_table_dataset_remove_and_delete_row(datatable, newrow);
+                    newrow = NULL;
                 }
             }
             break;
@@ -780,6 +796,7 @@ netsnmp_table_data_set_helper_handler(netsnmp_mib_handler *handler,
             if (newrowstash && newrowstash->state != STATE_FREE) {
                 newrowstash->state = STATE_FREE;
                 netsnmp_table_dataset_delete_row(newrow);
+		newrow = NULL;
             }
             break;
         }
@@ -844,7 +861,7 @@ netsnmp_register_auto_data_table(netsnmp_table_data_set *table_set,
     netsnmp_add_list_data(&auto_tables, netsnmp_create_data_list(registration_name, tables, NULL));     /* XXX */
 }
 
-#ifndef DISABLE_MIB_LOADING
+#ifndef NETSNMP_DISABLE_MIB_LOADING
 static void
 _table_set_add_indexes(netsnmp_table_data_set *table_set, struct tree *tp)
 {
@@ -853,6 +870,7 @@ _table_set_add_indexes(netsnmp_table_data_set *table_set, struct tree *tp)
     struct index_list *index;
     struct tree     *indexnode;
     u_char          type;
+    int             fixed_len = 0;
     
     /*
      * loop through indexes and add types 
@@ -877,17 +895,34 @@ _table_set_add_indexes(netsnmp_table_data_set *table_set, struct tree *tp)
          * if implied, mark it as such. also mark fixed length
          * octet strings as implied (ie no length prefix) as well.
          * */
-        if ((index->isimplied) ||
-            ((TYPE_OCTETSTR == indexnode->type) &&  /* octet str */
-             (NULL != indexnode->ranges) &&         /* & has range */
-             (NULL == indexnode->ranges->next) &&   /*   but only one */
-             (indexnode->ranges->high ==            /*   & high==low */
-              indexnode->ranges->low)))
+        if ((TYPE_OCTETSTR == indexnode->type) &&  /* octet str */
+            (NULL != indexnode->ranges) &&         /* & has range */
+            (NULL == indexnode->ranges->next) &&   /*   but only one */
+            (indexnode->ranges->high ==            /*   & high==low */
+             indexnode->ranges->low)) {
+            type |= ASN_PRIVATE;
+            fixed_len = indexnode->ranges->high;
+        }
+        else if (index->isimplied)
             type |= ASN_PRIVATE;
         
         DEBUGMSGTL(("table_set_add_table",
                     "adding default index of type %d\n", type));
         netsnmp_table_dataset_add_index(table_set, type);
+
+        /*
+         * hack alert: for fixed lenght strings, save the
+         * lenght for use during oid parsing.
+         */
+        if (fixed_len) {
+            /*
+             * find last (just added) index
+             */
+            netsnmp_variable_list *var =  table_set->table->indexes_template;
+            while (NULL != var->next_variable)
+                var = var->next_variable;
+            var->val_len = fixed_len;
+        }
     }
 }
 /** @internal */
@@ -955,6 +990,7 @@ netsnmp_config_parse_table_set(const char *token, char *line)
         if (!snmp_parse_oid(tp->augments, name, &name_length)) {
             config_pwarn("I can't parse the augment tabel name");
             snmp_log(LOG_WARNING, "  can't parse %s\n", tp->augments);
+            SNMP_FREE (table_set);
             return;
         }
         if(NULL == (tp2 = get_tree(name, name_length, get_tree_head()))) {
@@ -962,6 +998,7 @@ netsnmp_config_parse_table_set(const char *token, char *line)
                          "I can't find mib information about augment table");
             snmp_log(LOG_WARNING, "  table %s not found in tree\n",
                      tp->augments);
+            SNMP_FREE (table_set);
             return;
         }
         _table_set_add_indexes(table_set, tp2);
@@ -978,6 +1015,7 @@ netsnmp_config_parse_table_set(const char *token, char *line)
         type = mib_to_asn_type(tp->type);
         if (type == (u_char) - 1) {
             config_pwarn("unknown column type");
+	    SNMP_FREE (table_set);
             return;             /* xxx mem leak */
         }
 
@@ -1020,7 +1058,7 @@ netsnmp_config_parse_table_set(const char *token, char *line)
 
     netsnmp_register_auto_data_table(table_set, NULL);
 }
-#endif /* DISABLE_MIB_LOADING */
+#endif /* NETSNMP_DISABLE_MIB_LOADING */
 
 /** @internal */
 void
@@ -1053,6 +1091,7 @@ netsnmp_config_parse_add_row(const char *token, char *line)
          vb = vb->next_variable) {
         if (!line) {
             config_pwarn("missing an index value");
+            SNMP_FREE (row);
             return;
         }
 
@@ -1072,6 +1111,7 @@ netsnmp_config_parse_add_row(const char *token, char *line)
                          "All columns must be specified.");
             snmp_log(LOG_WARNING,"  can't find value for column %d\n",
                      dr->column - 1);
+            SNMP_FREE (row);
             return;
         }
 
@@ -1234,7 +1274,7 @@ netsnmp_set_row_column(netsnmp_table_row *row, unsigned int column,
  * ================================== */
 
 /** adds an index to the table.  Call this repeatly for each index. */
-NETSNMP_INLINE void
+void
 netsnmp_table_dataset_add_index(netsnmp_table_data_set *table, u_char type)
 {
     if (!table)
@@ -1265,13 +1305,12 @@ netsnmp_table_set_add_indexes(va_alist)
     tset = va_arg(debugargs, netsnmp_table_data_set *);
 #endif
 
-    while ((type = va_arg(debugargs, int)) != 0) {
-        netsnmp_table_dataset_add_index(tset, (u_char)type);
-    }
+    if (tset)
+        while ((type = va_arg(debugargs, int)) != 0)
+            netsnmp_table_data_add_index(tset->table, type);
 
     va_end(debugargs);
 }
 
 /** @} 
  */
-
