@@ -386,12 +386,13 @@ netsnmp_table_iterator_helper_handler(netsnmp_mib_handler *handler,
     netsnmp_table_request_info *table_info = NULL;
     oid             coloid[MAX_OID_LEN];
     size_t          coloid_len;
-    int             ret;
+    int             ret = SNMP_ERR_NOERROR;
     static oid      myname[MAX_OID_LEN];
     size_t          myname_len;
     int             oldmode = 0;
     netsnmp_iterator_info *iinfo;
     int notdone;
+    int hintok = 0;
     netsnmp_request_info *request, *reqtmp = NULL;
     netsnmp_variable_list *index_search = NULL;
     netsnmp_variable_list *free_this_index_search = NULL;
@@ -493,6 +494,7 @@ netsnmp_table_iterator_helper_handler(netsnmp_mib_handler *handler,
           if (!request->processed)
             request_count++;
         notdone = 1;
+        hintok = 1;
         while(notdone) {
             notdone = 0;
 
@@ -534,7 +536,7 @@ netsnmp_table_iterator_helper_handler(netsnmp_mib_handler *handler,
             }
 
             /* if sorted, pass in a hint */
-            if (iinfo->flags & NETSNMP_ITERATOR_FLAG_SORTED) {
+            if (hintok && (iinfo->flags & NETSNMP_ITERATOR_FLAG_SORTED)) {
                 callback_loop_context = table_info;
             }
             index_search =
@@ -710,7 +712,8 @@ netsnmp_table_iterator_helper_handler(netsnmp_mib_handler *handler,
                             break;
                         } else {
                           table_info->colnum = nc;
-                            notdone = 1;
+                          hintok = 0;
+                          notdone = 1;
                         }
                     }
                 }
@@ -804,7 +807,7 @@ netsnmp_table_iterator_helper_handler(netsnmp_mib_handler *handler,
     if (free_this_index_search)
         snmp_free_varbind(free_this_index_search);
 
-    return SNMP_ERR_NOERROR;
+    return ret;
 }
 
 /* ==================================
