@@ -418,7 +418,10 @@ extend_parse_config(const char *token, char *cptr)
     cptr = copy_nword(cptr, exec_name,    sizeof(exec_name));
     if ( *exec_name == '.' ) {
         oid_len = MAX_OID_LEN - 2;
-        read_objid( exec_name, oid_buf, &oid_len );
+        if (0 == read_objid( exec_name, oid_buf, &oid_len )) {
+            config_perror("ERROR: Unrecognised OID" );
+            return;
+        }
         cptr = copy_nword(cptr, exec_name,    sizeof(exec_name));
         if (!strcmp( token, "sh"   ) ||
             !strcmp( token, "exec" )) {
@@ -450,6 +453,9 @@ extend_parse_config(const char *token, char *cptr)
         extension->command  = strdup( exec_command );
         if (cptr)
             extension->args = strdup( cptr );
+    } else {
+        snmp_log(LOG_ERR, "Failed to register extend entry '%s' - possibly duplicate name.\n", exec_name );
+        return;
     }
 
 #ifndef USING_UCD_SNMP_EXTENSIBLE_MODULE
@@ -1236,7 +1242,7 @@ _extend_find_entry( netsnmp_request_info       *request,
             snmp_set_var_value( table_info->indexes,
                                 eptr->token, strlen(eptr->token));
             snmp_set_var_value( table_info->indexes->next_variable,
-                                &line_idx, sizeof(line_idx));
+                                (const u_char*)&line_idx, sizeof(line_idx));
         }
         return eptr;  /* Finally, signal success */
     }
