@@ -1105,13 +1105,14 @@ agentx_parse_oid(u_char * data, size_t * length, int *inc,
 {
     u_int           n_subid;
     u_int           prefix;
+    u_int           tmp_oid_len;
     int             i;
     int             int_offset;
     u_int          *int_ptr = (u_int *)oid_buf;
     u_char         *buf_ptr = data;
 
     if (*length < 4) {
-        DEBUGMSGTL(("agentx", "Incomplete Object ID"));
+        DEBUGMSGTL(("agentx", "Incomplete Object ID\n"));
         return NULL;
     }
 
@@ -1144,10 +1145,7 @@ agentx_parse_oid(u_char * data, size_t * length, int *inc,
         /*
          * Null OID 
          */
-        *int_ptr = 0;
-        int_ptr++;
-        *int_ptr = 0;
-        int_ptr++;
+        memset(int_ptr, 0, 2 * sizeof(oid));
         *oid_len = 2;
         DEBUGPRINTINDENT("dumpv_recv");
         DEBUGMSG(("dumpv_recv", "OID: NULL (0.0)\n"));
@@ -1155,6 +1153,14 @@ agentx_parse_oid(u_char * data, size_t * length, int *inc,
         return buf_ptr;
     }
 
+    /*
+     * Check that the expanded OID will fit in the buffer provided
+     */
+    tmp_oid_len = (prefix ? n_subid + 5 : n_subid);
+    if (*oid_len < tmp_oid_len) {
+        DEBUGMSGTL(("agentx", "Oversized Object ID\n"));
+        return NULL;
+    }
 
 #ifdef WORDS_BIGENDIAN
 # define endianoff 1
@@ -1162,7 +1168,7 @@ agentx_parse_oid(u_char * data, size_t * length, int *inc,
 # define endianoff 0
 #endif
     if (*length < 4 * n_subid) {
-        DEBUGMSGTL(("agentx", "Incomplete Object ID"));
+        DEBUGMSGTL(("agentx", "Incomplete Object ID\n"));
         return NULL;
     }
 
@@ -1199,7 +1205,7 @@ agentx_parse_oid(u_char * data, size_t * length, int *inc,
         *length -= 4;
     }
 
-    *oid_len = (prefix ? n_subid + 5 : n_subid);
+    *oid_len = tmp_oid_len;
 
     DEBUGINDENTLESS();
     DEBUGPRINTINDENT("dumpv_recv");
@@ -1220,19 +1226,19 @@ agentx_parse_string(u_char * data, size_t * length,
     u_int           len;
 
     if (*length < 4) {
-        DEBUGMSGTL(("agentx", "Incomplete string (too short: %d)",
+        DEBUGMSGTL(("agentx", "Incomplete string (too short: %d)\n",
                     *length));
         return NULL;
     }
 
     len = agentx_parse_int(data, network_byte_order);
     if (*length < len + 4) {
-        DEBUGMSGTL(("agentx", "Incomplete string (still too short: %d)",
+        DEBUGMSGTL(("agentx", "Incomplete string (still too short: %d)\n",
                     *length));
         return NULL;
     }
     if (len > *str_len) {
-        DEBUGMSGTL(("agentx", "String too long (too long)"));
+        DEBUGMSGTL(("agentx", "String too long (too long)\n"));
         return NULL;
     }
     memmove(string, data + 4, len);
@@ -1402,7 +1408,7 @@ agentx_parse_varbind(u_char * data, size_t * length, int *type,
 
         memcpy(data_buf, &tmp64, sizeof(tmp64));
         *data_len = sizeof(tmp64);
-	bufp += 2 * sizeof(long);
+	bufp    += 8;
 	*length -= 8;
         break;
 
@@ -1532,10 +1538,10 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
              size_t len)
 {
     register u_char *bufp = data;
-    u_char          buffer[BUFSIZ];
+    u_char          buffer[SNMP_MAX_MSG_SIZE];
     u_char         *prefix_ptr;
     oid             oid_buffer[MAX_OID_LEN], end_oid_buf[MAX_OID_LEN];
-    size_t          buf_len = BUFSIZ;
+    size_t          buf_len = sizeof(buffer);
     size_t          oid_buf_len = MAX_OID_LEN;
     size_t          end_oid_buf_len = MAX_OID_LEN;
 
@@ -1591,8 +1597,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
     if (pdu->flags & AGENTX_MSG_FLAG_NON_DEFAULT_CONTEXT) {
         DEBUGDUMPHEADER("recv", "Context");
         bufp = agentx_parse_string(bufp, length, buffer, &buf_len,
-                                   pdu->
-                                   flags &
+                                   pdu->flags &
                                    AGENTX_FLAGS_NETWORK_BYTE_ORDER);
         DEBUGINDENTLESS();
         if (bufp == NULL)
@@ -1601,7 +1606,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
         pdu->community_len = buf_len;
         snmp_clone_mem((void **) &pdu->community,
                        (void *) buffer, (unsigned) buf_len);
-        buf_len = BUFSIZ;
+        buf_len = sizeof(buffer);
     }
 
     DEBUGDUMPHEADER("recv", "PDU");
@@ -1618,8 +1623,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
         DEBUGDUMPHEADER("recv", "Subagent OID");
         bufp = agentx_parse_oid(bufp, length, NULL,
                                 oid_buffer, &oid_buf_len,
-                                pdu->
-                                flags & AGENTX_FLAGS_NETWORK_BYTE_ORDER);
+                                pdu->flags & AGENTX_FLAGS_NETWORK_BYTE_ORDER);
         DEBUGINDENTLESS();
         if (bufp == NULL) {
             DEBUGINDENTLESS();
@@ -1627,8 +1631,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
         }
         DEBUGDUMPHEADER("recv", "Subagent Description");
         bufp = agentx_parse_string(bufp, length, buffer, &buf_len,
-                                   pdu->
-                                   flags &
+                                   pdu->flags &
                                    AGENTX_FLAGS_NETWORK_BYTE_ORDER);
         DEBUGINDENTLESS();
         if (bufp == NULL) {
@@ -1639,7 +1642,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
                               ASN_OCTET_STR, buffer, buf_len);
 
         oid_buf_len = MAX_OID_LEN;
-        buf_len = BUFSIZ;
+        buf_len = sizeof(buffer);
         break;
 
     case AGENTX_MSG_CLOSE:
@@ -1674,8 +1677,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
         DEBUGDUMPHEADER("recv", "Registration OID");
         bufp = agentx_parse_oid(bufp, length, NULL,
                                 oid_buffer, &oid_buf_len,
-                                pdu->
-                                flags & AGENTX_FLAGS_NETWORK_BYTE_ORDER);
+                                pdu->flags & AGENTX_FLAGS_NETWORK_BYTE_ORDER);
         DEBUGINDENTLESS();
         if (bufp == NULL) {
             DEBUGINDENTLESS();
@@ -1683,9 +1685,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
         }
 
         if (pdu->range_subid) {
-            range_bound = agentx_parse_int(bufp,
-                                           pdu->
-                                           flags &
+            range_bound = agentx_parse_int(bufp, pdu->flags &
                                            AGENTX_FLAGS_NETWORK_BYTE_ORDER);
             bufp += 4;
             *length -= 4;
@@ -1709,15 +1709,11 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
 
     case AGENTX_MSG_GETBULK:
         DEBUGDUMPHEADER("recv", "Non-repeaters");
-        pdu->non_repeaters = agentx_parse_short(bufp,
-                                                pdu->
-                                                flags &
+        pdu->non_repeaters = agentx_parse_short(bufp, pdu->flags &
                                                 AGENTX_FLAGS_NETWORK_BYTE_ORDER);
         DEBUGINDENTLESS();
         DEBUGDUMPHEADER("recv", "Max-repeaters");
-        pdu->max_repetitions = agentx_parse_short(bufp + 2,
-                                                  pdu->
-                                                  flags &
+        pdu->max_repetitions = agentx_parse_short(bufp + 2, pdu->flags &
                                                   AGENTX_FLAGS_NETWORK_BYTE_ORDER);
         DEBUGINDENTLESS();
         bufp += 4;
@@ -1737,8 +1733,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
         while (*length > 0) {
             bufp = agentx_parse_oid(bufp, length, &inc,
                                     oid_buffer, &oid_buf_len,
-                                    pdu->
-                                    flags &
+                                    pdu->flags &
                                     AGENTX_FLAGS_NETWORK_BYTE_ORDER);
             if (bufp == NULL) {
                 DEBUGINDENTLESS();
@@ -1747,8 +1742,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
             }
             bufp = agentx_parse_oid(bufp, length, NULL,
                                     end_oid_buf, &end_oid_buf_len,
-                                    pdu->
-                                    flags &
+                                    pdu->flags &
                                     AGENTX_FLAGS_NETWORK_BYTE_ORDER);
             if (bufp == NULL) {
                 DEBUGINDENTLESS();
@@ -1786,21 +1780,16 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
         /*
          * sysUpTime 
          */
-        pdu->time = agentx_parse_int(bufp,
-                                     pdu->
-                                     flags &
+        pdu->time = agentx_parse_int(bufp, pdu->flags &
                                      AGENTX_FLAGS_NETWORK_BYTE_ORDER);
         bufp += 4;
         *length -= 4;
 
-        pdu->errstat = agentx_parse_short(bufp,
-                                          pdu->
-                                          flags &
+        pdu->errstat = agentx_parse_short(bufp, pdu->flags &
                                           AGENTX_FLAGS_NETWORK_BYTE_ORDER);
         pdu->errindex =
             agentx_parse_short(bufp + 2,
-                               pdu->
-                               flags & AGENTX_FLAGS_NETWORK_BYTE_ORDER);
+                               pdu->flags & AGENTX_FLAGS_NETWORK_BYTE_ORDER);
         bufp += 4;
         *length -= 4;
         /*
@@ -1822,8 +1811,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
             bufp = agentx_parse_varbind(bufp, length, &type,
                                         oid_buffer, &oid_buf_len,
                                         buffer, &buf_len,
-                                        pdu->
-                                        flags &
+                                        pdu->flags &
                                         AGENTX_FLAGS_NETWORK_BYTE_ORDER);
             if (bufp == NULL) {
                 DEBUGINDENTLESS();
@@ -1834,7 +1822,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
                                   (u_char) type, buffer, buf_len);
 
             oid_buf_len = MAX_OID_LEN;
-            buf_len = BUFSIZ;
+            buf_len = sizeof(buffer);
         }
         DEBUGINDENTLESS();
         break;
@@ -1856,13 +1844,11 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
          */
         bufp = agentx_parse_oid(bufp, length, NULL,
                                 oid_buffer, &oid_buf_len,
-                                pdu->
-                                flags & AGENTX_FLAGS_NETWORK_BYTE_ORDER);
+                                pdu->flags & AGENTX_FLAGS_NETWORK_BYTE_ORDER);
         if (bufp == NULL)
             return SNMPERR_ASN_PARSE_ERR;
         bufp = agentx_parse_string(bufp, length, buffer, &buf_len,
-                                   pdu->
-                                   flags &
+                                   pdu->flags &
                                    AGENTX_FLAGS_NETWORK_BYTE_ORDER);
         if (bufp == NULL)
             return SNMPERR_ASN_PARSE_ERR;
@@ -1870,7 +1856,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
                               ASN_OCTET_STR, buffer, buf_len);
 
         oid_buf_len = MAX_OID_LEN;
-        buf_len = BUFSIZ;
+        buf_len = sizeof(buffer);
         break;
 
     case AGENTX_MSG_REMOVE_AGENT_CAPS:
@@ -1879,8 +1865,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
          */
         bufp = agentx_parse_oid(bufp, length, NULL,
                                 oid_buffer, &oid_buf_len,
-                                pdu->
-                                flags & AGENTX_FLAGS_NETWORK_BYTE_ORDER);
+                                pdu->flags & AGENTX_FLAGS_NETWORK_BYTE_ORDER);
         if (bufp == NULL)
             return SNMPERR_ASN_PARSE_ERR;
         snmp_add_null_var(pdu, oid_buffer, oid_buf_len);
@@ -1923,7 +1908,7 @@ testit(netsnmp_pdu *pdu1)
      */
     len1 = BUFSIZ;
     if (agentx_build(&sess, pdu1, packet1, &len1) < 0) {
-        DEBUGMSGTL(("agentx", "First build failed"));
+        DEBUGMSGTL(("agentx", "First build failed\n"));
         exit(1);
     }
 
