@@ -6,6 +6,7 @@
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
+#include <net-snmp/agent/agent_callbacks.h>
 #include "utilities/iquery.h"
 #include "disman/event/mteObjects.h"
 #include "disman/event/mteTrigger.h"
@@ -56,6 +57,9 @@ init_mteTriggerConf(void)
      */
     snmp_register_callback(SNMP_CALLBACK_LIBRARY, SNMP_CALLBACK_STORE_DATA,
                            store_mteTTable, NULL);
+    snmp_register_callback(SNMP_CALLBACK_APPLICATION,
+                           SNMPD_CALLBACK_PRE_UPDATE_CONFIG,
+                           clear_mteTTable, NULL);
 }
 
 /* ==============================
@@ -92,6 +96,7 @@ _find_mteTrigger_entry( char *owner, char *tname )
     if (!row)
         return NULL;
     
+    /* return (struct mteTrigger *)netsnmp_tdata_row_entry( row ); */
     return (struct mteTrigger *)row->data;
 }
 
@@ -364,14 +369,24 @@ parse_mteMonitor(const char *token, char *line)
                 idx++;
                 cp     = copy_nword(cp, buf, SPRINT_MAX_LEN);
                 object = mteObjects_addOID( "snmpd.conf", tname, idx, buf, 0 );
-                idx    = object->mteOIndex;
+                if (!object) {
+                    snmp_log(LOG_ERR, "Unknown payload OID: %s\n", buf);
+                    config_perror("Unknown payload OID");
+                    mteObjects_removeEntries( "snmpd.conf", tname );
+                } else
+                    idx = object->mteOIndex;
                 break;
     
             case 'o':   /*  object  */
                 idx++;
                 cp     = copy_nword(cp, buf, SPRINT_MAX_LEN);
                 object = mteObjects_addOID( "snmpd.conf", tname, idx, buf, 1 );
-                idx    = object->mteOIndex;
+                if (!object) {
+                    snmp_log(LOG_ERR, "Unknown payload OID: %s\n", buf);
+                    config_perror("Unknown payload OID");
+                    mteObjects_removeEntries( "snmpd.conf", tname );
+                } else
+                    idx = object->mteOIndex;
                 break;
     
             case 'r':   /*  repeat frequency */
@@ -487,7 +502,6 @@ parse_mteMonitor(const char *token, char *line)
                     memcpy(oid_name_buf, buf, SPRINT_MAX_LEN);
                     memset(         buf,   0, SPRINT_MAX_LEN);
                     cp  = copy_nword(cp, buf, SPRINT_MAX_LEN);
-                    if ( buf[0] != '-' )
                         value = strtol(buf, NULL, 0);
     
                     /*
@@ -497,7 +511,7 @@ parse_mteMonitor(const char *token, char *line)
                     memcpy( buf, cp, strlen(cp));
                     cp = NULL;  /* To terminate the processing loop */
                     DEBUGMSGTL(("disman:event:conf", "%s: Thresh (%s, %d, %s)\n",
-                                              tname, oid_name_buf, op, buf));
+                                              tname, oid_name_buf, value, buf));
                     break;
                 }
             }
@@ -619,7 +633,6 @@ parse_mteMonitor(const char *token, char *line)
         }
         break;
     case MTE_TRIGGER_THRESHOLD:
-        if ( buf[0] != '-' ) {
             entry->mteTThFallValue  = value;
             value = strtol(buf, NULL, 0);
             entry->mteTThRiseValue  = value;
@@ -664,10 +677,7 @@ parse_mteMonitor(const char *token, char *line)
                 memcpy(entry->mteTThFallEvent,  "_mteTriggerFalling", 18);
             }
             cp = skip_token(buf);   /* skip riseThreshold value */
-        } else {
-            /* Skip absolute threshold placeholders */
-            cp = skip_token(buf);
-        }
+
         /*
          * Parse and set (optional) Delta thresholds & events
          */
@@ -728,10 +738,11 @@ parse_linkUpDown_traps(const char *token, char *line)
     if (strncmp( line, "yes", 3) == 0) {
         DEBUGMSGTL(("disman:event:conf", "Registering linkUpDown traps\n"));
 
+        /* ifOperStatus */
         parse_mteMonitor("monitor",
-            "-r 60 -S -e _linkUp   \"linkUp\"   ifOperStatus != 2");
+            "-r 60 -S -e _linkUp   \"linkUp\"   .1.3.6.1.2.1.2.2.1.8 != 2");
         parse_mteMonitor("monitor",
-            "-r 60 -S -e _linkDown \"linkDown\" ifOperStatus == 2");
+            "-r 60 -S -e _linkDown \"linkDown\" .1.3.6.1.2.1.2.2.1.8 == 2");
     }
 }
 
@@ -753,7 +764,7 @@ parse_default_mteMonitors(const char *token, char *line)
         parse_mteMonitor("monitor",
             "-o laNames -o laErrMessage   \"laTable\"       laErrorFlag  != 0");
         parse_mteMonitor("monitor",
-            "-o fileName -o fileErrorMsg  \"fileTable\" fileErrorFlag    != 0");
+            "-o fileName -o fileErrorMsg  \"fileTable\"    fileErrorFlag != 0");
         parse_mteMonitor("monitor",
             "-o snmperrErrMessage         \"snmperrs\"  snmperrErrorFlag != 0");
     }
@@ -1451,3 +1462,23 @@ store_mteTTable(int majorID, int minorID, void *serverarg, void *clientarg)
     DEBUGMSGTL(("disman:event:conf", "  done.\n"));
     return SNMPERR_SUCCESS;
 }
+
+int
+clear_mteTTable(int majorID, int minorID, void *serverarg, void *clientarg)
+{
+    netsnmp_tdata_row *row;
+   
+    while (( row = netsnmp_tdata_row_first( trigger_table_data ))) {
+        struct mteTrigger *entry = (struct mteTrigger *)
+            netsnmp_tdata_remove_and_delete_row(trigger_table_data, row);
+        if (entry) {
+            /* Remove from the callbacks list and disable triggers */
+            snmp_unregister_callback( SNMP_CALLBACK_LIBRARY,
+                                      SNMP_CALLBACK_POST_READ_CONFIG,
+                                      _mteTrigger_callback_enable, entry, 0 ); 
+            mteTrigger_disable( entry );
+            SNMP_FREE(entry);
+        }
+    }
+    return SNMPERR_SUCCESS;
+}
