@@ -54,10 +54,10 @@ void init_cpu_kstat( void ) {
                     state_begin = ks_data->value.i32;
                 } else if ( strcmp( ks_data->name, "cpu_type" ) == 0 ) {
                     strncpy( ctype, ks_data->value.c, sizeof(ctype));
-                    state[sizeof(ctype)-1] = '\0';
+                    ctype[sizeof(ctype)-1] = '\0';
                 } else if ( strcmp( ks_data->name, "fpu_type" ) == 0 ) {
                     strncpy( ftype, ks_data->value.c, sizeof(ftype));
-                    state[sizeof(ftype)-1] = '\0';
+                    ftype[sizeof(ftype)-1] = '\0';
                 } else if ( strcmp( ks_data->name, "clock_MHz" ) == 0 ) {
                     clock = ks_data->value.i32;
                 }
@@ -103,6 +103,8 @@ int netsnmp_cpu_arch_load( netsnmp_cache *cache, void *magic ) {
         if (strcmp(ksp->ks_module, "cpu_stat") == 0) {
             i    = ksp->ks_instance;
             cpu2 = netsnmp_cpu_get_byIdx( i, 0 );
+            if ( !cpu2 )  
+                break;   /* or continue ? */    /* Skip new CPUs */
             if ((ksp->ks_type != KSTAT_TYPE_RAW) ||
                 (ksp->ks_data_size != sizeof(cs))||
                 (kstat_read(kstat_fd, ksp, &cs) == -1)) {
