@@ -618,7 +618,7 @@ Init_HR_FileSys(void)
     fp = fopen(ETC_MNTTAB, "r");
     if (!fp) {
       char tmpbuf[STRINGMAX];
-      snprintf( tmpbuf, sizeof(tmpbuf), "Can't open %s\n", ETC_MNTTAB );
+      snprintf( tmpbuf, sizeof(tmpbuf), "Can't open mnttab %s\n", ETC_MNTTAB );
       config_perror(tmpbuf);
     }
 #endif
