@@ -112,7 +112,7 @@
 #endif                          /* if HAVE_SYS_SYSCTL_H */
 #endif                          /* ifndef dynix */
 
-#if (defined(aix4) || defined(aix5)) && HAVE_LIBPERFSTAT_H
+#if (defined(aix4) || defined(aix5) || defined(aix6)) && HAVE_LIBPERFSTAT_H
 #include <libperfstat.h>
 #endif
 
@@ -524,6 +524,7 @@ really_try_next:
 
         store_idx = name[ HRSTORE_ENTRY_NAME_LENGTH ];
         if (HRFS_entry &&
+	    store_idx > NETSNMP_MEM_TYPE_MAX &&
             netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID,
                                    NETSNMP_DS_AGENT_SKIPNFSINHOSTRESOURCES) &&
             Check_HR_FileSys_NFS())
@@ -556,7 +557,11 @@ really_try_next:
         if (store_idx > NETSNMP_MEM_TYPE_MAX)
             if (storageUseNFS && Check_HR_FileSys_NFS())
                 storage_type_id[storage_type_len - 1] = 10;     /* Network Disk */
-#if HAVE_HASMNTOPT
+#if HAVE_HASMNTOPT && !(defined(aix4) || defined(aix5) || defined(aix6))
+            /* 
+             * hasmntopt takes "const struct mntent*", but HRFS_entry has been
+             * defined differently for AIX, so skip this for AIX
+             */
             else if (hasmntopt(HRFS_entry, "loop") != NULL)
                 storage_type_id[storage_type_len - 1] = 5;      /* Removable Disk */
 #endif
