@@ -84,9 +84,14 @@
 #include <net-snmp/agent/auto_nlist.h>
 #include "kernel.h"
 #ifdef solaris2
+#if _SLASH_PROC_METHOD_ && defined _ILP32
+#include <net-snmp/agent/cache_handler.h>
+#include <net-snmp/agent/hardware/memory.h>
+#endif
+
 #include "kernel_sunos5.h"
 #endif
-#if defined(aix4) || defined(aix5)
+#if defined(aix4) || defined(aix5) || defined(aix6)
 #include <procinfo.h>
 #include <sys/types.h>
 #endif
@@ -121,7 +126,7 @@ struct pst_dynamic pst_dyn;
 struct kinfo_proc *proc_table;
 #elif defined(solaris2)
 int            *proc_table;
-#elif defined(aix4) || defined(aix5)
+#elif defined(aix4) || defined(aix5) || defined(aix6)
 struct procsinfo *proc_table;
 #else
 struct proc    *proc_table;
@@ -590,7 +595,7 @@ var_hrswrun(struct variable * vp,
         strncpy(string, proc_buf->p_user.u_comm, sizeof(string));
         string[ sizeof(string)-1 ] = 0;
 #endif
-#elif defined(aix4) || defined(aix5)
+#elif defined(aix4) || defined(aix5) || defined(aix6)
         strncpy(string, proc_table[LowProcIndex].pi_comm, sizeof(string));
         string[ sizeof(string)-1 ] = 0;
         cp = strchr(string, ' ');
@@ -713,7 +718,7 @@ var_hrswrun(struct variable * vp,
             *cp1++ = *cp++;
         *cp1 = 0;
 #endif
-#elif defined(aix4) || defined(aix5)
+#elif defined(aix4) || defined(aix5) || defined(aix6)
         strncpy(string, proc_table[LowProcIndex].pi_comm, sizeof(string));
         string[ sizeof(string)-1 ] = 0;
         cp = strchr(string, ' ');
@@ -742,8 +747,15 @@ var_hrswrun(struct variable * vp,
             sprintf(string, "/proc/%d/status", pid);
             if ((fp = fopen(string, "r")) == NULL)
                 return NULL;
-            fgets(buf, sizeof(buf), fp);        /* Name: process name */
+            if (!fgets(buf, sizeof(buf), fp)) {
+                fclose(fp);
+                return NULL;    /* the process probably died */
+            }
             cp = strchr(buf, ':');
+            if ( cp == NULL ) {
+                fclose(fp);
+                return NULL;    /* the process file is malformed */
+            }
             ++cp;
             while (isspace(*cp))
                 ++cp;
@@ -822,7 +834,7 @@ var_hrswrun(struct variable * vp,
             cp++;
         strcpy(string, cp);
 #endif
-#elif defined(aix4) || defined(aix5)
+#elif defined(aix4) || defined(aix5) || defined(aix6)
         cp = strchr(proc_table[LowProcIndex].pi_comm, ' ');
         if (cp != NULL) {
             cp++;
@@ -906,7 +918,7 @@ var_hrswrun(struct variable * vp,
             long_return = 2;    /* operatingSystem */
         else
             long_return = 4;    /* application */
-#elif defined(aix4) || defined(aix5)
+#elif defined(aix4) || defined(aix5) || defined(aix6)
 		if (proc_table[LowProcIndex].pi_flags & SKPROC) {
 			long_return = 2;	/* kernel process */
 		} else
@@ -973,7 +985,7 @@ var_hrswrun(struct variable * vp,
 #else
         switch (proc_buf->p_stat) {
 #endif
-#elif defined(aix4) || defined(aix5)
+#elif defined(aix4) || defined(aix5) || defined(aix6)
         switch (proc_table[LowProcIndex].pi_state) {
 #else
         switch (proc_table[LowProcIndex].p_stat) {
@@ -1120,7 +1132,7 @@ var_hrswrun(struct variable * vp,
                 long_return = 0;
             }
         }
-#elif defined(aix4) || defined(aix5)
+#elif defined(aix4) || defined(aix5) || defined(aix6)
         long_return = proc_table[LowProcIndex].pi_ru.ru_utime.tv_sec * 100 +
             proc_table[LowProcIndex].pi_ru.ru_utime.tv_usec / 10000000 + /* nanoseconds */
             proc_table[LowProcIndex].pi_ru.ru_stime.tv_sec * 100 +
@@ -1134,16 +1146,45 @@ var_hrswrun(struct variable * vp,
         return (u_char *) & long_return;
     case HRSWRUNPERF_MEM:
 #ifdef HAVE_SYS_PSTAT_H
+# ifdef PGSHIFT
         long_return = (proc_buf.pst_rssize << PGSHIFT) / 1024;
+# else
+        long_return = proc_buf.pst_rssize * getpagesize() / 1024;
+# endif
 #elif defined(dynix)
         long_return = (lowpsinfo.pr_rssize * MMU_PAGESIZE) / 1024;
 #elif defined(solaris2)
 #if _SLASH_PROC_METHOD_
+#ifdef _ILP32
+        if(NULL != proc_buf && 0 == proc_buf->pr_rssize)
+        { /* Odds on that we are looking with a 32 bit app at a 64 bit psinfo.*/
+            netsnmp_memory_info *mem;
+            netsnmp_memory_load();
+            mem = netsnmp_memory_get_byIdx( NETSNMP_MEM_TYPE_PHYSMEM, 0 );
+            if (!mem) 
+            {
+                snmp_log(LOG_INFO, "netsnmp_memory_get_byIdx returned NULL pointer\n");
+                long_return = 0;/* Tried my best, giving up.*/
+            } 
+            else 
+            {/* 0x8000 is the maximum range of pr_pctmem. devision of 1024 is to go from B to kB*/
+                uint32_t pct_unit = (mem->size/0x8000) * (mem->units/1024);
+                long_return = proc_buf ? proc_buf->pr_pctmem * pct_unit : 0;
+            }
+        }
+        else
+        {    
+            long_return = proc_buf ? proc_buf->pr_rssize : 0;
+        
+        }
+#else /*_LP64*/
         long_return = proc_buf ? proc_buf->pr_rssize : 0;
+#endif /*_LP64*/
+
 #else
         long_return = proc_buf->p_swrss;
 #endif
-#elif defined(aix4) || defined(aix5)
+#elif defined(aix4) || defined(aix5) || defined(aix6)
         long_return = proc_table[LowProcIndex].pi_size * getpagesize() / 1024;
 #elif HAVE_KVM_GETPROCS && !defined(darwin8)
   #if defined(NOT_DEFINED) && defined(freebsd5) && __FreeBSD_version >= 500014
@@ -1390,7 +1431,7 @@ Init_HR_SWRun(void)
             nproc = current_proc_entry;
         closedir(f);
     }
-#elif defined(aix4) || defined(aix5)
+#elif defined(aix4) || defined(aix5) || defined(aix6)
     {
 		pid_t proc_index = 0;
 		int avail = 1024;
@@ -1481,7 +1522,7 @@ Get_Next_HR_SWRun(void)
         if (proc_table[current_proc_entry].kp_proc.p_stat != 0)
             return proc_table[current_proc_entry++].kp_proc.p_pid;
     #endif
-#elif defined(aix4) || defined(aix5)
+#elif defined(aix4) || defined(aix5) || defined(aix6)
         if (proc_table[current_proc_entry].pi_state != 0)
             return proc_table[current_proc_entry++].pi_pid;
         else
@@ -1516,7 +1557,7 @@ count_processes(void)
 #if defined(hpux10) || defined(hpux11) || HAVE_KVM_GETPROCS || defined(solaris2)
     total = nproc;
 #else
-#if defined(aix4) || defined(aix5)
+#if defined(aix4) || defined(aix5) || defined(aix6)
     for (i = 0; i < nproc; ++i) {
         if (proc_table[i].pi_state != 0)
 #elif !defined(linux) && !defined(cygwin) && !defined(dynix)
