@@ -1,7 +1,7 @@
 /*
  *  Interface MIB architecture support
  *
- * $Id: interface.c 15768 2007-01-22 16:18:29Z rstory $
+ * $Id: interface.c 17263 2008-10-16 09:56:01Z jsafranek $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -466,7 +466,16 @@ _access_interface_entry_save_name(const char *name, oid index)
                     index, name));
     }
     else
-        netsnmp_assert(index == tmp);
+        if (index != tmp) {
+            static int logged = 0;
+            if (!logged) {
+                snmp_log(LOG_ERR, "IfIndex of an interface changed. Such " \
+                         "interfaces will appear multiple times in IF-MIB.\n");
+                logged = 1;
+            }
+            DEBUGMSGTL(("access:interface:ifIndex", "index %d != tmp for %s\n",
+                         index, name));
+        }
 }
 
 /**
@@ -541,6 +550,19 @@ netsnmp_access_interface_entry_update_stats(netsnmp_interface_entry * prev_vals,
                                        &prev_vals->old_stats->obcast,
                                        &need_wrap_check);
     
+    /*
+     * Copy 32 bit counters
+     */
+    prev_vals->stats.ierrors = new_vals->stats.ierrors;
+    prev_vals->stats.idiscards = new_vals->stats.idiscards;
+    prev_vals->stats.iunknown_protos = new_vals->stats.iunknown_protos;
+    prev_vals->stats.inucast = new_vals->stats.inucast;
+    prev_vals->stats.oerrors = new_vals->stats.oerrors;
+    prev_vals->stats.odiscards = new_vals->stats.odiscards;
+    prev_vals->stats.oqlen = new_vals->stats.oqlen;
+    prev_vals->stats.collisions = new_vals->stats.collisions;
+    prev_vals->stats.onucast = new_vals->stats.onucast;
+
     /*
      * if we've decided we no longer need to check wraps, free old stats
      */
@@ -643,6 +665,7 @@ netsnmp_access_interface_entry_guess_speed(netsnmp_interface_entry *entry)
         entry->speed = 4000000;
     else
         entry->speed = 0;
+    entry->speed_high = entry->speed / 1000000LL;
 }
 
 netsnmp_conf_if_list *
@@ -680,7 +703,12 @@ netsnmp_access_interface_entry_overrides(netsnmp_interface_entry *entry)
         netsnmp_access_interface_entry_overrides_get(entry->name);
     if (if_ptr) {
         entry->type = if_ptr->type;
-        entry->speed = if_ptr->speed;
+        if (if_ptr->speed > 0xffffffff) {
+            entry->speed = 0xffffffff;
+        } else {
+            entry->speed = if_ptr->speed;
+        }
+        entry->speed_high = if_ptr->speed / 1000000LL;
     }
 }
 
@@ -725,7 +753,7 @@ _parse_interface_config(const char *token, char *cptr)
         config_perror("Out of memory");
         return;
     }
-    if_new->speed = strtoul(speed, &ecp, 0);
+    if_new->speed = strtoull(speed, &ecp, 0);
     if (*ecp) {
         config_perror("Bad SPEED value");
         free(if_new);
