@@ -1,7 +1,7 @@
 /*
  *  Interface MIB architecture support
  *
- * $Id: interface_linux.c 16595 2007-07-06 21:14:03Z rstory $
+ * $Id: interface_linux.c 18100 2010-02-05 09:48:46Z jsafranek $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -41,11 +41,13 @@ typedef __u8 u8;           /* ditto */
 #define IF_NAMESIZE 16
 #endif
 
-unsigned int
-netsnmp_linux_interface_get_if_speed(int fd, const char *name);
+unsigned long long
+netsnmp_linux_interface_get_if_speed(int fd, const char *name,
+        unsigned long long defaultspeed);
 #ifdef HAVE_LINUX_ETHTOOL_H
-unsigned int
-netsnmp_linux_interface_get_if_speed_mii(int fd, const char *name);
+unsigned long long
+netsnmp_linux_interface_get_if_speed_mii(int fd, const char *name,
+        unsigned long long defaultspeed);
 #endif
 
 #define PROC_SYS_NET_IPVx_NEIGH_RETRANS_TIME_MS "/proc/sys/net/ipv%d/neigh/%s/retrans_time_ms"
@@ -126,6 +128,14 @@ _arch_interface_has_ipv6(oid if_index, u_int *flags,
 
     *flags &= ~NETSNMP_INTERFACE_FLAGS_HAS_IPV6;
 
+#ifndef SIOCGMIIPHY
+#define SIOCGMIIPHY 0x8947
+#endif
+
+#ifndef SIOCGMIIREG
+#define SIOCGMIIREG 0x8948
+#endif
+
 #ifdef NETSNMP_ENABLE_IPV6
     /*
      * get ipv6 addresses
@@ -627,9 +637,24 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
                 entry->type = IANAIFTYPE_OTHER;
         }
 
-        if (IANAIFTYPE_ETHERNETCSMACD == entry->type)
-            entry->speed =
-                netsnmp_linux_interface_get_if_speed(fd, entry->name);
+        if (IANAIFTYPE_ETHERNETCSMACD == entry->type) {
+            unsigned long long speed;
+            unsigned long long defaultspeed = NOMINAL_LINK_SPEED;
+            if (!(entry->os_flags & IFF_RUNNING)) {
+                /*
+                 * use speed 0 if the if speed cannot be determined *and* the
+                 * interface is down
+                 */
+                defaultspeed = 0;
+            }
+            speed = netsnmp_linux_interface_get_if_speed(fd,
+                    entry->name, defaultspeed);
+            if (speed > 0xffffffffL) {
+                entry->speed = 0xffffffff;
+            } else
+                entry->speed = speed;
+            entry->speed_high = speed / 1000000LL;
+        }
 #ifdef APPLIED_PATCH_836390   /* xxx-rks ifspeed fixes */
         else if (IANAIFTYPE_PROPVIRTUAL == entry->type)
             entry->speed = _get_bonded_if_speed(entry);
@@ -672,8 +697,6 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
 
         netsnmp_access_interface_entry_overrides(entry);
 
-        entry->speed_high = entry->speed / 1000000;
-
         if (! (load_flags & NETSNMP_ACCESS_INTERFACE_LOAD_NO_STATS))
             _parse_stats(entry, stats, scan_expected);
 
@@ -719,14 +742,16 @@ netsnmp_arch_set_admin_status(netsnmp_interface_entry * entry,
 /**
  * Determines network interface speed from ETHTOOL_GSET
  */
-unsigned int
-netsnmp_linux_interface_get_if_speed(int fd, const char *name)
+unsigned long long
+netsnmp_linux_interface_get_if_speed(int fd, const char *name,
+            unsigned long long defaultspeed)
 {
     struct ifreq ifr;
     struct ethtool_cmd edata;
 
     memset(&ifr, 0, sizeof(ifr));
     edata.cmd = ETHTOOL_GSET;
+    edata.speed = 0;
     
     strncpy(ifr.ifr_name, name, sizeof(ifr.ifr_name)-1);
     ifr.ifr_data = (char *) &edata;
@@ -734,35 +759,43 @@ netsnmp_linux_interface_get_if_speed(int fd, const char *name)
     if (ioctl(fd, SIOCETHTOOL, &ifr) == -1) {
         DEBUGMSGTL(("mibII/interfaces", "ETHTOOL_GSET on %s failed\n",
                     ifr.ifr_name));
-        return netsnmp_linux_interface_get_if_speed_mii(fd,name);
+        return netsnmp_linux_interface_get_if_speed_mii(fd,name,defaultspeed);
     }
     
-    if (edata.speed != SPEED_10 && edata.speed != SPEED_100 &&
-        edata.speed != SPEED_1000) {
+    if (edata.speed != SPEED_10 && edata.speed != SPEED_100
+#ifdef SPEED_10000
+        && edata.speed != SPEED_10000
+#endif
+#ifdef SPEED_2500
+        && edata.speed != SPEED_2500
+#endif
+        && edata.speed != SPEED_1000 ) {
         DEBUGMSGTL(("mibII/interfaces", "fallback to mii for %s\n",
                     ifr.ifr_name));
         /* try MII */
-        return netsnmp_linux_interface_get_if_speed_mii(fd,name);
+        return netsnmp_linux_interface_get_if_speed_mii(fd,name,defaultspeed);
     }
 
     /* return in bps */
     DEBUGMSGTL(("mibII/interfaces", "ETHTOOL_GSET on %s speed = %d\n",
                 ifr.ifr_name, edata.speed));
-    return edata.speed*1000*1000;
+    return edata.speed*1000LL*1000LL;
 }
 #endif
  
 /**
  * Determines network interface speed from MII
  */
-unsigned int
+unsigned long long
 #ifdef HAVE_LINUX_ETHTOOL_H
-netsnmp_linux_interface_get_if_speed_mii(int fd, const char *name)
+netsnmp_linux_interface_get_if_speed_mii(int fd, const char *name,
+        unsigned long long  defaultspeed)
 #else
-netsnmp_linux_interface_get_if_speed(int fd, const char *name)
+netsnmp_linux_interface_get_if_speed(int fd, const char *name,
+        unsigned long long defaultspeed)
 #endif
 {
-    unsigned int retspeed = 10000000;
+    unsigned long long retspeed = defaultspeed;
     struct ifreq ifr;
 
     /* the code is based on mii-diag utility by Donald Becker
@@ -770,22 +803,21 @@ netsnmp_linux_interface_get_if_speed(int fd, const char *name)
      */
     ushort *data = (ushort *)(&ifr.ifr_data);
     unsigned phy_id;
-    unsigned char new_ioctl_nums = 0;
     int mii_reg, i;
     ushort mii_val[32];
     ushort bmcr, bmsr, nway_advert, lkpar;
-    const unsigned int media_speeds[] = {10000000, 10000000, 100000000, 100000000, 10000000, 0};	
+    const unsigned long long media_speeds[] = {10000000, 10000000, 100000000, 100000000, 10000000, 0};
     /* It corresponds to "10baseT", "10baseT-FD", "100baseTx", "100baseTx-FD", "100baseT4", "Flow-control", 0, */
 
     strncpy(ifr.ifr_name, name, sizeof(ifr.ifr_name));
     ifr.ifr_name[ sizeof(ifr.ifr_name)-1 ] = 0;
     data[0] = 0;
     
-    if (ioctl(fd, 0x8947, &ifr) >= 0) {
-        new_ioctl_nums = 1;
-    } else if (ioctl(fd, SIOCDEVPRIVATE, &ifr) >= 0) {
-        new_ioctl_nums = 0;
-    } else {
+    /*
+     * SIOCGMIIPHY has been defined since at least kernel 2.4.10 (Sept 2001).
+     * It's probably safe to drop the interim SIOCDEVPRIVATE handling now!
+     */
+    if (ioctl(fd, SIOCGMIIPHY, &ifr) < 0) {
         DEBUGMSGTL(("mibII/interfaces", "SIOCGMIIPHY on %s failed\n",
                     ifr.ifr_name));
         return retspeed;
@@ -796,7 +828,7 @@ netsnmp_linux_interface_get_if_speed(int fd, const char *name)
     for (mii_reg = 0; mii_reg < 8; mii_reg++){
         data[0] = phy_id;
         data[1] = mii_reg;
-        if(ioctl(fd, new_ioctl_nums ? 0x8948 : SIOCDEVPRIVATE+1, &ifr) <0){
+        if(ioctl(fd, SIOCGMIIREG, &ifr) <0){
             DEBUGMSGTL(("mibII/interfaces", "SIOCGMIIREG on %s failed\n", ifr.ifr_name));
         }
         mii_val[mii_reg] = data[3];		
