@@ -33,7 +33,7 @@
 #define IP_STATS_CACHE_TIMEOUT	MIB_STATS_CACHE_TIMEOUT
 #endif
 
-#if defined(HAVE_LIBPERFSTAT_H) && (defined(aix4) || defined(aix5)) && !defined(FIRST_PROTOCOL)
+#if defined(HAVE_LIBPERFSTAT_H) && (defined(aix4) || defined(aix5) || defined(aix6)) && !defined(FIRST_PROTOCOL)
 #include <libperfstat.h>
 #ifdef FIRST_PROTOCOL
 perfstat_protocol_t ps_proto;
@@ -283,7 +283,7 @@ ip_handler(netsnmp_mib_handler          *handler,
         type = ASN_INTEGER;
         break;
     case IPINRECEIVES:
-        ret_value = ipstat.ipInReceives;
+        ret_value = ipstat.ipInReceives & 0xffffffff;
         break;
     case IPINHDRERRORS:
         ret_value = ipstat.ipInHdrErrors;
@@ -301,10 +301,10 @@ ip_handler(netsnmp_mib_handler          *handler,
         ret_value = ipstat.ipInDiscards;
         break;
     case IPINDELIVERS:
-        ret_value = ipstat.ipInDelivers;
+        ret_value = ipstat.ipInDelivers & 0xffffffff;
         break;
     case IPOUTREQUESTS:
-        ret_value = ipstat.ipOutRequests;
+        ret_value = ipstat.ipOutRequests & 0xffffffff;
         break;
     case IPOUTDISCARDS:
         ret_value = ipstat.ipOutDiscards;
@@ -359,7 +359,7 @@ ip_handler(netsnmp_mib_handler          *handler,
         type = ASN_INTEGER;
         break;
     case IPINRECEIVES:
-        ret_value = ipstat.ips_total;
+        ret_value = ipstat.ips_total & 0xffffffff;
         break;
     case IPINHDRERRORS:
         ret_value = ipstat.ips_badsum
@@ -390,7 +390,7 @@ ip_handler(netsnmp_mib_handler          *handler,
 #endif
     case IPINDELIVERS:
 #if STRUCT_IPSTAT_HAS_IPS_DELIVERED
-        ret_value = ipstat.ips_delivered;
+        ret_value = ipstat.ips_delivered & 0xffffffff;
         break;
 #else
         netsnmp_set_request_error(reqinfo, request, SNMP_NOSUCHOBJECT);
@@ -398,7 +398,7 @@ ip_handler(netsnmp_mib_handler          *handler,
 #endif
     case IPOUTREQUESTS:
 #if STRUCT_IPSTAT_HAS_IPS_LOCALOUT
-        ret_value = ipstat.ips_localout;
+        ret_value = ipstat.ips_localout & 0xffffffff;
         break;
 #else
         netsnmp_set_request_error(reqinfo, request, SNMP_NOSUCHOBJECT);
