@@ -778,7 +778,7 @@ var_ifv6Entry(register struct variable * vp,
             memset(&ifr, 0, sizeof(ifr));
             ifr.ifr_addr.sa_family = AF_INET6;
             strncpy(ifr.ifr_name, if_getname(interface),
-                    sizeof(ifr.ifr_name));
+                    sizeof(ifr.ifr_name) - 1);
             if ((s = socket(ifr.ifr_addr.sa_family, SOCK_DGRAM, 0)) < 0)
                 break;
             if (ioctl(s, SIOCGIFMTU, (caddr_t) & ifr) < 0) {
@@ -823,7 +823,8 @@ var_ifv6Entry(register struct variable * vp,
                     DEBUGMSGTL(("mibII/ipv6:var_ipv6", "klookup failed\n"));
                     break;
                 }
-                if (!NETSNMP_KLOOKUP(ifaddr.ifa_addr, (char *) &sdl, sizeof(sdl));
+                if (!NETSNMP_KLOOKUP(ifaddr.ifa_addr,
+                                     (char *) &sdl, sizeof(sdl))) {
                     DEBUGMSGTL(("mibII/ipv6:var_ipv6", "klookup failed\n"));
                     break;
                 }
@@ -944,7 +945,7 @@ var_ifv6Entry(register struct variable * vp,
 
             memset(&ifr, 0, sizeof(ifr));
             strncpy(ifr.ifr_name, if_getname(interface),
-                    sizeof(ifr.ifr_name));
+                    sizeof(ifr.ifr_name) - 1);
             if ((s = socket(AF_INET6, SOCK_DGRAM, 0)) < 0)
                 break;
             if (ioctl(s, SIOCGIFSTAT_IN6, (caddr_t) & ifr) < 0) {
@@ -1097,7 +1098,7 @@ var_icmpv6Entry(register struct variable * vp,
 
             memset(&ifr, 0, sizeof(ifr));
             strncpy(ifr.ifr_name, if_getname(interface),
-                    sizeof(ifr.ifr_name));
+                    sizeof(ifr.ifr_name) - 1);
             if ((s = socket(AF_INET6, SOCK_DGRAM, 0)) < 0)
                 break;
             if (ioctl(s, SIOCGIFSTAT_ICMP6, (caddr_t) & ifr) < 0) {
@@ -1950,7 +1951,7 @@ var_ifv6Entry(register struct variable * vp,
                 break;
             memset(&ifr, 0, sizeof(ifr));
             ifr.ifr_addr.sa_family = AF_INET6;
-            strncpy(ifr.ifr_name, p, sizeof(ifr.ifr_name));
+            strncpy(ifr.ifr_name, p, sizeof(ifr.ifr_name) - 1);
             if ((s = socket(ifr.ifr_addr.sa_family, SOCK_DGRAM, 0)) < 0)
                 break;
             if (ioctl(s, SIOCGIFMTU, (caddr_t) & ifr) < 0) {
@@ -1970,7 +1971,7 @@ var_ifv6Entry(register struct variable * vp,
                 break;
             memset(&ifr, 0, sizeof(ifr));
             ifr.ifr_addr.sa_family = AF_INET6;
-            strncpy(ifr.ifr_name, p, sizeof(ifr.ifr_name));
+            strncpy(ifr.ifr_name, p, sizeof(ifr.ifr_name) - 1);
             if ((s = socket(ifr.ifr_addr.sa_family, SOCK_DGRAM, 0)) < 0)
                 break;
             if (ioctl(s, SIOCGIFHWADDR, &ifr) < 0) {
@@ -1999,7 +2000,7 @@ var_ifv6Entry(register struct variable * vp,
                 break;
             memset(&ifr, 0, sizeof(ifr));
             ifr.ifr_addr.sa_family = AF_INET6;
-            strncpy(ifr.ifr_name, p, sizeof(ifr.ifr_name));
+            strncpy(ifr.ifr_name, p, sizeof(ifr.ifr_name) - 1);
             if ((s = socket(ifr.ifr_addr.sa_family, SOCK_DGRAM, 0)) < 0)
                 break;
             if (ioctl(s, SIOCGIFFLAGS, &ifr) < 0) {
