@@ -188,7 +188,8 @@ udpTable_handler(netsnmp_mib_handler          *handler,
     netsnmp_table_request_info *table_info;
     UDPTABLE_ENTRY_TYPE	  *entry;
     oid      subid;
-    long     port,addr;
+    long     port;
+    in_addr_t addr;
 
     DEBUGMSGTL(("mibII/udpTable", "Handler - mode %s\n",
                     se_find_label_in_slist("agent_mode", reqinfo->mode)));
@@ -213,12 +214,12 @@ udpTable_handler(netsnmp_mib_handler          *handler,
                 addr = ntohl(IN6_EXTRACT_V4ADDR(&entry->pcb.inp_laddr));
 	        snmp_set_var_typed_value(requestvb, ASN_IPADDRESS,
                                          (u_char*)&addr,
-                                         sizeof(addr));
+                                         sizeof(uint32_t));
 #else
                 addr = UDP_ADDRESS_TO_HOST_ORDER(entry->UDPTABLE_LOCALADDRESS);
 	        snmp_set_var_typed_value(requestvb, ASN_IPADDRESS,
                                          (u_char *)&addr,
-                                         sizeof(addr));
+                                         sizeof(uint32_t));
 #endif
                 break;
             case UDPLOCALPORT:
@@ -360,7 +361,8 @@ udpTable_next_entry( void **loop_context,
                      netsnmp_iterator_info *data)
 {
     UDPTABLE_ENTRY_TYPE	 *entry = (UDPTABLE_ENTRY_TYPE *)*loop_context;
-    long port, addr;
+    long port;
+    long addr;
 
     if (!entry)
         return NULL;
@@ -373,9 +375,9 @@ udpTable_next_entry( void **loop_context,
                               (u_char*)&IN6_EXTRACT_V4ADDR(&entry->pcb.inp_laddr),
                                  sizeof(IN6_EXTRACT_V4ADDR(&entry->pcb.inp_laddr)));
 #else
-    addr = UDP_ADDRESS_TO_NETWORK_ORDER((u_long)entry->UDPTABLE_LOCALADDRESS);
+    addr = UDP_ADDRESS_TO_NETWORK_ORDER((in_addr_t)entry->UDPTABLE_LOCALADDRESS);
     snmp_set_var_value(index, (u_char *)&addr,
-                                 sizeof(entry->UDPTABLE_LOCALADDRESS));
+                                 sizeof(addr));
 #endif
     port = UDP_PORT_TO_HOST_ORDER(entry->UDPTABLE_LOCALPORT);
     snmp_set_var_value(index->next_variable,
@@ -541,6 +543,8 @@ udpTable_load(netsnmp_cache *cache, void *vmagic)
         /*
          * Not interested in 'idle' entries, apparently....
          */
+        DEBUGMSGTL(("mibII/udpTable", "UDP Entry %x:%d (%d)\n",
+                     entry.udpLocalAddress, entry.udpLocalPort, entry.udpEntryInfo.ue_state));
         if (entry.udpEntryInfo.ue_state == MIB2_UDP_idle) {
             /*
              * Build up a linked list copy of the getMibstat results
@@ -608,7 +612,7 @@ udpTable_load(netsnmp_cache *cache, void *vmagic)
 }
 #else                           /* WIN32 cygwin*/
 
-#if (defined(CAN_USE_SYSCTL) && defined(UDPCTL_PCBLIST))
+#if (defined(NETSNMP_CAN_USE_SYSCTL) && defined(UDPCTL_PCBLIST))
 int
 udpTable_load(netsnmp_cache *cache, void *vmagic)
 {
@@ -663,7 +667,7 @@ udpTable_load(netsnmp_cache *cache, void *vmagic)
     DEBUGMSGTL(("mibII/udpTable", "Failed to load UDP Table (sysctl)\n"));
     return -1;
 }
-#else		/* (defined(CAN_USE_SYSCTL) && defined(UDPCTL_PCBLIST)) */
+#else		/* (defined(NETSNMP_CAN_USE_SYSCTL) && defined(UDPCTL_PCBLIST)) */
 #ifdef PCB_TABLE
 int
 udpTable_load(netsnmp_cache *cache, void *vmagic)
@@ -687,7 +691,11 @@ udpTable_load(netsnmp_cache *cache, void *vmagic)
         nnew = SNMP_MALLOC_TYPEDEF(struct inpcb);
         if (!nnew)
             break;
-        klookup((unsigned long) entry, (char *) nnew, sizeof(struct inpcb));
+
+        if (!NETSNMP_KLOOKUP(entry, (char *) nnew, sizeof(struct inpcb))) {
+            DEBUGMSGTL(("mibII/udpTable:udpTable_load", "klookup failed\n"));
+            break;
+        }
 
         entry    = nnew->inp_queue.cqe_next;	/* Next kernel entry */
 	nnew->inp_queue.cqe_next = udp_head;
@@ -729,7 +737,11 @@ udpTable_load(netsnmp_cache *cache, void *vmagic)
         nnew = SNMP_MALLOC_TYPEDEF(struct inpcb);
         if (!nnew)
             break;
-        klookup((unsigned long) entry, (char *) nnew, sizeof(struct inpcb));
+
+        if (!NETSNMP_KLOOKUP(entry, (char *) nnew, sizeof(struct inpcb))) {
+            DEBUGMSGTL(("mibII/udpTable:udpTable_load", "klookup failed\n"));
+            break;
+        }
 
         entry    = nnew->INP_NEXT_SYMBOL;		/* Next kernel entry */
 	nnew->INP_NEXT_SYMBOL = udp_head;
@@ -756,7 +768,7 @@ udpTable_load(netsnmp_cache *cache, void *vmagic)
 }
 #endif				/* UDB_SYMBOL */
 #endif				/* PCB_TABLE */
-#endif		/* (defined(CAN_USE_SYSCTL) && defined(UDPCTL_PCBLIST)) */
+#endif		/* (defined(NETSNMP_CAN_USE_SYSCTL) && defined(UDPCTL_PCBLIST)) */
 #endif                          /* WIN32 cygwin*/
 #endif                          /* linux */
 #endif                          /* solaris2 */
