@@ -94,6 +94,8 @@ init_vacm_config_tokens(void) {
                                   NULL, NULL);
     snmpd_register_config_handler("vacmAccess", vacm_parse_config_access,
                                   NULL, NULL);
+    snmpd_register_config_handler("vacmAuthAccess", vacm_parse_config_auth_access,
+                                  NULL, NULL);
 
     /* easy community auth handler */
     snmpd_register_config_handler("authcommunity",
@@ -130,12 +132,12 @@ init_vacm_config_tokens(void) {
  */
 void
 init_vacm_snmpd_easy_tokens(void) {
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
     snmpd_register_config_handler("rwcommunity", vacm_parse_rwcommunity, NULL,
                                   "community [default|hostname|network/bits [oid]]");
     snmpd_register_config_handler("rocommunity", vacm_parse_rocommunity, NULL,
                                   "community [default|hostname|network/bits [oid]]");
-#ifdef SNMP_TRANSPORT_UDPIPV6_DOMAIN
+#ifdef NETSNMP_TRANSPORT_UDPIPV6_DOMAIN
     snmpd_register_config_handler("rwcommunity6", vacm_parse_rwcommunity6, NULL,
                                   "community [default|hostname|network/bits [oid]]");
     snmpd_register_config_handler("rocommunity6", vacm_parse_rocommunity6, NULL,
@@ -820,15 +822,17 @@ vacm_create_simple(const char *token, char *confline,
     char            theoid[SPRINT_MAX_LEN];
     char            viewname[SPRINT_MAX_LEN];
     char           *view_ptr = viewname;
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
     char            addressname[SPRINT_MAX_LEN];
 #endif
     const char     *rw = "none";
     char            model[SPRINT_MAX_LEN];
-    char           *cp;
+    char           *cp, *tmp;
     char            secname[SPRINT_MAX_LEN];
     char            grpname[SPRINT_MAX_LEN];
-    char            authtype[SPRINT_MAX_LEN];
+    char            authlevel[SPRINT_MAX_LEN];
+    char            context[SPRINT_MAX_LEN];
+    int             ctxprefix = 1;  /* Default to matching all contexts */
     static int      commcount = 0;
     /* Conveniently, the community-based security
        model values can also be used as bit flags */
@@ -839,6 +843,9 @@ vacm_create_simple(const char *token, char *confline,
      * init 
      */
     strcpy(model, "any");
+    memset(context, 0, sizeof(context));
+    memset(secname, 0, sizeof(secname));
+    memset(grpname, 0, sizeof(grpname));
 
     /*
      * community name or user name 
@@ -854,7 +861,7 @@ vacm_create_simple(const char *token, char *confline,
              * -s model ... 
              */
             if (cp)
-                cp = copy_nword(cp, model, sizeof(authtype));
+                cp = copy_nword(cp, model, sizeof(model));
             if (!cp) {
                 config_perror("illegal line");
                 return;
@@ -865,21 +872,21 @@ vacm_create_simple(const char *token, char *confline,
             strcpy(model, "usm");
         }
         /*
-         * authentication type 
+         * authentication level 
          */
         if (cp && *cp)
-            cp = copy_nword(cp, authtype, sizeof(authtype));
+            cp = copy_nword(cp, authlevel, sizeof(authlevel));
         else
-            strcpy(authtype, "auth");
-        DEBUGMSGTL((token, "setting auth type: \"%s\"\n", authtype));
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+            strcpy(authlevel, "auth");
+        DEBUGMSGTL((token, "setting auth level: \"%s\"\n", authlevel));
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
     } else {
         if (strcmp(community, "-v") == 0) {
             /*
              * -v version ... 
              */
             if (cp)
-                cp = copy_nword(cp, model, sizeof(authtype));
+                cp = copy_nword(cp, model, sizeof(model));
             if (!cp) {
                 config_perror("illegal line");
                 return;
@@ -904,9 +911,9 @@ vacm_create_simple(const char *token, char *confline,
             strcpy(addressname, "default");
         }
         /*
-         * authtype has to be noauth 
+         * authlevel has to be noauth 
          */
-        strcpy(authtype, "noauth");
+        strcpy(authlevel, "noauth");
 #endif /* support for community based SNMP */
     }
 
@@ -926,13 +933,34 @@ vacm_create_simple(const char *token, char *confline,
         strcpy(viewname, "_all_");
         view_ptr = NULL;
     }
+    /*
+     * optional, non-default context
+     */
+    if (cp && *cp) {
+        cp = copy_nword(cp, context, sizeof(context));
+        tmp = (context + strlen(context)-1);
+        if (tmp && *tmp == '*') {
+            *tmp = '\0';
+            ctxprefix = 1;
+        } else {
+            /*
+             * If no context field is given, then we default to matching
+             *   all contexts (for compatability with previous releases).
+             * But if a field context is specified (not ending with '*')
+             *   then this should be taken as an exact match.
+             * Specifying a context field of "" will match the default
+             *   context (and *only* the default context).
+             */
+            ctxprefix = 0;
+        }
+    }
 
     if (viewtypes & VACM_VIEW_WRITE_BIT)
         rw = viewname;
 
     commcount++;
 
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
     if (parsetype == VACM_CREATE_SIMPLE_COMIPV4 ||
         parsetype == VACM_CREATE_SIMPLE_COM) {
         vacm_gen_com2sec(commcount, community, addressname,
@@ -941,7 +969,7 @@ vacm_create_simple(const char *token, char *confline,
                          view_ptr, sizeof(viewname), commversion);
     }
     
-#ifdef SNMP_TRANSPORT_UNIX_DOMAIN
+#ifdef NETSNMP_TRANSPORT_UNIX_DOMAIN
     if (parsetype == VACM_CREATE_SIMPLE_COMUNIX ||
         parsetype == VACM_CREATE_SIMPLE_COM) {
         snprintf(line, sizeof(line), "%s %s '%s'",
@@ -952,7 +980,7 @@ vacm_create_simple(const char *token, char *confline,
     }
 #endif
 
-#ifdef SNMP_TRANSPORT_UDPIPV6_DOMAIN
+#ifdef NETSNMP_TRANSPORT_UDPIPV6_DOMAIN
     if (parsetype == VACM_CREATE_SIMPLE_COMIPV6 ||
         parsetype == VACM_CREATE_SIMPLE_COM) {
         vacm_gen_com2sec(commcount, community, addressname,
@@ -982,6 +1010,9 @@ vacm_create_simple(const char *token, char *confline,
              * group   anonymousGroupNameNUM  any      anonymousSecNameNUM 
              */
             snprintf(grpname, sizeof(grpname), "grp%.28s", secname);
+            for (tmp=grpname; *tmp; tmp++)
+                if (!isalnum(*tmp))
+                    *tmp = '_';
             snprintf(line, sizeof(line),
                      "%s %s %s", grpname, model, secname);
             line[ sizeof(line)-1 ] = 0;
@@ -990,6 +1021,9 @@ vacm_create_simple(const char *token, char *confline,
         }
     } else {
         snprintf(grpname, sizeof(grpname), "grp%.28s", secname);
+        for (tmp=grpname; *tmp; tmp++)
+            if (!isalnum(*tmp))
+                *tmp = '_';
     }
 
     /*
@@ -1015,8 +1049,11 @@ vacm_create_simple(const char *token, char *confline,
          * access  anonymousGroupNameNUM  "" MODEL AUTHTYPE prefix anonymousViewNUM [none/anonymousViewNUM] [none/anonymousViewNUM] 
          */
         snprintf(line, sizeof(line),
-                 "%s  \"\" %s %s prefix %s %s %s",
-                 grpname, model, authtype, viewname, rw, rw);
+                 "%s %s %s %s %s %s %s %s",
+                 grpname, context[0] ? context : "\"\"",
+                 model, authlevel,
+                (ctxprefix ? "prefix" : "exact"),
+                 viewname, rw, rw);
         line[ sizeof(line)-1 ] = 0;
         DEBUGMSGTL((token, "passing: %s %s\n", "access", line));
         vacm_parse_access("access", line);
@@ -1030,8 +1067,10 @@ vacm_create_simple(const char *token, char *confline,
         for(i = 0; i <= VACM_MAX_VIEWS; i++) {
             if (viewtypes & (1 << i)) {
                 snprintf(line, sizeof(line),
-                         "%s  \"\" %s %s prefix %s %s",
-                         grpname, model, authtype,
+                         "%s %s %s %s %s %s %s",
+                         grpname, context[0] ? context : "\"\"",
+                         model, authlevel,
+                        (ctxprefix ? "prefix" : "exact"),
                          se_find_label_in_slist(VACM_VIEW_ENUM_NAME, i),
                          viewname);
                 line[ sizeof(line)-1 ] = 0;
@@ -1079,19 +1118,34 @@ vacm_warn_if_not_configured(int majorID, int minorID, void *serverarg,
         name = "snmpd";
     
     if (!vacm_is_configured()) {
-        if (MASTER_AGENT == agent_mode) {
+        /*
+         *  An AgentX subagent relies on the master agent to apply suitable
+         *    access control checks, so doesn't need local VACM configuration.
+         *  The trap daemon has a separate check (see below).
+         *
+         *  Otherwise, an AgentX master or SNMP standalone agent requires some
+         *    form of VACM configuration.  No config means that no incoming
+         *    requests will be accepted, so warn the user accordingly.
+         */
+        if ((MASTER_AGENT == agent_mode) && (strcmp(name, "snmptrapd") != 0)) {
             snmp_log(LOG_WARNING,
-                 "Warning: no access control information configured.\n  It's "
-                 "unlikely this agent can serve any useful purpose in this "
-                 "state.\n  Run \"snmpconf -g basic_setup\" to help you "
-                 "configure the %s.conf file for this agent.\n", name );
+                 "Warning: no access control information configured.\n"
+                 "  (Config search path: %s)\n"
+                 "  It's unlikely this agent can serve any useful purpose in this state.\n"
+                 "  Run \"snmpconf -g basic_setup\" to help you "
+                 "configure the %s.conf file for this agent.\n",
+                 get_configuration_directory(), name);
         }
 
-/*
- * XXX - HACK ALERT!!
- *
- * This duplicates a definition from "apps/snmptrapd_ds.h"
- */
+        /*
+         *  The trap daemon implements VACM-style access control for incoming
+         *    notifications, but offers a way of turning this off (for backwards
+         *    compatability).  Check for this explicitly, and warn if necessary.
+         *
+         *  NB:  The NETSNMP_DS_APP_NO_AUTHORIZATION definition is a duplicate
+         *       of an identical setting in "apps/snmptrapd_ds.h".
+         *       These two need to be kept in synch.
+         */
 #ifndef NETSNMP_DS_APP_NO_AUTHORIZATION
 #define NETSNMP_DS_APP_NO_AUTHORIZATION 17
 #endif
@@ -1100,7 +1154,9 @@ vacm_warn_if_not_configured(int majorID, int minorID, void *serverarg,
                                     NETSNMP_DS_APP_NO_AUTHORIZATION)) {
             snmp_log(LOG_WARNING,
                  "Warning: no access control information configured.\n"
-                 "This receiver will *NOT* accept any incoming notifications.\n");
+                 "  (Config search path: %s)\n"
+                 "This receiver will *NOT* accept any incoming notifications.\n",
+                 get_configuration_directory());
         }
     }
     return SNMP_ERR_NOERROR;
@@ -1218,16 +1274,17 @@ vacm_check_view(netsnmp_pdu *pdu, oid * name, size_t namelen,
 #define CONTEXTNAMEINDEXLEN 32
     char            contextNameIndex[CONTEXTNAMEINDEXLEN + 1];
 
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
-#if defined(DISABLE_SNMPV1)
-    if (pdu->version == SNMP_VERSION_2c) {
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
+#if defined(NETSNMP_DISABLE_SNMPV1)
+    if (pdu->version == SNMP_VERSION_2c)
 #else
-#if defined(DISABLE_SNMPV2C)
-    if (pdu->version == SNMP_VERSION_1) {
+#if defined(NETSNMP_DISABLE_SNMPV2C)
+    if (pdu->version == SNMP_VERSION_1)
 #else
-    if (pdu->version == SNMP_VERSION_1 || pdu->version == SNMP_VERSION_2c) {
+    if (pdu->version == SNMP_VERSION_1 || pdu->version == SNMP_VERSION_2c)
 #endif
 #endif
+    {
         pdu_community = pdu->community;
         if (!pdu_community)
             pdu_community = "";
@@ -1255,7 +1312,7 @@ vacm_check_view(netsnmp_pdu *pdu, oid * name, size_t namelen,
          */
 
         if (pdu->tDomain == netsnmpUDPDomain
-#ifdef SNMP_TRANSPORT_TCP_DOMAIN
+#ifdef NETSNMP_TRANSPORT_TCP_DOMAIN
             || pdu->tDomain == netsnmp_snmpTCPDomain
 #endif
             ) {
@@ -1273,9 +1330,9 @@ vacm_check_view(netsnmp_pdu *pdu, oid * name, size_t namelen,
             SNMP_FREE(pdu->contextName);
             pdu->contextName = strdup(contextName);
             pdu->contextNameLen = strlen(contextName);
-#ifdef SNMP_TRANSPORT_UDPIPV6_DOMAIN
+#ifdef NETSNMP_TRANSPORT_UDPIPV6_DOMAIN
         } else if (pdu->tDomain == netsnmp_UDPIPv6Domain
-#ifdef SNMP_TRANSPORT_TCPIPV6_DOMAIN
+#ifdef NETSNMP_TRANSPORT_TCPIPV6_DOMAIN
                    || pdu->tDomain == netsnmp_TCPIPv6Domain
 #endif
             ) {
@@ -1294,7 +1351,7 @@ vacm_check_view(netsnmp_pdu *pdu, oid * name, size_t namelen,
             pdu->contextName = strdup(contextName);
             pdu->contextNameLen = strlen(contextName);
 #endif
-#ifdef SNMP_TRANSPORT_UNIX_DOMAIN
+#ifdef NETSNMP_TRANSPORT_UNIX_DOMAIN
         } else if (pdu->tDomain == netsnmp_UnixDomain){
             if (!netsnmp_unix_getSecName(pdu->transport_data,
                                          pdu->transport_data_length,
@@ -1338,7 +1395,7 @@ vacm_check_view(netsnmp_pdu *pdu, oid * name, size_t namelen,
     }
 
     if (sn == NULL) {
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
         snmp_increment_statistic(STAT_SNMPINBADCOMMUNITYNAMES);
 #endif
         DEBUGMSGTL(("mibII/vacm_vars",
@@ -1413,16 +1470,17 @@ vacm_check_view(netsnmp_pdu *pdu, oid * name, size_t namelen,
     DEBUGMSG(("mibII/vacm_vars", ", vt=%d\n", vp->viewType));
 
     if (vp->viewType == SNMP_VIEW_EXCLUDED) {
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
-#if defined(DISABLE_SNMPV1)
-        if (pdu->version == SNMP_VERSION_2c) {
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
+#if defined(NETSNMP_DISABLE_SNMPV1)
+        if (pdu->version == SNMP_VERSION_2c)
 #else
-#if defined(DISABLE_SNMPV2C)
-        if (pdu->version == SNMP_VERSION_1) {
+#if defined(NETSNMP_DISABLE_SNMPV2C)
+        if (pdu->version == SNMP_VERSION_1)
 #else
-        if (pdu->version == SNMP_VERSION_1 || pdu->version == SNMP_VERSION_2c) {
+        if (pdu->version == SNMP_VERSION_1 || pdu->version == SNMP_VERSION_2c)
 #endif
 #endif
+        {
             snmp_increment_statistic(STAT_SNMPINBADCOMMUNITYUSES);
         }
 #endif
