@@ -139,7 +139,7 @@ smux_parse_peer_auth(const char *token, char *cptr)
 {
     smux_peer_auth *aptr;
     char           *password_cptr;
-    int             cptr_len;
+    int             rv;
 
     if ((aptr =
          (smux_peer_auth *) calloc(1, sizeof(smux_peer_auth))) == NULL) {
@@ -151,30 +151,31 @@ smux_parse_peer_auth(const char *token, char *cptr)
 	return;
     }
 
+    password_cptr = strchr(cptr, ' ');
+    if (password_cptr)
+        *(password_cptr++) = '\0';
+
     /*
      * oid 
      */
     aptr->sa_active_fd = -1;
     aptr->sa_oid_len = MAX_OID_LEN;
-    read_objid( cptr, aptr->sa_oid, &aptr->sa_oid_len );
+    rv = read_objid( cptr, aptr->sa_oid, &aptr->sa_oid_len );
     DEBUGMSGTL(("smux_conf", "parsing registration for: %s\n", cptr));
-
-    password_cptr = strchr(cptr, ' ');
-    cptr_len = strlen(cptr);
+    if (!rv)
+        config_perror("Error parsing smux oid");
 
     if (password_cptr != NULL) {    /* Do we have a password or not? */
-        *password_cptr = 0x0;
-	if ((&password_cptr - &cptr + 1) < cptr_len) {
-	    cptr = ++password_cptr;
 	    DEBUGMSGTL(("smux_conf", "password is: %s\n",
-	                SNMP_STRORNULL(cptr)));
-	}
+	                SNMP_STRORNULL(password_cptr)));
 
         /*
          * password 
          */
-        if (cptr)
-            strcpy(aptr->sa_passwd, cptr);
+        if (*password_cptr) {
+            strncpy(aptr->sa_passwd, password_cptr, SMUXMAXSTRLEN-1);
+            aptr->sa_passwd[SMUXMAXSTRLEN-1] = '\0';
+        }
     } else {
         /*
          * null passwords OK 
@@ -415,7 +416,7 @@ var_smux_write(int action,
             return SNMP_ERR_GENERR;
         }
 
-        if (send(rptr->sr_fd, buf, len, 0) < 0) {
+        if (sendto(rptr->sr_fd, buf, len, 0, NULL, 0) < 0) {
             DEBUGMSGTL(("smux", "[var_smux_write] send failed\n"));
             return SNMP_ERR_GENERR;
         }
@@ -424,8 +425,8 @@ var_smux_write(int action,
             /*
              * peek at what's received 
              */
-            if ((len = recv(rptr->sr_fd, buf,
-                            SMUXMAXPKTSIZE, MSG_PEEK)) <= 0) {
+            if ((len = recvfrom(rptr->sr_fd, buf,
+                            SMUXMAXPKTSIZE, MSG_PEEK, NULL, 0)) <= 0) {
                 if ((len == -1) && ((errno == EINTR) || (errno == EAGAIN)))
                 {
                    continue;
@@ -464,7 +465,7 @@ var_smux_write(int action,
             do
             {
                len = tmp_len;
-               len = recv(rptr->sr_fd, buf, len, 0);
+               len = recvfrom(rptr->sr_fd, buf, len, 0, NULL, 0);
             }
             while((len == -1) && ((errno == EINTR) || (errno == EAGAIN)));
 
@@ -548,7 +549,7 @@ var_smux_write(int action,
                         "[var_smux_write] entering FREE - sending Commit \n"));
         }
 
-        if ((send(rptr->sr_fd, sout, 3, 0)) < 0) {
+        if ((sendto(rptr->sr_fd, sout, 3, 0, NULL, 0)) < 0) {
             DEBUGMSGTL(("smux",
                         "[var_smux_write] send rollback/commit failed\n"));
             return SNMP_ERR_GENERR;
@@ -606,7 +607,7 @@ smux_accept(int sd)
          */
         do
         {
-           length = recv(fd, (char *) data, SMUXMAXPKTSIZE, 0);
+           length = recvfrom(fd, (char *) data, SMUXMAXPKTSIZE, 0, NULL, 0);
         }
         while((length == -1) && ((errno == EINTR) || (errno == EAGAIN)));
 
@@ -625,14 +626,14 @@ smux_accept(int sd)
         if ((ptr = asn_parse_header(ptr, &len, &type)) == NULL) {
             smux_send_close(fd, SMUXC_PACKETFORMAT);
             close(fd);
-            DEBUGMSGTL(("smux", "[smux_accept] peer on %d sent bad open"));
+            DEBUGMSGTL(("smux", "[smux_accept] peer on %d sent bad open", fd));
             return -1;
         } else if (type != (u_char) SMUX_OPEN) {
             smux_send_close(fd, SMUXC_PROTOCOLERROR);
             close(fd);
             DEBUGMSGTL(("smux",
                         "[smux_accept] peer on %d did not send open: (%d)\n",
-                        type));
+                        fd, type));
             return -1;
         }
         ptr = smux_open_process(fd, ptr, &len, &fail);
@@ -684,7 +685,7 @@ smux_process(int fd)
 
     do
     {
-       length = recv(fd, (char *) data, SMUXMAXPKTSIZE, MSG_PEEK);
+       length = recvfrom(fd, (char *) data, SMUXMAXPKTSIZE, MSG_PEEK, NULL, 0);
     }
     while((length == -1) && ((errno == EINTR) || (errno == EAGAIN)));
 
@@ -712,7 +713,7 @@ smux_process(int fd)
     do
     {
        length = tmp_length;
-       length = recv(fd, (char *) data, length, 0);
+       length = recvfrom(fd, (char *) data, length, 0, NULL, 0);
     }
     while((length == -1) && ((errno == EINTR) || (errno == EAGAIN)));
 
@@ -912,7 +913,7 @@ smux_send_close(int fd, int reason)
     /*
      * send a response back 
      */
-    if (send(fd, (char *) outpacket, 3, 0) < 0) {
+    if (sendto(fd, (char *) outpacket, 3, 0, NULL, 0) < 0) {
         snmp_log_perror("[smux_snmp_close] send failed");
     }
 }
@@ -1062,7 +1063,7 @@ smux_rreq_process(int sd, u_char * ptr, size_t * len)
                (operation == SMUX_REGOP_REGISTER_RW)) {
         if (priority < -1) {
             DEBUGMSGTL(("smux",
-                        "[smux_rreq_process] peer fd %d invalid priority",
+                        "[smux_rreq_process] peer fd %d invalid priority %ld",
                         sd, priority));
             smux_send_rrsp(sd, -1);
             return NULL;
@@ -1351,7 +1352,7 @@ smux_snmp_process(int exact,
     DEBUGMSGOID(("smux", objid, *len));
     DEBUGMSG(("smux", "\n"));
 
-    if (send(sd, (char *) packet, length, 0) < 0) {
+    if (sendto(sd, (char *) packet, length, 0, NULL, 0) < 0) {
         snmp_log_perror("[smux_snmp_process] send failed");
     }
 
@@ -1363,7 +1364,7 @@ smux_snmp_process(int exact,
         /*
          * peek at what's received 
          */
-        length = recv(sd, (char *) result, SMUXMAXPKTSIZE, MSG_PEEK);
+        length = recvfrom(sd, (char *) result, SMUXMAXPKTSIZE, MSG_PEEK, NULL, 0);
         if (length <= 0) {
             if ((length == -1) && ((errno == EINTR) || (errno == EAGAIN)))
             {
@@ -1402,7 +1403,7 @@ smux_snmp_process(int exact,
         do
         {
            length = tmp_length;
-           length = recv(sd, (char *) result, length, 0);
+           length = recvfrom(sd, (char *) result, length, 0, NULL, 0);
         }
         while((length == -1) && ((errno == EINTR) || (errno == EAGAIN)));
 
@@ -1805,7 +1806,7 @@ smux_send_rrsp(int sd, int pri)
         pri <<= 8;
     }
 
-    sent = send(sd, (char *) outdata, sizeof outdata, 0);
+    sent = sendto(sd, (char *) outdata, sizeof outdata, 0, NULL, 0);
     if (sent < 0) {
         DEBUGMSGTL(("smux", "[smux_send_rrsp] send failed\n"));
     }
