@@ -712,7 +712,7 @@ write_usmUserAuthProtocol(int action,
                 ((oid *) var_val, var_val_len / sizeof(oid),
                  usmNoAuthProtocol,
                  sizeof(usmNoAuthProtocol) / sizeof(oid)) == 0
-#ifndef DISABLE_MD5
+#ifndef NETSNMP_DISABLE_MD5
                 || snmp_oid_compare((oid *) var_val,
                                     var_val_len / sizeof(oid),
                                     usmHMACMD5AuthProtocol,
@@ -823,7 +823,7 @@ write_usmUserAuthKeyChange(int action,
         if ((uptr = usm_parse_user(name, name_len)) == NULL) {
             return SNMP_ERR_INCONSISTENTNAME;
         } else {
-#ifndef DISABLE_MD5
+#ifndef NETSNMP_DISABLE_MD5
             if (snmp_oid_compare(uptr->authProtocol, uptr->authProtocolLen,
                                  usmHMACMD5AuthProtocol,
                                  sizeof(usmHMACMD5AuthProtocol) /
@@ -987,18 +987,16 @@ write_usmUserPrivProtocol(int action,
                     ((oid *) var_val, var_val_len / sizeof(oid),
                      usmNoPrivProtocol,
                      sizeof(usmNoPrivProtocol) / sizeof(oid)) != 0
-#ifndef DISABLE_DES
-                    && snmp_oid_compare((oid *) var_val,
-                                        var_val_len / sizeof(oid),
-                                        usmDESPrivProtocol,
-                                        sizeof(usmDESPrivProtocol) /
-                                        sizeof(oid) != 0)
+#ifndef NETSNMP_DISABLE_DES
+                 && snmp_oid_compare
+                    ((oid *) var_val, var_val_len / sizeof(oid),
+                     usmDESPrivProtocol,
+                     sizeof(usmDESPrivProtocol) / sizeof(oid)) != 0
 #endif
-                    && snmp_oid_compare((oid *) var_val,
-                                        var_val_len / sizeof(oid),
-                                        usmAESPrivProtocol,
-                                        sizeof(usmAESPrivProtocol) /
-                                        sizeof(oid) != 0)) {
+                 && snmp_oid_compare
+                    ((oid *) var_val, var_val_len / sizeof(oid),
+                     usmAESPrivProtocol,
+                     sizeof(usmAESPrivProtocol) / sizeof(oid)) != 0) {
                     return SNMP_ERR_WRONGVALUE;
                 }
             }
@@ -1075,7 +1073,7 @@ write_usmUserPrivKeyChange(int action,
         if ((uptr = usm_parse_user(name, name_len)) == NULL) {
             return SNMP_ERR_INCONSISTENTNAME;
         } else {
-#ifndef DISABLE_DES
+#ifndef NETSNMP_DISABLE_DES
             if (snmp_oid_compare(uptr->privProtocol, uptr->privProtocolLen,
                                  usmDESPrivProtocol,
                                  sizeof(usmDESPrivProtocol) /
@@ -1352,6 +1350,8 @@ write_usmUserStatus(int action,
         if (usm_parse_oid(&name[USM_MIB_LENGTH], name_len - USM_MIB_LENGTH,
                           &engineID, &engineIDLen, (u_char **) & newName,
                           &nameLen)) {
+            DEBUGMSGTL(("usmUser",
+                        "can't parse the OID for engineID or name\n"));
             return SNMP_ERR_INCONSISTENTNAME;
         }
 
@@ -1462,9 +1462,12 @@ write_usmUserStatus(int action,
             }
         }
     } else if (action == UNDO || action == FREE) {
-        usm_parse_oid(&name[USM_MIB_LENGTH], name_len - USM_MIB_LENGTH,
+        if (usm_parse_oid(&name[USM_MIB_LENGTH], name_len - USM_MIB_LENGTH,
                       &engineID, &engineIDLen, (u_char **) & newName,
-                      &nameLen);
+                      &nameLen)) {
+            /* Can't extract engine info from the OID - nothing to undo */
+            return SNMP_ERR_NOERROR;
+        }
         uptr = usm_get_user(engineID, engineIDLen, newName);
         SNMP_FREE(engineID);
         SNMP_FREE(newName);
