@@ -52,7 +52,7 @@ static kstat_io_t kio;
 static int      cache_disknr = -1;
 #endif                          /* solaris2 */
 
-#if defined(aix4) || defined(aix5)
+#if defined(aix4) || defined(aix5) || defined(aix6)
 /*
  * handle disk statistics via libperfstat
  */
@@ -148,9 +148,11 @@ init_diskio(void)
         {DISKIO_NWRITTEN, ASN_COUNTER, RONLY, var_diskio, 1, {4}},
         {DISKIO_READS, ASN_COUNTER, RONLY, var_diskio, 1, {5}},
         {DISKIO_WRITES, ASN_COUNTER, RONLY, var_diskio, 1, {6}},
+#if defined(freebsd4) || defined(freebsd5)
         {DISKIO_LA1, ASN_INTEGER, RONLY, var_diskio, 1, {9}},
         {DISKIO_LA5, ASN_INTEGER, RONLY, var_diskio, 1, {10}},
         {DISKIO_LA15, ASN_INTEGER, RONLY, var_diskio, 1, {11}},
+#endif
         {DISKIO_NREADX, ASN_COUNTER64, RONLY, var_diskio, 1, {12}},
         {DISKIO_NWRITTENX, ASN_COUNTER64, RONLY, var_diskio, 1, {13}},
     };
@@ -190,7 +192,7 @@ init_diskio(void)
     IOMasterPort(bootstrap_port, &masterPort);
 #endif
 
-#if defined(aix4) || defined(aix5)
+#if defined(aix4) || defined(aix5) || defined(aix6)
     /*
      * initialize values to gather information on first request
      */
@@ -811,16 +813,16 @@ var_diskio(struct variable * vp,
       *var_len = strlen(head.indices[indx].name);
       return (u_char *) head.indices[indx].name;
     case DISKIO_NREAD:
-      long_ret = head.indices[indx].rsect*512;
+      long_ret = (head.indices[indx].rsect*512) & 0xffffffff;
       return (u_char *) & long_ret;
     case DISKIO_NWRITTEN:
-      long_ret = head.indices[indx].wsect*512;
+      long_ret = (head.indices[indx].wsect*512) & 0xffffffff;
       return (u_char *) & long_ret;
     case DISKIO_READS:
-      long_ret = head.indices[indx].rio;
+      long_ret = head.indices[indx].rio & 0xffffffff;
       return (u_char *) & long_ret;
     case DISKIO_WRITES:
-      long_ret = head.indices[indx].wio;
+      long_ret = head.indices[indx].wio & 0xffffffff;
       return (u_char *) & long_ret;
     case DISKIO_NREADX:
       *var_len = sizeof(struct counter64);
@@ -833,7 +835,7 @@ var_diskio(struct variable * vp,
       c64_ret.high = head.indices[indx].wsect >> (32 - 9);
       return (u_char *) & c64_ret;
     default:
-	snmp_log(LOG_ERR, "diskio.c: don't know how to handle %d request\n", vp->magic);
+	snmp_log(LOG_ERR, "don't know how to handle %d request\n", vp->magic);
   }
   return NULL;
 }
@@ -1067,7 +1069,7 @@ var_diskio(struct variable * vp,
 #endif                          /* darwin */
 
 
-#if defined(aix4) || defined(aix5)
+#if defined(aix4) || defined(aix5) || defined(aix6)
 /*
  * collect statistics for all disks
  */
