@@ -87,7 +87,7 @@ init_memory_hpux(void)
      * Define the OID pointer to the top of the mib tree that we're
      * registering underneath 
      */
-    oid             mem_variables_oid[] = { UCDAVIS_MIB, MEMMIBNUM };
+    oid             mem_variables_oid[] = { NETSNMP_UCDAVIS_MIB, NETSNMP_MEMMIBNUM };
 
     /*
      * register ourselves with the agent to handle our mib tree 
