@@ -22,12 +22,14 @@
 #if HAVE_SYS_WAIT_H
 # include <sys/wait.h>
 #endif
+#ifdef HAVE_LIMITS_H
+#include <limits.h>
+#endif
 #if HAVE_WINSOCK_H
 #include <winsock.h>
 #endif
-
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
+#ifdef WIN32
+#include <limits.h>
 #endif
 
 #include <signal.h>
@@ -86,9 +88,41 @@ void
 pass_persist_parse_config(const char *token, char *cptr)
 {
     struct extensible **ppass = &persistpassthrus, **etmp, *ptmp;
-    char           *tcptr;
-    int             i;
+    char           *tcptr, *endopt;
+    int             i, priority;
 
+    /*
+     * options
+     */
+    priority = DEFAULT_MIB_PRIORITY;
+    while (*cptr == '-') {
+      cptr++;
+      switch (*cptr) {
+      case 'p':
+	/* change priority level */
+	cptr++;
+	cptr = skip_white(cptr);
+	if (! isdigit(*cptr)) {
+	  config_perror("priority must be an integer");
+	  return;
+	}
+	priority = strtol((const char*) cptr, &endopt, 0);
+	if ((priority == LONG_MIN) || (priority == LONG_MAX)) {
+	  config_perror("priority under/overflow");
+	  return;
+	}
+	cptr = endopt;
+	cptr = skip_white(cptr);
+	break;
+      default:
+	config_perror("unknown option for pass directive");
+	return;
+      }
+    }
+
+    /*
+     * MIB
+     */
     if (*cptr == '.')
         cptr++;
     if (!isdigit(*cptr)) {
@@ -108,7 +142,7 @@ pass_persist_parse_config(const char *token, char *cptr)
     while (isdigit(*cptr) || *cptr == '.')
         cptr++;
     /*
-     * name 
+     * path
      */
     cptr = skip_white(cptr);
     if (cptr == NULL) {
@@ -124,10 +158,10 @@ pass_persist_parse_config(const char *token, char *cptr)
     (*ppass)->name[ sizeof((*ppass)->name)-1 ] = 0;
     (*ppass)->next = NULL;
 
-    register_mib("pass_persist",
+    register_mib_priority("pass_persist",
                  (struct variable *) extensible_persist_passthru_variables,
                  sizeof(struct variable2), 1, (*ppass)->miboid,
-                 (*ppass)->miblen);
+                 (*ppass)->miblen, priority);
 
     /*
      * argggg -- pasthrus must be sorted 
@@ -184,6 +218,7 @@ var_extensible_pass_persist(struct variable *vp,
     oid             newname[MAX_OID_LEN];
     int             i, rtest, newlen;
     static long     long_ret;
+    static in_addr_t addr_ret;
     char            buf[SNMP_MAXBUF];
     static char     buf2[SNMP_MAXBUF];
     static oid      objid[MAX_OID_LEN];
@@ -240,6 +275,7 @@ var_extensible_pass_persist(struct variable *vp,
             /*
              * valid call.  Exec and get output 
              */
+		
             if ((file = persist_pipes[i].fIn)) {
                 if (fgets(buf, sizeof(buf), file) == NULL) {
                     *var_len = 0;
@@ -247,7 +283,7 @@ var_extensible_pass_persist(struct variable *vp,
                     return (NULL);
                 }
                 /*
-                 * persistant scripts return "NONE\n" on invalid items 
+                 * persistent scripts return "NONE\n" on invalid items 
                  */
                 if (!strncmp(buf, "NONE", 4)) {
                     if (exact) {
@@ -276,7 +312,6 @@ var_extensible_pass_persist(struct variable *vp,
                     close_persist_pipe(i);
                     return (NULL);
                 }
-
                 /*
                  * buf contains the return type, and buf2 contains the data 
                  */
@@ -332,13 +367,13 @@ var_extensible_pass_persist(struct variable *vp,
                         *var_len = 0;
                         return (NULL);
                     }
-                    long_ret =
+                    addr_ret =
                         (objid[0] << (8 * 3)) + (objid[1] << (8 * 2)) +
                         (objid[2] << 8) + objid[3];
-                    long_ret = htonl(long_ret);
-                    *var_len = sizeof(long_ret);
+                    addr_ret = htonl(addr_ret);
+                    *var_len = sizeof(addr_ret);
                     vp->type = ASN_IPADDRESS;
-                    return ((unsigned char *) &long_ret);
+                    return ((unsigned char *) &addr_ret);
                 }
             }
             *var_len = 0;
@@ -430,7 +465,7 @@ setPassPersist(int action,
                 buf[ sizeof(buf)-1 ] = 0;
                 break;
             case ASN_OBJECT_ID:
-                sprint_mib_oid(buf2, (oid *) var_val, var_val_len);
+                sprint_mib_oid(buf2, (oid *) var_val, var_val_len/sizeof(oid));
                 snprintf(buf, sizeof(buf), "objectid \"%s\"\n", buf2);
                 buf[ sizeof(buf)-1 ] = 0;
                 break;
@@ -462,6 +497,12 @@ setPassPersist(int action,
                 return SNMP_ERR_NOTWRITABLE;
             } else if (!strncasecmp(buf, "wrong-type", 10)) {
                 return SNMP_ERR_WRONGTYPE;
+            } else if (!strncasecmp(buf, "wrong-length", 12)) {
+                return SNMP_ERR_WRONGLENGTH;
+            } else if (!strncasecmp(buf, "wrong-value", 11)) {
+                return SNMP_ERR_WRONGVALUE;
+            } else if (!strncasecmp(buf, "inconsistent-value", 18)) {
+                return SNMP_ERR_INCONSISTENTVALUE;
             }
             return SNMP_ERR_NOERROR;
         }
@@ -485,7 +526,7 @@ pass_persist_compare(const void *a, const void *b)
 }
 
 /*
- * Initialize our persistant pipes
+ * Initialize our persistent pipes
  *   - Returns 1 on success, 0 on failure.
  *   - Initializes all FILE pointers to NULL to indicate "closed"
  */
@@ -518,7 +559,7 @@ init_persist_pipes(void)
 }
 
 /*
- * Destruct our persistant pipes
+ * Destruct our persistent pipes
  *
  */
 static void
@@ -686,12 +727,44 @@ write_persist_pipe(int iindex, const char *data)
         return 0;
     }
 #endif                          /* HAVE_SIGNAL */
+#if defined(WIN32) && !defined (mingw32) && !defined (HAVE_SIGNAL)
+/* We have no signal here (maybe we can make a Thread?) so write may block, 
+ * but probably never will.
+ */
+    int wret = 0, werrno = 0;
+
+    /*
+     * Do the write 
+     */
+    wret = write(persist_pipes[iindex].fdOut, data,strlen(data));
+    werrno = errno;
+    
+    if (wret < 0) {
+      if (werrno != EINTR) {
+        DEBUGMSGTL(("ucd-snmp/pass_persist", "write_persist_pipe: write returned unknown error %d\n",errno));
+      }
+      close_persist_pipe(iindex);
+      return 0;
+    }
+#endif                          /* WIN32 */
     return 1;
 }
 
 static void
 close_persist_pipe(int iindex)
 {
+/*	Alexander Prömel, alexander@proemel.de 08/24/2006
+	The hard coded pathnames, are temporary.
+	I'll fix it soon.
+	If you changed them here, you have to do it in ../util_funcs.c too.
+*/
+#ifdef __uClinux__
+	char fifo_in_path[256];
+	char fifo_out_path[256];
+
+	snprintf(fifo_in_path, 256, "/flash/cp_%d", persist_pipes[iindex].pid);
+	snprintf(fifo_out_path, 256, "/flash/pc_%d", persist_pipes[iindex].pid);
+#endif
 
     /*
      * Check and nix every item 
@@ -712,6 +785,19 @@ close_persist_pipe(int iindex)
         close(persist_pipes[iindex].fdIn);
         persist_pipes[iindex].fdIn = -1;
     }
+
+#if defined(WIN32) && !defined (mingw32) && !defined (HAVE_SIGNAL)
+    if (!CloseHandle((HANDLE)persist_pipes[iindex].pid)) {
+          DEBUGMSGTL(("ucd-snmp/pass_persist","close_persist_pipe pid: close error\n"));
+        } 
+#endif
+    
+#ifdef __uClinux__
+	/*remove the pipes*/
+	unlink(fifo_in_path);
+	unlink(fifo_out_path);
+#endif
+
     if (persist_pipes[iindex].pid != -1) {
 #if HAVE_SYS_WAIT_H
         waitpid(persist_pipes[iindex].pid, 0, 0);
