@@ -97,7 +97,7 @@ init_proc(void)
      * Define the OID pointer to the top of the mib tree that we're
      * registering underneath 
      */
-    oid             proc_variables_oid[] = { UCDAVIS_MIB, PROCMIBNUM, 1 };
+    oid             proc_variables_oid[] = { NETSNMP_UCDAVIS_MIB, NETSNMP_PROCMIBNUM, 1 };
 
     /*
      * register ourselves with the agent to handle our mib tree 
@@ -213,8 +213,8 @@ proc_parse_config(const char *token, char *cptr)
         (*procp)->max = 0;
         (*procp)->min = 0;
     }
-#ifdef PROCFIXCMD
-    sprintf((*procp)->fixcmd, PROCFIXCMD, (*procp)->name);
+#ifdef NETSNMP_PROCFIXCMD
+    sprintf((*procp)->fixcmd, NETSNMP_PROCFIXCMD, (*procp)->name);
 #endif
     DEBUGMSGTL(("ucd-snmp/proc", "Read:  %s (%d) (%d)\n",
                 (*procp)->name, (*procp)->max, (*procp)->min));
@@ -399,7 +399,35 @@ sh_count_procs(char *procname)
     return ret;
 }
 
-#elif OSTYPE == LINUXID
+#elif defined(aix4) || defined(aix5) || defined(aix6)
+#include <procinfo.h>
+#include <sys/types.h>
+
+struct procsinfo pinfo;
+char pinfo_name[256];
+
+int
+sh_count_procs(char *procname)
+{
+    pid_t index;
+    int count;
+    char *sep;
+    
+    index = 0;
+    count = 0;
+
+    while(getprocs(&pinfo, sizeof(pinfo), NULL, 0, &index, 1) == 1) {
+        strncpy(pinfo_name, pinfo.pi_comm, 256);
+        pinfo_name[255] = 0;
+        sep = strchr(pinfo_name, ' ');
+        if(sep != NULL) *sep = 0;
+        if(strcmp(procname, pinfo_name) == 0) count++;
+    }
+
+    return count;
+}
+
+#elif NETSNMP_OSTYPE == NETSNMP_LINUXID
 
 #include <dirent.h>
 #include <fcntl.h>
@@ -410,6 +438,7 @@ sh_count_procs(char *procname)
 {
     DIR *dir;
     char cmdline[512], *tmpc;
+    char state[64];
     struct dirent *ent;
 #ifdef USE_PROC_CMDLINE
     int fd;
@@ -423,25 +452,34 @@ sh_count_procs(char *procname)
 #ifdef USE_PROC_CMDLINE  /* old method */
       /* read /proc/XX/cmdline */
       sprintf(cmdline,"/proc/%s/cmdline",ent->d_name);
-      if((fd = open(cmdline, O_RDONLY)) < 0) break;
+      if((fd = open(cmdline, O_RDONLY)) < 0) continue;
       len = read(fd,cmdline,sizeof(cmdline) - 1);
       close(fd);
       if(len <= 0) continue;
       cmdline[len] = 0;
       while(--len && !cmdline[len]);
+      if(len <= 0) continue;
       while(--len) if(!cmdline[len]) cmdline[len] = ' ';
       if(!strncmp(cmdline,procname,plen)) total++;
 #else
       /* read /proc/XX/status */
       sprintf(cmdline,"/proc/%s/status",ent->d_name);
       if ((status = fopen(cmdline, "r")) == NULL)
-          break;
+          continue;
       if (fgets(cmdline, sizeof(cmdline), status) == NULL) {
           fclose(status);
           break;
       }
+      /* Grab the state of the process as well
+       * (so we can ignore zombie processes)
+       * XXX: Assumes the second line is the status
+       */
+      if (fgets(state, sizeof(state), status) == NULL) {
+          state[0]='\0';
+      }
       fclose(status);
       cmdline[sizeof(cmdline)-1] = '\0';
+      state[sizeof(state)-1] = '\0';
       /* XXX: assumes Name: is first */
       if (strncmp("Name:",cmdline, 5) != 0)
           break;
@@ -456,8 +494,13 @@ sh_count_procs(char *procname)
       DEBUGMSGTL(("proc","Comparing wanted %s against %s\n",
                   procname, tmpc));
       if(len==plen && !strncmp(tmpc,procname,plen)) {
-          total++;
-          DEBUGMSGTL(("proc", " Matched.  total count now=%d\n", total));
+          /* Do not count zombie process as they are not running processes */
+          if ( strstr(state, "zombie") == NULL ) {
+              total++;
+              DEBUGMSGTL(("proc", " Matched.  total count now=%d\n", total));
+          } else {
+              DEBUGMSGTL(("proc", " Skipping zombie process.\n"));
+          }
       }
 #endif      
     }
@@ -465,7 +508,7 @@ sh_count_procs(char *procname)
     return total;
 }
 
-#elif OSTYPE == ULTRIXID
+#elif NETSNMP_OSTYPE == NETSNMP_ULTRIXID
 
 #define	NPROCS		32      /* number of proces to read at once */
 
@@ -646,7 +689,7 @@ getstruct(off_t loc, char *name, off_t dest, int size)
         return (0);
     return (1);
 }
-#elif OSTYPE == SOLARISID
+#elif NETSNMP_OSTYPE == NETSNMP_SOLARISID
 
 #ifdef _SLASH_PROC_METHOD_
 
@@ -758,7 +801,7 @@ sh_count_procs(char *procname)
     char            line[STRMAX], *cptr, *cp;
     int             ret = 0, fd;
     FILE           *file;
-#ifndef EXCACHETIME
+#ifndef NETSNMP_EXCACHETIME
 #endif
     struct extensible ex;
     int             slow = strstr(PSCMD, "ax") != NULL;
@@ -789,7 +832,7 @@ sh_count_procs(char *procname)
                 if (*cp == ':')
                     *cp = 0;
             } else {
-                if ((cptr = find_field(line, LASTFIELD)) == NULL)
+                if ((cptr = find_field(line, NETSNMP_LASTFIELD)) == NULL)
                     continue;
                 copy_nword(cptr, line, sizeof(line));
             }
