@@ -4,7 +4,7 @@
  */
 /*
  * Portions of this file are copyrighted by:
- * Copyright @ 2003 Sun Microsystems, Inc. All rights reserved.
+ * Copyright @ 2009 Sun Microsystems, Inc. All rights reserved.
  * Use is subject to license terms specified in the COPYING file
  * distributed with the Net-SNMP package.
  */
@@ -258,11 +258,11 @@ proxy_fill_in_session(netsnmp_mib_handler *handler,
         return 0;
     }
 
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
-#if defined(DISABLE_SNMPV1)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
+#if defined(NETSNMP_DISABLE_SNMPV1)
     if (session->version == SNMP_VERSION_2c) {
 #else
-#if defined(DISABLE_SNMPV2C)
+#if defined(NETSNMP_DISABLE_SNMPV2C)
     if (session->version == SNMP_VERSION_1) {
 #else
     if (session->version == SNMP_VERSION_1 ||
@@ -408,6 +408,8 @@ proxy_handler(netsnmp_mib_handler *handler,
                 /*
                  * too large 
                  */
+                if (pdu)
+                    snmp_free_pdu(pdu);
                 snmp_log(LOG_ERR,
                          "proxy oid request length is too long\n");
                 return SNMP_ERR_NOERROR;
@@ -437,6 +439,8 @@ proxy_handler(netsnmp_mib_handler *handler,
      */
     if (!proxy_fill_in_session(handler, reqinfo, (void **)&configured)) {
         netsnmp_set_request_error(reqinfo, requests, SNMP_ERR_GENERR);
+        if (pdu)
+            snmp_free_pdu(pdu);
         return SNMP_ERR_NOERROR;
     }
 
@@ -525,8 +529,28 @@ proxy_got_response(int operation, netsnmp_session * sess, int reqid,
                 netsnmp_handler_mark_requests_as_delegated(requests,
                                                            REQUEST_IS_NOT_DELEGATED);
             }
-            else
-                netsnmp_set_request_error(cache->reqinfo, requests, pdu->errstat);
+	    else if ((cache->reqinfo->mode == MODE_SET_ACTION)) {
+		/*
+		 * In order for netsnmp_wrap_up_request to consider the
+		 * SET request complete,
+		 * there must be no delegated requests pending.
+		 * https://sourceforge.net/tracker/
+		 *	?func=detail&atid=112694&aid=1554261&group_id=12694
+		 */
+		DEBUGMSGTL(("proxy",
+		    "got SET error %s, index %d\n",
+		    snmp_errstring(pdu->errstat), pdu->errindex));
+		netsnmp_handler_mark_requests_as_delegated(
+		    requests, REQUEST_IS_NOT_DELEGATED);
+		netsnmp_request_set_error_idx(requests, pdu->errstat,
+                                                        pdu->errindex);
+	    }
+            else {
+		netsnmp_handler_mark_requests_as_delegated( requests,
+                                             REQUEST_IS_NOT_DELEGATED);
+		netsnmp_request_set_error_idx(requests, pdu->errstat,
+                                                        pdu->errindex);
+            }
 
         /*
          * update the original request varbinds with the results 
