@@ -5,10 +5,12 @@
 #ifndef _MIBGROUP_VMSTAT_H
 #define _MIBGROUP_VMSTAT_H
 
+config_require(hardware/cpu)
+
 #include "mibdefs.h"
 
-FindVarMethod   var_extensible_vmstat;
-void            init_vmstat(void);
+Netsnmp_Node_Handler   vmstat_handler;
+void              init_vmstat(void);
 
 #define SWAPIN 3
 #define SWAPOUT 4
