@@ -58,7 +58,7 @@ maybe_source_perl_startup(void)
     return;
 
   bail_out:
-    snmp_log(LOG_ERR, "embedded perl support failed to initalize\n");
+    snmp_log(LOG_ERR, "embedded perl support failed to initialize\n");
     netsnmp_ds_set_boolean(NETSNMP_DS_APPLICATION_ID, 
 			   NETSNMP_DS_AGENT_DISABLE_PERL, 1);
     return;
@@ -77,16 +77,21 @@ do_something_perlish(char *something)
         return;
     }
     DEBUGMSGTL(("perl", "calling perl\n"));
-#ifdef HAVE_EVAL_PV
+#if defined(HAVE_EVAL_PV) || defined(eval_pv)
     /* newer perl */
     eval_pv(something, TRUE);
 #else
-#ifdef HAVE_PERL_EVAL_PV
-    /* older perl */
+#if defined(HAVE_PERL_EVAL_PV_LC) || defined(perl_eval_pv)
+    /* older perl? */
     perl_eval_pv(something, TRUE);
-#else /* !HAVE_PERL_EVAL_PV */
+#else /* HAVE_PERL_EVAL_PV_LC */
+#ifdef HAVE_PERL_EVAL_PV_UC
+    /* older perl? */
+    Perl_eval_pv(my_perl, something, TRUE);
+#else /* !HAVE_PERL_EVAL_PV_UC */
 #error embedded perl broken 
-#endif /* !HAVE_PERL_EVAL_PV */
+#endif /* !HAVE_PERL_EVAL_PV_LC */
+#endif /* !HAVE_PERL_EVAL_PV_UC */
 #endif /* !HAVE_EVAL_PV */
     DEBUGMSGTL(("perl", "finished calling perl\n"));
 }
