@@ -275,8 +275,9 @@ usage(char *prog)
     printf("  -c FILE[,...]\t\tread FILE(s) as configuration file(s)\n");
     printf("  -C\t\t\tdo not read the default configuration files\n");
     printf("  -d\t\t\tdump sent and received SNMP packets\n");
-    printf("  -D TOKEN[,...]\tturn on debugging output for the given TOKEN(s)\n"
-	   "\t\t\t  (try ALL for extremely verbose output)\n");
+    printf("  -DTOKEN[,...]\tturn on debugging output for the given TOKEN(s)\n"
+	   "\t\t\t  (try ALL for extremely verbose output)\n"
+	   "\t\t\t  Don't put space(s) between -D and TOKEN(s).\n");
     printf("  -f\t\t\tdo not fork from the shell\n");
 #if HAVE_UNISTD_H
     printf("  -g GID\t\tchange to this numeric gid after opening\n"
@@ -573,8 +574,26 @@ main(int argc, char *argv[])
 #if HAVE_UNISTD_H
         case 'g':
             if (optarg != NULL) {
+                char           *ecp;
+                int             gid;
+
+                gid = strtoul(optarg, &ecp, 10);
+                if (*ecp) {
+#if HAVE_GETPWNAM && HAVE_PWD_H
+                    struct group  *info;
+                    info = getgrnam(optarg);
+                    if (info) {
+                        gid = info->gr_gid;
+                    } else {
+#endif
+                        fprintf(stderr, "Bad group id: %s\n", optarg);
+                        exit(1);
+#if HAVE_GETPWNAM && HAVE_PWD_H
+                    }
+#endif
+                }
                 netsnmp_ds_set_int(NETSNMP_DS_APPLICATION_ID, 
-				   NETSNMP_DS_AGENT_GROUPID, atoi(optarg));
+				   NETSNMP_DS_AGENT_GROUPID, gid);
             } else {
                 usage(argv[0]);
             }
