@@ -68,6 +68,9 @@ static char *rcsid = "$OpenBSD: route.c,v 1.66 2004/11/17 01:47:20 itojun Exp $"
 
 #include "main.h"
 #include "netstat.h"
+#if HAVE_WINSOCK_H
+#include "winstub.h"
+#endif
 
 #define SET_MASK 0x01
 #define SET_GWAY 0x02
@@ -227,7 +230,7 @@ pr_family(int af)
 	case AF_INET:
 		afname = "Internet";
 		break;
-#ifdef INET6
+#ifdef NETSNMP_ENABLE_IPV6
 	case AF_INET6:
 		afname = "Internet6";
 		break;
@@ -260,7 +263,7 @@ pr_family(int af)
 }
 
 /* column widths; each followed by one space */
-#ifndef INET6
+#ifndef NETSNMP_ENABLE_IPV6
 #define	WID_DST(af)	26	/* width of destination column */
 #define	WID_GW(af)	18	/* width of gateway column */
 #else
@@ -274,7 +277,7 @@ pr_family(int af)
 #define	WID_DST(af)	((af) == AF_INET6 ? (nflag ? 29 : 18) : 18)
 #define	WID_GW(af)	((af) == AF_INET6 ? (nflag ? 25 : 18) : 18)
 #endif
-#endif /* INET6 */
+#endif /* NETSNMP_ENABLE_IPV6 */
 
 /*
  * Print header for routing table columns.
@@ -370,21 +373,21 @@ netname(in_addr_t in, in_addr_t mask)
 	if (cp) {
 		strlcpy(line, cp, sizeof(line));
 	} else if (mbits < 9)
-		snprintf(line, sizeof line, "%u", C(in >> 24));
+		snprintf(line, sizeof line, "%u/%d", C(in >> 24), mbits);
 	else if (mbits < 17)
-		snprintf(line, sizeof line, "%u.%u",
-		    C(in >> 24) , C(in >> 16));
+		snprintf(line, sizeof line, "%u.%u/%d",
+		    C(in >> 24) , C(in >> 16), mbits);
 	else if (mbits < 25)
-		snprintf(line, sizeof line, "%u.%u.%u",
-		    C(in >> 24), C(in >> 16), C(in >> 8));
+		snprintf(line, sizeof line, "%u.%u.%u/%d",
+		    C(in >> 24), C(in >> 16), C(in >> 8), mbits);
 	else
-		snprintf(line, sizeof line, "%u.%u.%u.%u", C(in >> 24),
-		    C(in >> 16), C(in >> 8), C(in));
+		snprintf(line, sizeof line, "%u.%u.%u.%u/%d", C(in >> 24),
+		    C(in >> 16), C(in >> 8), C(in), mbits);
 	return (line);
 }
 
-#undef INET6
-#ifdef INET6
+#undef NETSNMP_ENABLE_IPV6
+#ifdef NETSNMP_ENABLE_IPV6
 char *
 netname6(struct sockaddr_in6 *sa6, struct in6_addr *mask)
 {
@@ -488,7 +491,7 @@ routename6(struct sockaddr_in6 *sa6)
 		strlcpy(line, "", sizeof line);
 	return line;
 }
-#endif /*INET6*/
+#endif /*NETSNMP_ENABLE_IPV6*/
 
 char *
 s_rtflags( struct route_entry *rp )
@@ -497,6 +500,7 @@ s_rtflags( struct route_entry *rp )
     char  *cp = flag_buf;
 
     memset( flag_buf, 0, sizeof(flag_buf));
+    *cp++ = '<';
     *cp++ = 'U';   /* route is in use */
     if (rp->mask  == 0xffffffff)
         *cp++ = 'H';   /* host */
@@ -504,6 +508,7 @@ s_rtflags( struct route_entry *rp )
         *cp++ = 'D';   /* ICMP redirect */
     if (rp->type  == 4)
         *cp++ = 'G';   /* remote destination/net */
+    *cp++ = '>';
     return flag_buf;
 }
 
@@ -521,6 +526,6 @@ p_rtnode( struct route_entry *rp )
                     netname(rp->destination, 0L));
     printf("%-*.*s %-6.6s  %s\n",
 	    WID_GW(af), WID_GW(af),
-                routename(rp->gateway),
+	    rp->gateway ? routename(rp->gateway) : "*",
             s_rtflags(rp), rp->ifname);
 }
