@@ -309,7 +309,7 @@ main(int argc, char *argv[])
         exit(1);
     }
 
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
     if (ss->version == SNMP_VERSION_1)
         use_getbulk = 0;
 #endif
@@ -397,7 +397,7 @@ print_table(void)
             printf("\nSNMP table %s, part %d\n\n", table_name, part);
         first_field = last_field;
         dp = data;
-        if (show_index && !no_headers && !column_width) {
+        if (show_index && !no_headers && !column_width && first_pass) {
             width = index_width;
             printf(index_fmt, "index");
         } else
@@ -455,14 +455,14 @@ print_table(void)
 void
 get_field_names(void)
 {
-    u_char         *buf = NULL, *name_p = NULL;
+    char           *buf = NULL, *name_p = NULL;
     size_t          buf_len = 0, out_len = 0;
-#ifndef DISABLE_MIB_LOADING
+#ifndef NETSNMP_DISABLE_MIB_LOADING
     struct tree    *tbl = NULL;
-#endif /* DISABLE_MIB_LOADING */
+#endif /* NETSNMP_DISABLE_MIB_LOADING */
     int             going = 1;
 
-#ifndef DISABLE_MIB_LOADING
+#ifndef NETSNMP_DISABLE_MIB_LOADING
     tbl = get_tree(root, rootlen, get_tree_head());
     if (tbl) {
         tbl = tbl->child_list;
@@ -474,10 +474,10 @@ get_field_names(void)
             going = 0;
         }
     }
-#endif /* DISABLE_MIB_LOADING */
+#endif /* NETSNMP_DISABLE_MIB_LOADING */
 
     if (sprint_realloc_objid
-        (&buf, &buf_len, &out_len, 1, root, rootlen - 1)) {
+        ((u_char **)&buf, &buf_len, &out_len, 1, root, rootlen - 1)) {
         table_name = buf;
         buf = NULL;
         buf_len = out_len = 0;
@@ -489,7 +489,7 @@ get_field_names(void)
     fields = 0;
     while (going) {
         fields++;
-#ifndef DISABLE_MIB_LOADING
+#ifndef NETSNMP_DISABLE_MIB_LOADING
         if (tbl) {
             if (tbl->access == MIB_ACCESS_NOACCESS) {
                 fields--;
@@ -504,14 +504,14 @@ get_field_names(void)
             if (!tbl)
                 going = 0;
         } else {
-#endif /* DISABLE_MIB_LOADING */
+#endif /* NETSNMP_DISABLE_MIB_LOADING */
             root[rootlen] = fields;
-#ifndef DISABLE_MIB_LOADING
+#ifndef NETSNMP_DISABLE_MIB_LOADING
         }
-#endif /* DISABLE_MIB_LOADING */
+#endif /* NETSNMP_DISABLE_MIB_LOADING */
         out_len = 0;
         if (sprint_realloc_objid
-            (&buf, &buf_len, &out_len, 1, root, rootlen + 1)) {
+            ((u_char **)&buf, &buf_len, &out_len, 1, root, rootlen + 1)) {
             name_p = strrchr(buf, '.');
             if (name_p == NULL) {
                 name_p = strrchr(buf, ':');
@@ -546,15 +546,17 @@ get_field_names(void)
         fprintf(stderr, "Was that a table? %s\n", table_name);
         exit(1);
     }
-    *name_p = 0;
-    memmove(name, root, rootlen * sizeof(oid));
-    name_length = rootlen + 1;
-    name_p = strrchr(buf, '.');
-    if (name_p == NULL) {
-        name_p = strrchr(buf, ':');
-    }
-    if (name_p != NULL) {
+    if (name_p) {
         *name_p = 0;
+        memmove(name, root, rootlen * sizeof(oid));
+        name_length = rootlen + 1;
+        name_p = strrchr(buf, '.');
+        if (name_p == NULL) {
+            name_p = strrchr(buf, ':');
+        }
+        if (name_p != NULL) {
+            *name_p = 0;
+        }
     }
     if (brief && fields > 1) {
         char           *f1, *f2;
@@ -590,7 +592,7 @@ get_table_entries(netsnmp_session * ss)
     int             status;
     int             i;
     int             col;
-    u_char         *buf = NULL;
+    char           *buf = NULL;
     size_t          out_len = 0, buf_len = 0;
     char           *cp;
     char           *name_p = NULL;
@@ -690,7 +692,7 @@ get_table_entries(netsnmp_session * ss)
                                name_length * sizeof(oid));
                         out_len = 0;
                         if (!sprint_realloc_objid
-                            (&buf, &buf_len, &out_len, 1, vars->name,
+                            ((u_char **)&buf, &buf_len, &out_len, 1, vars->name,
                              vars->name_length)) {
                             break;
                         }
@@ -739,7 +741,7 @@ get_table_entries(netsnmp_session * ss)
                         printf("%s => taken\n", buf);
                     }
                     out_len = 0;
-                    sprint_realloc_value(&buf, &buf_len, &out_len, 1,
+                    sprint_realloc_value((u_char **)&buf, &buf_len, &out_len, 1,
                                          vars->name, vars->name_length,
                                          vars);
                     for (cp = buf; *cp; cp++) {
@@ -819,7 +821,7 @@ getbulk_table_entries(netsnmp_session * ss)
     int             status;
     int             i;
     int             row, col;
-    u_char         *buf = NULL;
+    char           *buf = NULL;
     size_t          buf_len = 0, out_len = 0;
     char           *cp;
     char           *name_p = NULL;
@@ -847,7 +849,7 @@ getbulk_table_entries(netsnmp_session * ss)
                 last_var = NULL;
                 while (vars) {
                     out_len = 0;
-                    sprint_realloc_objid(&buf, &buf_len, &out_len, 1,
+                    sprint_realloc_objid((u_char **)&buf, &buf_len, &out_len, 1,
                                          vars->name, vars->name_length);
                     if (vars->type == SNMP_ENDOFMIBVIEW ||
                         memcmp(vars->name, name,
@@ -871,7 +873,7 @@ getbulk_table_entries(netsnmp_session * ss)
                                                   NETSNMP_DS_LIB_OID_OUTPUT_FORMAT)) {
                         case NETSNMP_OID_OUTPUT_MODULE:
 			case 0:
-                            name_p = strrchr(buf, ':');
+                            name_p = strchr(buf, ':')+1;
                             break;
                         case NETSNMP_OID_OUTPUT_SUFFIX:
                             name_p = buf;
@@ -888,7 +890,14 @@ getbulk_table_entries(netsnmp_session * ss)
 					              NETSNMP_DS_LIB_OID_OUTPUT_FORMAT));
 			    exit(1);
                         }
-                        name_p = strchr(name_p, '.') + 1;
+                        name_p = strchr(name_p, '.');
+                        if ( name_p == NULL ) {
+                            /* The 'strchr' call above failed, i.e. the results
+                             * don't seem to include instance subidentifiers! */
+                            running = 0;
+                            break;
+                        }
+                        name_p++;  /* Move on to the instance identifier */
                     }
                     for (row = 0; row < entries; row++)
                         if (strcmp(name_p, indices[row]) == 0)
@@ -930,7 +939,7 @@ getbulk_table_entries(netsnmp_session * ss)
                     }
                     dp = data + row * fields;
                     out_len = 0;
-                    sprint_realloc_value(&buf, &buf_len, &out_len, 1,
+                    sprint_realloc_value((u_char **)&buf, &buf_len, &out_len, 1,
                                          vars->name, vars->name_length,
                                          vars);
                     for (cp = buf; *cp; cp++)
