@@ -52,7 +52,7 @@ SOFTWARE.
 #include <netinet/in.h>
 #endif
 #include <stdio.h>
-#if HAVE_SYS_TIME_H
+#if !defined(mingw32) && defined(HAVE_SYS_TIME_H)
 # include <sys/time.h>
 # if TIME_WITH_SYS_TIME
 #  include <time.h>
@@ -674,6 +674,16 @@ main(int argc, char *argv[])
     extern void init_perl(void);
 #endif
 
+#ifndef WIN32
+    /*
+     * close all non-standard file descriptors we may have
+     * inherited from the shell.
+     */
+    for (i = getdtablesize() - 1; i > 2; --i) {
+        (void) close(i);
+    }
+#endif /* #WIN32 */
+    
 #ifdef SIGTERM
     signal(SIGTERM, term_handler);
 #endif
@@ -1066,6 +1076,8 @@ main(int argc, char *argv[])
         }
     }
 
+    SOCK_STARTUP;
+
     /*
      * I'm being lazy here, and not checking the
      * return value from these registration calls.
@@ -1283,8 +1295,6 @@ main(int argc, char *argv[])
         snmp_log(LOG_WARNING, "-s and -S options are deprecated; use -Ls <facility> instead\n");
     }
     
-    SOCK_STARTUP;
-
     if (listen_ports)
         cp = listen_ports;
     else
@@ -1313,7 +1323,7 @@ main(int argc, char *argv[])
                  */
                 snmptrapd_close_sessions(sess_list);
                 netsnmp_transport_free(transport);
-                snmp_log(LOG_ERR, "couldn't open snmp - %m");
+                snmp_log(LOG_ERR, "couldn't open snmp - %s", strerror(errno));
                 SOCK_CLEANUP;
                 exit(1);
             } else {
