@@ -78,16 +78,32 @@ snmptrapd_parse_traphandle(const char *token, char *line)
     char            buf[STRINGMAX];
     oid             obuf[MAX_OID_LEN];
     size_t          olen = MAX_OID_LEN;
-    char           *cptr;
+    char           *cptr, *cp;
     netsnmp_trapd_handler *traph;
+    int             flags = 0;
 
+    memset( buf, 0, sizeof(buf));
+    memset(obuf, 0, sizeof(obuf));
     cptr = copy_nword(line, buf, sizeof(buf));
+
     DEBUGMSGTL(("read_config:traphandle", "registering handler for: "));
+
     if (!strcmp(buf, "default")) {
         DEBUGMSG(("read_config:traphandle", "default"));
         traph = netsnmp_add_default_traphandler( command_handler );
     } else {
-
+        cp = buf+strlen(buf)-1;
+        if ( *cp == '*' ) {
+            flags |= NETSNMP_TRAPHANDLER_FLAG_MATCH_TREE;
+            *(cp--) = '\0';
+            if ( *cp == '.' ) {
+                /* 
+                 * Distinguish between 'oid.*' & 'oid*'
+                 */
+                flags |= NETSNMP_TRAPHANDLER_FLAG_STRICT_SUBTREE;
+                *(cp--) = '\0';
+            }
+        }
         if (!read_objid(buf, obuf, &olen)) {
             char            buf1[STRINGMAX];
             snprintf(buf1,  sizeof(buf1),
@@ -103,6 +119,7 @@ snmptrapd_parse_traphandle(const char *token, char *line)
     DEBUGMSG(("read_config:traphandle", "\n"));
 
     if (traph) {
+        traph->flags = flags;
         traph->authtypes = TRAP_AUTH_EXE;
         traph->token = strdup(cptr);
     }
@@ -115,15 +132,33 @@ parse_forward(const char *token, char *line)
     char            buf[STRINGMAX];
     oid             obuf[MAX_OID_LEN];
     size_t          olen = MAX_OID_LEN;
-    char           *cptr;
+    char           *cptr, *cp;
     netsnmp_trapd_handler *traph;
+    int             flags = 0;
 
+    memset( buf, 0, sizeof(buf));
+    memset(obuf, 0, sizeof(obuf));
     cptr = copy_nword(line, buf, sizeof(buf));
     DEBUGMSGTL(("read_config:forward", "registering forward for: "));
     if (!strcmp(buf, "default")) {
         DEBUGMSG(("read_config:forward", "default"));
-        traph = netsnmp_add_default_traphandler( forward_handler );
+        if ( !strcmp( cptr, "agentx" ))
+            traph = netsnmp_add_default_traphandler( axforward_handler );
+        else
+            traph = netsnmp_add_default_traphandler( forward_handler );
     } else {
+        cp = buf+strlen(buf)-1;
+        if ( *cp == '*' ) {
+            flags |= NETSNMP_TRAPHANDLER_FLAG_MATCH_TREE;
+            *(cp--) = '\0';
+            if ( *cp == '.' ) {
+                /* 
+                 * Distinguish between 'oid.*' & 'oid*'
+                 */
+                flags |= NETSNMP_TRAPHANDLER_FLAG_STRICT_SUBTREE;
+                *(cp--) = '\0';
+            }
+        }
 
         if (!read_objid(buf, obuf, &olen)) {
             char            buf1[STRINGMAX];
@@ -134,12 +169,16 @@ parse_forward(const char *token, char *line)
             return;
         }
         DEBUGMSGOID(("read_config:forward", obuf, olen));
-        traph = netsnmp_add_traphandler( forward_handler, obuf, olen );
+        if ( !strcmp( cptr, "agentx" ))
+            traph = netsnmp_add_traphandler( axforward_handler, obuf, olen );
+        else
+            traph = netsnmp_add_traphandler( forward_handler, obuf, olen );
     }
 
     DEBUGMSG(("read_config:forward", "\n"));
 
     if (traph) {
+        traph->flags = flags;
         traph->authtypes = TRAP_AUTH_NET;
         traph->token = strdup(cptr);
     }
@@ -167,28 +206,35 @@ parse_format(const char *token, char *line)
         return;
     }
 
-    *cp = '\0';
-    cp++;
+    *(cp++) = '\0';
+    while (*cp && !isspace(*cp))
+        cp++;
 
     /*
      * OK - now "line" contains the format type,
      *      and cp points to the actual format string.
      * So update the appropriate pointer(s).
-     *
-     * XXX - the previous values really need to be freed too
      */
-    if (!strcmp( line, "print1"))
+    if (!strcmp( line, "print1")) {
+        SNMP_FREE( print_format1 );
         print_format1 = strdup(cp);
-    else if (!strcmp( line, "print2"))
+    } else if (!strcmp( line, "print2")) {
+        SNMP_FREE( print_format2 );
         print_format2 = strdup(cp);
-    else if (!strcmp( line, "print")) {
+    } else if (!strcmp( line, "print")) {
+        SNMP_FREE( print_format1 );
+        SNMP_FREE( print_format2 );
         print_format1 = strdup(cp);
         print_format2 = strdup(cp);
-    } else if (!strcmp( line, "syslog1"))
+    } else if (!strcmp( line, "syslog1")) {
+        SNMP_FREE( syslog_format1 );
         syslog_format1 = strdup(cp);
-    else if (!strcmp( line, "syslog2"))
+    } else if (!strcmp( line, "syslog2")) {
+        SNMP_FREE( syslog_format2 );
         syslog_format2 = strdup(cp);
-    else if (!strcmp( line, "syslog")) {
+    } else if (!strcmp( line, "syslog")) {
+        SNMP_FREE( syslog_format1 );
+        SNMP_FREE( syslog_format2 );
         syslog_format1 = strdup(cp);
         syslog_format2 = strdup(cp);
     }
@@ -264,7 +310,7 @@ netsnmp_trapd_handler *netsnmp_specific_traphandlers = NULL;
  * to be applied to *all* incoming traps
  */
 netsnmp_trapd_handler *
-netsnmp_add_global_traphandler(int list, Netsnmp_Trap_Handler handler) {
+netsnmp_add_global_traphandler(int list, Netsnmp_Trap_Handler* handler) {
     netsnmp_trapd_handler *traph;
 
     if ( !handler )
@@ -306,7 +352,7 @@ netsnmp_add_global_traphandler(int list, Netsnmp_Trap_Handler handler) {
  * traps with no specific trap handlers of their own.
  */
 netsnmp_trapd_handler *
-netsnmp_add_default_traphandler( Netsnmp_Trap_Handler handler) {
+netsnmp_add_default_traphandler( Netsnmp_Trap_Handler* handler) {
     netsnmp_trapd_handler *traph;
 
     if ( !handler )
@@ -332,7 +378,7 @@ netsnmp_add_default_traphandler( Netsnmp_Trap_Handler handler) {
  * Register a new trap-specific traphandler
  */
 netsnmp_trapd_handler *
-netsnmp_add_traphandler(Netsnmp_Trap_Handler handler,
+netsnmp_add_traphandler(Netsnmp_Trap_Handler* handler,
                         oid *trapOid, int trapOidLen ) {
     netsnmp_trapd_handler *traph, *traph2;
 
@@ -475,10 +521,39 @@ netsnmp_get_traphandler( oid *trapOid, int trapOidLen ) {
     for (traph = netsnmp_specific_traphandlers;
          traph; traph=traph->nextt ) {
 
-        if ( !snmp_oidtree_compare(traph->trapoid, traph->trapoid_len,
-                                   trapOid, trapOidLen)) {
-            DEBUGMSGTL(( "snmptrapd", "get_traphandler matched (%x)\n", traph));
-	    return traph;
+        /*
+         * If the trap handler wasn't wildcarded, then the trapOID
+         *   should match the registered OID exactly.
+         */
+        if (!(traph->flags & NETSNMP_TRAPHANDLER_FLAG_MATCH_TREE)) {
+            if (snmp_oid_compare(traph->trapoid, traph->trapoid_len,
+                                 trapOid, trapOidLen) == 0) {
+                DEBUGMSGTL(( "snmptrapd", "get_traphandler exact match (%x)\n", traph));
+	        return traph;
+            }
+	} else {
+           /*
+            * If the trap handler *was* wildcarded, then the trapOID
+            *   should have the registered OID as a prefix...
+            */
+            if (snmp_oidsubtree_compare(traph->trapoid,
+                                        traph->trapoid_len,
+                                        trapOid, trapOidLen) == 0) {
+                if (traph->flags & NETSNMP_TRAPHANDLER_FLAG_STRICT_SUBTREE) {
+                    /*
+                     * ... and (optionally) *strictly* as a prefix
+                     *   i.e. not including an exact match.
+                     */
+                    if (snmp_oid_compare(traph->trapoid, traph->trapoid_len,
+                                         trapOid, trapOidLen) != 0) {
+                        DEBUGMSGTL(( "snmptrapd", "get_traphandler strict subtree match (%x)\n", traph));
+	                return traph;
+                    }
+                } else {
+                    DEBUGMSGTL(( "snmptrapd", "get_traphandler subtree match (%x)\n", traph));
+	            return traph;
+                }
+            }
 	}
     }
 
@@ -860,7 +935,7 @@ int   command_handler( netsnmp_pdu           *pdu,
         /*
          *  and pass this formatted string to the command specified
          */
-        run_shell_command(handler->token, rbuf, NULL, 0);   /* Not interested in output */
+        run_shell_command(handler->token, (char*)rbuf, NULL, 0);   /* Not interested in output */
         snmp_set_quick_print(oldquick);
         if (pdu->command == SNMP_MSG_TRAP)
             snmp_free_pdu(v2_pdu);
@@ -889,6 +964,17 @@ int   event_handler( netsnmp_pdu           *pdu,
 }
 
 
+/*
+ *  Trap handler for forwarding to the AgentX master agent
+ */
+int axforward_handler( netsnmp_pdu           *pdu,
+                       netsnmp_transport     *transport,
+                       netsnmp_trapd_handler *handler)
+{
+    send_v2trap( pdu->variables );
+    return NETSNMPTRAPD_HANDLER_OK;
+}
+
 /*
  *  Trap handler for forwarding to another destination
  */
@@ -933,7 +1019,7 @@ int   forward_handler( netsnmp_pdu           *pdu,
     return NETSNMPTRAPD_HANDLER_OK;
 }
 
-#if defined(USING_NOTIFICATION_LOG_MIB_NOTIFICATION_LOG_MODULE) && defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
+#if defined(USING_NOTIFICATION_LOG_MIB_NOTIFICATION_LOG_MODULE) && defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(NETSNMP_SNMPTRAPD_DISABLE_AGENTX)
 /*
  *  "Notification" handler for implementing NOTIFICATION-MIB
  *  		(presumably)
@@ -1074,7 +1160,7 @@ t        *     d) any other global handlers
         traph = netsnmp_auth_global_traphandlers;
         DEBUGMSGTL(("snmptrapd", "Running auth trap handlers\n"));
 	while (traph) {
-            if (!netsnmp_trapd_check_auth(traph->authtypes, pdu)) {
+            if (!netsnmp_trapd_check_auth(traph->authtypes)) {
                 traph = traph->nexth;
                 continue; /* we continue on and skip this one */
             }
@@ -1092,7 +1178,7 @@ t        *     d) any other global handlers
         traph = netsnmp_pre_global_traphandlers;
         DEBUGMSGTL(("snmptrapd", "Running pre-global trap handlers\n"));
 	while (traph) {
-            if (!netsnmp_trapd_check_auth(traph->authtypes, pdu)) {
+            if (!netsnmp_trapd_check_auth(traph->authtypes)) {
                 traph = traph->nexth;
                 continue; /* we continue on and skip this one */
             }
@@ -1110,7 +1196,7 @@ t        *     d) any other global handlers
         DEBUGMSGTL(("snmptrapd", "Running trap specific handlers\n"));
         traph = netsnmp_get_traphandler(trapOid, trapOidLen);
 	while (traph) {
-            if (!netsnmp_trapd_check_auth(traph->authtypes, pdu)) {
+            if (!netsnmp_trapd_check_auth(traph->authtypes)) {
                 traph = traph->nexth;
                 continue; /* we continue on and skip this one */
             }
@@ -1128,7 +1214,7 @@ t        *     d) any other global handlers
         DEBUGMSGTL(("snmptrapd", "Running global handlers\n"));
         traph = netsnmp_post_global_traphandlers;
 	while (traph) {
-            if (!netsnmp_trapd_check_auth(traph->authtypes, pdu)) {
+            if (!netsnmp_trapd_check_auth(traph->authtypes)) {
                 traph = traph->nexth;
                 continue; /* we continue on and skip this one */
             }
@@ -1161,6 +1247,16 @@ t        *     d) any other global handlers
     case NETSNMP_CALLBACK_OP_TIMED_OUT:
         snmp_log(LOG_ERR, "Timeout: This shouldn't happen!\n");
         break;
+
+    case NETSNMP_CALLBACK_OP_SEND_FAILED:
+        snmp_log(LOG_ERR, "Send Failed: This shouldn't happen either!\n");
+        break;
+
+    case NETSNMP_CALLBACK_OP_CONNECT:
+    case NETSNMP_CALLBACK_OP_DISCONNECT:
+        /* Ignore silently */
+        break;
+
     default:
         snmp_log(LOG_ERR, "Unknown operation (%d): This shouldn't happen!\n", op);
         break;
