@@ -1,7 +1,7 @@
 /*
  * interface data access header
  *
- * $Id: interface.h 15205 2006-09-14 17:15:32Z rstory $
+ * $Id: interface.h 18100 2010-02-05 09:48:46Z jsafranek $
  */
 #ifndef NETSNMP_ACCESS_INTERFACE_H
 #define NETSNMP_ACCESS_INTERFACE_H
@@ -71,6 +71,9 @@ extern          "C" {
 
 #endif                          /* ifOperStatus_ENUMS */
 
+/* nominal speed of network interface - used when the real speed is unknown */
+#define NOMINAL_LINK_SPEED 10000000
+
 /**---------------------------------------------------------------------*/
 /*
  * structure definitions
@@ -176,7 +179,7 @@ typedef struct netsnmp_interface_entry_s {
 typedef struct _conf_if_list {
     const char     *name;
     int             type;
-    u_long          speed;
+    uint64_t        speed;
     struct _conf_if_list *next;
 } netsnmp_conf_if_list;
 
