@@ -8,6 +8,12 @@
  * Use is subject to license terms specified in the COPYING file
  * distributed with the Net-SNMP package.
  */
+/*
+ * Note:
+ *    If new default_store entries are added to this header file,
+ *    then remember to run 'perl/default_store/gen' to update the
+ *    corresponding perl interface.
+ */
 /*
  * @file default_store.h: storage space for defaults
  *
@@ -101,10 +107,10 @@ extern          "C" {
     /*
      * special meanings for the default SNMP version slot (NETSNMP_DS_LIB_SNMPVERSION) 
      */
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
 #define NETSNMP_DS_SNMP_VERSION_1    128        /* bogus */
 #endif
-#ifndef DISABLE_SNMPV2C
+#ifndef NETSNMP_DISABLE_SNMPV2C
 #define NETSNMP_DS_SNMP_VERSION_2c   1  /* real */
 #endif
 #define NETSNMP_DS_SNMP_VERSION_3    3  /* real */
