@@ -1,7 +1,7 @@
 #ifndef _SNMPSTDDOMAIN_H
 #define _SNMPSTDDOMAIN_H
 
-#ifdef SNMP_TRANSPORT_STD_DOMAIN
+#ifdef NETSNMP_TRANSPORT_STD_DOMAIN
 
 #ifdef __cplusplus
 extern          "C" {
@@ -25,7 +25,8 @@ extern oid netsnmp_snmpSTDDomain[];
     } netsnmp_std_data;
     
     netsnmp_transport *netsnmp_std_transport(const char *instring,
-                                             size_t instring_len);
+                                             size_t instring_len,
+                                             const char *default_target);
 
     /*
      * "Constructor" for transport domain object.  
@@ -36,6 +37,6 @@ extern oid netsnmp_snmpSTDDomain[];
 #ifdef __cplusplus
 }
 #endif
-#endif                          /*SNMP_TRANSPORT_STD_DOMAIN */
+#endif                          /*NETSNMP_TRANSPORT_STD_DOMAIN */
 
 #endif/*_SNMPSTDDOMAIN_H*/
