@@ -301,6 +301,9 @@ struct snmp_session {
 
     /**
      * use as you want data 
+     *
+     *     used by 'SNMP_FLAGS_RESP_CALLBACK' handling in the agent
+     * XXX: or should we add a new field into this structure?
      */
     void           *myvoid;
 };
@@ -394,6 +397,7 @@ typedef struct request_list {
 
 #define SNMP_DETAIL_SIZE        512
 
+#define SNMP_FLAGS_RESP_CALLBACK   0x400      /* Additional callback on response */
 #define SNMP_FLAGS_USER_CREATED    0x200      /* USM user has been created */
 #define SNMP_FLAGS_DONT_PROBE      0x100      /* don't probe for an engineID */
 #define SNMP_FLAGS_STREAM_SOCKET   0x80
@@ -528,7 +532,7 @@ struct variable_list {
     netsnmp_vardata val;
    /** the length of the value to be copied into buf */
    size_t          val_len;
-   /** 90 percentile < 24. */
+   /** buffer to hold the OID */
    oid             name_loc[MAX_OID_LEN];  
    /** 90 percentile < 40. */
    u_char          buf[40];
@@ -733,6 +737,7 @@ struct variable_list {
                                        size_t);
     int             netsnmp_oid_tree_equals(const oid *, size_t, const oid *,
                                             size_t);
+    NETSNMP_IMPORT
     int             netsnmp_oid_is_subtree(const oid *, size_t, const oid *,
                                            size_t);
     int             netsnmp_oid_find_prefix(const oid * in_name1, size_t len1,
