@@ -5,6 +5,7 @@
 extern          "C" {
 #endif
 
+#include <stdio.h>
 #if HAVE_SYSLOG_H
 #include <syslog.h>
 #endif
@@ -35,6 +36,7 @@ extern          "C" {
 #endif
 
     void            init_snmp_logging(void);
+    void            shutdown_snmp_logging(void);
     int             snmp_get_do_logging(void);
     void            snmp_disable_syslog(void);
     void            snmp_disable_filelog(void);
