@@ -1,4 +1,7 @@
-#include "generic.h"
+#include <net-snmp/system/generic.h>
+
+/* got socklen_t? */
+#define HAVE_SOCKLEN_T 1
 
 #ifdef HAVE_STDINT_H
 #include <stdint.h>	/* uint32_t */
