@@ -1,6 +1,7 @@
 package NetSNMP::OID;
 
 use strict;
+use warnings;
 use Carp;
 
 require Exporter;
@@ -91,7 +92,7 @@ use vars qw(@ISA %EXPORT_TAGS @EXPORT_OK @EXPORT $VERSION $AUTOLOAD);
 	snmp_oid_compare
         compare
 );
-$VERSION = '5.04011';
+$VERSION = '5.0403';
 
 sub new {
     my $type = shift;
@@ -141,6 +142,8 @@ sub to_array($) {
     return $self->{oidptr}->to_array();
 }
 
+sub DESTROY {}
+
 sub AUTOLOAD {
     # This AUTOLOAD is used to 'autoload' constants from the constant()
     # XS function.  If a constant is not found then control is passed
