@@ -1,6 +1,7 @@
 package NetSNMP::agent;
 
 use strict;
+use warnings;
 use Carp;
 
 require Exporter;
@@ -86,7 +87,7 @@ use vars qw(@ISA %EXPORT_TAGS @EXPORT_OK @EXPORT $VERSION $AUTOLOAD);
 	SNMP_ERR_AUTHORIZATIONERROR
 	SNMP_ERR_NOTWRITABLE
 );
-$VERSION = '5.0203';
+$VERSION = '5.0403';
 
 sub AUTOLOAD {
     # This AUTOLOAD is used to 'autoload' constants from the constant()
@@ -234,6 +235,7 @@ module implements in a perl-way, and a perl specific tutorial at:
 =head2 Sub-agent example
 
     	use NetSNMP::agent (':all');
+    	use NetSNMP::ASN qw(ASN_OCTET_STR);
 
         my $value = "hello world";
 	sub myhandler {
@@ -270,8 +272,6 @@ module implements in a perl-way, and a perl specific tutorial at:
     				'Name' => "my_agent_name",
     				'AgentX' => 1
     				);
-	}
-
     	$agent->register("my_agent_name", ".1.3.6.1.4.1.8072.9999.9999.7375",
                          \&myhandler);
 
