@@ -21,20 +21,26 @@
 
 #ifndef HAVE_INET_PTON
 
+#include <ctype.h>
+
 #if HAVE_ARPA_NAMESER_H
 #include <arpa/nameser.h>
 #endif
 
-  /*
-   * Net-SNMP Win32 additions
-   */
-#if defined(HAVE_WINSOCK_H) || defined(cygwin)
+#if defined(HAVE_WINSOCK_H)
 #include <winsock2.h>
 #include <ws2tcpip.h>
+#endif
 #include <errno.h>
 #include <stdio.h>
+#if HAVE_STRING_H
+#include <string.h>
+#else
+#include <strings.h>
 #endif
 
+#include <net-snmp/types.h>
+
 #ifndef EAFNOSUPPORT
 #define EAFNOSUPPORT            WSAEAFNOSUPPORT
 #endif
@@ -60,7 +66,7 @@
  */
 
 static int	inet_pton4(const char *src, u_char *dst, int pton);
-#ifdef INET6
+#ifdef NETSNMP_ENABLE_IPV6
 static int	inet_pton6(const char *src, u_char *dst);
 #endif
 
@@ -85,7 +91,7 @@ inet_pton(af, src, dst)
 	switch (af) {
 	case AF_INET:
 		return (inet_pton4(src, dst, 1));
-#ifdef INET6
+#ifdef NETSNMP_ENABLE_IPV6
 	case AF_INET6:
 		return (inet_pton6(src, dst));
 #endif
@@ -217,7 +223,7 @@ inet_pton4(src, dst, pton)
 	return (1);
 }
 
-#ifdef INET6
+#ifdef NETSNMP_ENABLE_IPV6
 /* int
  * inet_pton6(src, dst)
  *	convert presentation level address to network order binary form.
