@@ -41,15 +41,15 @@
 #include <net-snmp/utilities.h>
 
 #include <net-snmp/library/snmp_api.h>
-#ifdef USE_OPENSSL
+#ifdef NETSNMP_USE_OPENSSL
 #	include <openssl/hmac.h>
 #else
-#ifdef USE_INTERNAL_MD5
+#ifdef NETSNMP_USE_INTERNAL_MD5
 #include <net-snmp/library/md5.h>
 #endif
 #endif
 
-#ifdef USE_PKCS
+#ifdef NETSNMP_USE_PKCS11
 #include <security/cryptoki.h>
 #endif
 
@@ -94,7 +94,7 @@
 int
 generate_Ku(const oid * hashtype, u_int hashtype_len,
             u_char * P, size_t pplen, u_char * Ku, size_t * kulen)
-#if defined(USE_INTERNAL_MD5) || defined(USE_OPENSSL)
+#if defined(NETSNMP_USE_INTERNAL_MD5) || defined(NETSNMP_USE_OPENSSL)
 {
     int             rval = SNMPERR_SUCCESS,
         nbytes = USM_LENGTH_EXPANDED_PASSPHRASE;
@@ -103,8 +103,9 @@ generate_Ku(const oid * hashtype, u_int hashtype_len,
 
     u_char          buf[USM_LENGTH_KU_HASHBLOCK], *bufp;
 
-#ifdef USE_OPENSSL
-    EVP_MD_CTX     *ctx = malloc(sizeof(EVP_MD_CTX));
+#ifdef NETSNMP_USE_OPENSSL
+    EVP_MD_CTX     *ctx = (EVP_MD_CTX *)malloc(sizeof(EVP_MD_CTX));
+    unsigned int    tmp_len;
 #else
     MDstruct        MD;
 #endif
@@ -127,9 +128,9 @@ generate_Ku(const oid * hashtype, u_int hashtype_len,
     /*
      * Setup for the transform type.
      */
-#ifdef USE_OPENSSL
+#ifdef NETSNMP_USE_OPENSSL
 
-#ifndef DISABLE_MD5
+#ifndef NETSNMP_DISABLE_MD5
     if (ISTRANSFORM(hashtype, HMACMD5Auth))
         EVP_DigestInit(ctx, EVP_md5());
     else
@@ -142,31 +143,33 @@ generate_Ku(const oid * hashtype, u_int hashtype_len,
     }
 #else
     MDbegin(&MD);
-#endif                          /* USE_OPENSSL */
+#endif                          /* NETSNMP_USE_OPENSSL */
 
     while (nbytes > 0) {
         bufp = buf;
         for (i = 0; i < USM_LENGTH_KU_HASHBLOCK; i++) {
             *bufp++ = P[pindex++ % pplen];
         }
-#ifdef USE_OPENSSL
+#ifdef NETSNMP_USE_OPENSSL
         EVP_DigestUpdate(ctx, buf, USM_LENGTH_KU_HASHBLOCK);
-#elif USE_INTERNAL_MD5
+#elif NETSNMP_USE_INTERNAL_MD5
         if (MDupdate(&MD, buf, USM_LENGTH_KU_HASHBLOCK * 8)) {
             rval = SNMPERR_USM_ENCRYPTIONERROR;
             goto md5_fin;
         }
-#endif                          /* USE_OPENSSL */
+#endif                          /* NETSNMP_USE_OPENSSL */
 
         nbytes -= USM_LENGTH_KU_HASHBLOCK;
     }
 
-#ifdef USE_OPENSSL
-    EVP_DigestFinal(ctx, (unsigned char *) Ku, (unsigned int *) kulen);
+#ifdef NETSNMP_USE_OPENSSL
+    tmp_len = *kulen;
+    EVP_DigestFinal(ctx, (unsigned char *) Ku, &tmp_len);
+    *kulen = tmp_len;
     /*
      * what about free() 
      */
-#elif USE_INTERNAL_MD5
+#elif NETSNMP_USE_INTERNAL_MD5
     if (MDupdate(&MD, buf, 0)) {
         rval = SNMPERR_USM_ENCRYPTIONERROR;
         goto md5_fin;
@@ -175,26 +178,26 @@ generate_Ku(const oid * hashtype, u_int hashtype_len,
     MDget(&MD, Ku, *kulen);
   md5_fin:
     memset(&MD, 0, sizeof(MD));
-#endif                          /* USE_INTERNAL_MD5 */
+#endif                          /* NETSNMP_USE_INTERNAL_MD5 */
 
 
-#ifdef SNMP_TESTING_CODE
+#ifdef NETSNMP_ENABLE_TESTING_CODE
     DEBUGMSGTL(("generate_Ku", "generating Ku (from %s): ", P));
     for (i = 0; i < *kulen; i++)
         DEBUGMSG(("generate_Ku", "%02x", Ku[i]));
     DEBUGMSG(("generate_Ku", "\n"));
-#endif                          /* SNMP_TESTING_CODE */
+#endif                          /* NETSNMP_ENABLE_TESTING_CODE */
 
 
   generate_Ku_quit:
     memset(buf, 0, sizeof(buf));
-#ifdef USE_OPENSSL
+#ifdef NETSNMP_USE_OPENSSL
     free(ctx);
 #endif
     return rval;
 
 }                               /* end generate_Ku() */
-#elif USE_PKCS
+#elif NETSNMP_USE_PKCS11
 {
     int             rval = SNMPERR_SUCCESS;
 
@@ -217,7 +220,7 @@ generate_Ku(const oid * hashtype, u_int hashtype_len,
      * Setup for the transform type.
      */
 
-#ifndef DISABLE_MD5
+#ifndef NETSNMP_DISABLE_MD5
     if (ISTRANSFORM(hashtype, HMACMD5Auth))
         return pkcs_generate_Ku(CKM_MD5, P, pplen, Ku, kulen);
     else
@@ -278,14 +281,15 @@ generate_kul(const oid * hashtype, u_int hashtype_len,
              u_char * engineID, size_t engineID_len,
              u_char * Ku, size_t ku_len,
              u_char * Kul, size_t * kul_len)
-#if defined(USE_OPENSSL) || defined(USE_INTERNAL_MD5) || defined(USE_PKCS)
+#if defined(NETSNMP_USE_OPENSSL) || defined(NETSNMP_USE_INTERNAL_MD5) || defined(NETSNMP_USE_PKCS11)
 {
     int             rval = SNMPERR_SUCCESS;
     u_int           nbytes = 0;
     size_t          properlength;
+    int             iproperlength;
 
     u_char          buf[SNMP_MAXBUF];
-#ifdef SNMP_TESTING_CODE
+#ifdef NETSNMP_ENABLE_TESTING_CODE
     int             i;
 #endif
 
@@ -300,10 +304,11 @@ generate_kul(const oid * hashtype, u_int hashtype_len,
     }
 
 
-    properlength = sc_get_properlength(hashtype, hashtype_len);
-    if (properlength == SNMPERR_GENERR)
+    iproperlength = sc_get_properlength(hashtype, hashtype_len);
+    if (iproperlength == SNMPERR_GENERR)
         QUITFUN(SNMPERR_GENERR, generate_kul_quit);
 
+    properlength = (size_t) iproperlength;
 
     if (((int) *kul_len < properlength) || ((int) ku_len < properlength)) {
         QUITFUN(SNMPERR_GENERR, generate_kul_quit);
@@ -323,12 +328,12 @@ generate_kul(const oid * hashtype, u_int hashtype_len,
 
     rval = sc_hash(hashtype, hashtype_len, buf, nbytes, Kul, kul_len);
 
-#ifdef SNMP_TESTING_CODE
+#ifdef NETSNMP_ENABLE_TESTING_CODE
     DEBUGMSGTL(("generate_kul", "generating Kul (from Ku): "));
     for (i = 0; i < *kul_len; i++)
         DEBUGMSG(("generate_kul", "%02x", Kul[i]));
     DEBUGMSG(("generate_kul", "keytools\n"));
-#endif                          /* SNMP_TESTING_CODE */
+#endif                          /* NETSNMP_ENABLE_TESTING_CODE */
 
     QUITFUN(rval, generate_kul_quit);
 
@@ -384,7 +389,7 @@ encode_keychange(const oid * hashtype, u_int hashtype_len,
                  u_char * oldkey, size_t oldkey_len,
                  u_char * newkey, size_t newkey_len,
                  u_char * kcstring, size_t * kcstring_len)
-#if defined(USE_OPENSSL) || defined(USE_INTERNAL_MD5) || defined(USE_PKCS)
+#if defined(NETSNMP_USE_OPENSSL) || defined(NETSNMP_USE_INTERNAL_MD5) || defined(NETSNMP_USE_PKCS11)
 {
     int             rval = SNMPERR_SUCCESS;
     size_t          properlength;
@@ -396,6 +401,9 @@ encode_keychange(const oid * hashtype, u_int hashtype_len,
     /*
      * Sanity check.
      */
+    if (!kcstring || !kcstring_len)
+	return SNMPERR_GENERR;
+
     if (!hashtype || !oldkey || !newkey || !kcstring || !kcstring_len
         || (oldkey_len <= 0) || (newkey_len <= 0) || (*kcstring_len <= 0)
         || (hashtype_len != USM_LENGTH_OID_TRANSFORM)) {
@@ -426,18 +434,18 @@ encode_keychange(const oid * hashtype, u_int hashtype_len,
      */
     nbytes = properlength;
 
-#if defined(SNMP_TESTING_CODE) && defined(RANDOMZEROS)
+#if defined(NETSNMP_ENABLE_TESTING_CODE) && defined(RANDOMZEROS)
     memset(kcstring, 0, nbytes);
     DEBUGMSG(("encode_keychange",
               "** Using all zero bits for \"random\" delta of )"
               "the keychange string! **\n"));
-#else                           /* !SNMP_TESTING_CODE */
+#else                           /* !NETSNMP_ENABLE_TESTING_CODE */
     rval = sc_random(kcstring, &nbytes);
     QUITFUN(rval, encode_keychange_quit);
     if ((int) nbytes != properlength) {
         QUITFUN(SNMPERR_GENERR, encode_keychange_quit);
     }
-#endif                          /* !SNMP_TESTING_CODE */
+#endif                          /* !NETSNMP_ENABLE_TESTING_CODE */
 
     tmpbuf = (u_char *) malloc(properlength * 2);
     if (tmpbuf) {
@@ -509,10 +517,11 @@ decode_keychange(const oid * hashtype, u_int hashtype_len,
                  u_char * oldkey, size_t oldkey_len,
                  u_char * kcstring, size_t kcstring_len,
                  u_char * newkey, size_t * newkey_len)
-#if defined(USE_OPENSSL) || defined(USE_INTERNAL_MD5) || defined(USE_PKCS)
+#if defined(NETSNMP_USE_OPENSSL) || defined(NETSNMP_USE_INTERNAL_MD5) || defined(NETSNMP_USE_PKCS11)
 {
     int             rval = SNMPERR_SUCCESS;
     size_t          properlength = 0;
+    int             iproperlength = 0;
     u_int           nbytes = 0;
 
     u_char         *bufp, tmp_buf[SNMP_MAXBUF];
@@ -534,10 +543,11 @@ decode_keychange(const oid * hashtype, u_int hashtype_len,
     /*
      * Setup for the transform type.
      */
-    properlength = sc_get_properlength(hashtype, hashtype_len);
-    if (properlength == SNMPERR_GENERR)
+    iproperlength = sc_get_properlength(hashtype, hashtype_len);
+    if (iproperlength == SNMPERR_GENERR)
         QUITFUN(SNMPERR_GENERR, decode_keychange_quit);
 
+    properlength = (size_t) iproperlength;
 
     if (((oldkey_len * 2) != kcstring_len) || (*newkey_len < oldkey_len)) {
         QUITFUN(SNMPERR_GENERR, decode_keychange_quit);
