@@ -171,7 +171,8 @@ internal_register_config_handler(const char *type_param,
                 *cptr = '\0';
                 ++cptr;
             }
-            ltmp2 = register_config_handler(c, token, parser, releaser, help);
+            ltmp2 = internal_register_config_handler(c, token, parser,
+                                                     releaser, help, when);
         }
         return ltmp2;
     }
@@ -257,11 +258,11 @@ register_app_prenetsnmp_mib_handler(const char *token,
  * management of where to put tokens as the module or modules get more complex
  * in regard to handling token registrations.
  *
- * @param type_param the configuration file used, e.g., if snmp.conf is the
+ * @param type     the configuration file used, e.g., if snmp.conf is the
  *                 file where the token is located use "snmp" here.
  *                 Multiple colon separated tokens might be used.
  *                 If NULL or "" then the configuration file used will be
- *                 <application>.conf.
+ *                 \<application\>.conf.
  *
  * @param token    the token being parsed from the file.  Must be non-NULL.
  *
@@ -1050,7 +1051,7 @@ read_config_files_in_path(const char *path, struct config_files *ctmp,
                 ctmp->fileHeader, envconfpath, perspath));
     cptr1 = cptr2 = envconfpath;
     done = 0;
-    while ((*cptr2 != 0) && (!done)) {
+    while ((!done) && (*cptr2 != 0)) {
         while (*cptr1 != 0 && *cptr1 != ENV_SEPARATOR_CHAR)
             cptr1++;
         if (*cptr1 == 0)
@@ -1161,7 +1162,8 @@ read_config_files_in_path(const char *path, struct config_files *ctmp,
 void
 read_config_files(int when)
 {
-    const char     *confpath, *perspath, *persfile, *envconfpath;
+    const char     *confpath, *persfile, *envconfpath;
+    char           *perspath;
     struct config_files *ctmp = config_files;
 
     if (netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
@@ -1187,9 +1189,11 @@ read_config_files(int when)
     for (; ctmp != NULL; ctmp = ctmp->next) {
 
         /*
-         * read the config files 
+         * read the config files. strdup() the result of
+         * get_persistent_directory() to avoid that parsing the "persistentDir"
+         * keyword transforms the perspath pointer into a dangling pointer.
          */
-        perspath = get_persistent_directory();
+        perspath = strdup(get_persistent_directory());
         if (envconfpath == NULL) {
             /*
              * read just the config files (no persistent stuff), since
@@ -1198,7 +1202,8 @@ read_config_files(int when)
              */
             read_config_files_in_path(confpath, ctmp, when, perspath,
                                       persfile);
-            perspath = get_persistent_directory();
+            free(perspath);
+            perspath = strdup(get_persistent_directory());
             read_config_files_in_path(perspath, ctmp, when, perspath,
                                       persfile);
         }
@@ -1209,6 +1214,7 @@ read_config_files(int when)
             read_config_files_in_path(envconfpath, ctmp, when, perspath,
                                       persfile);
         }
+        free(perspath);
     }
 
     if (config_errors) {
@@ -1467,7 +1473,7 @@ skip_white(char *ptr)
 {
     if (ptr == NULL)
         return (NULL);
-    while (*ptr != 0 && isspace(*ptr))
+    while (*ptr != 0 && isspace((unsigned char)*ptr))
         ptr++;
     if (*ptr == 0 || *ptr == '#')
         return (NULL);
@@ -1479,7 +1485,7 @@ skip_not_white(char *ptr)
 {
     if (ptr == NULL)
         return (NULL);
-    while (*ptr != 0 && !isspace(*ptr))
+    while (*ptr != 0 && !isspace((unsigned char)*ptr))
         ptr++;
     if (*ptr == 0 || *ptr == '#')
         return (NULL);
