@@ -51,7 +51,7 @@
 #define NETSNMP_STREAM_QUEUE_LEN  5
 #endif
 
-#ifdef SNMP_TRANSPORT_CALLBACK_DOMAIN
+#ifdef NETSNMP_TRANSPORT_CALLBACK_DOMAIN
 
 static netsnmp_transport_list *trlist = NULL;
 
@@ -189,7 +189,7 @@ netsnmp_callback_recv(netsnmp_transport *t, void *buf, int size,
 
     while (rc < 0) {
 #ifdef WIN32
-	rc = recv(mystuff->pipefds[0], newbuf, 1, 0);
+	rc = recvfrom(mystuff->pipefds[0], newbuf, 1, 0, NULL, 0);
 #else
 	rc = read(mystuff->pipefds[0], newbuf, 1);
 #endif
@@ -272,7 +272,7 @@ netsnmp_callback_send(netsnmp_transport *t, void *buf, int size,
 
 	while (rc < 0) {
 #ifdef WIN32
-	    rc = send(((netsnmp_callback_info*) other_side->data)->pipefds[1], " ", 1, 0);
+	    rc = sendto(((netsnmp_callback_info*) other_side->data)->pipefds[1], " ", 1, 0, NULL, 0);
 #else
 	    rc = write(((netsnmp_callback_info *)other_side->data)->pipefds[1],
 		       " ", 1);
@@ -309,7 +309,7 @@ netsnmp_callback_send(netsnmp_transport *t, void *buf, int size,
         }
 	while (rc < 0) {
 #ifdef WIN32
-	    rc = send(((netsnmp_callback_info*) other_side->data)->pipefds[1], " ", 1, 0);
+	    rc = sendto(((netsnmp_callback_info*) other_side->data)->pipefds[1], " ", 1, 0, NULL, 0);
 #else
 	    rc = write(((netsnmp_callback_info *)other_side->data)->pipefds[1],
 		       " ", 1);
@@ -480,13 +480,13 @@ netsnmp_callback_hook_build(netsnmp_session * sp,
      * Copy missing values from session defaults
      */
     switch (pdu->version) {
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
     case SNMP_VERSION_1:
 #endif
-#ifndef DISABLE_SNMPV2C
+#ifndef NETSNMP_DISABLE_SNMPV2C
     case SNMP_VERSION_2c:
 #endif
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
         if (pdu->community_len == 0) {
             if (sp->community_len == 0) {
                 sp->s_snmp_errno = SNMPERR_BAD_COMMUNITY;
@@ -505,7 +505,7 @@ netsnmp_callback_hook_build(netsnmp_session * sp,
 #endif
     case SNMP_VERSION_3:
         if (pdu->securityNameLen == 0) {
-            pdu->securityName = (u_char *) malloc(sp->securityNameLen);
+	  pdu->securityName = (char *)malloc(sp->securityNameLen);
             if (pdu->securityName == NULL) {
                 sp->s_snmp_errno = SNMPERR_MALLOC;
                 return -1;
@@ -611,8 +611,8 @@ netsnmp_clear_callback_list(void)
 
 	if (tr != NULL) {
 	    tr->f_close(tr);
-  	    netsnmp_transport_remove_from_list(&trlist, list->transport);
-	    netsnmp_transport_free(list->transport);
+  	    netsnmp_transport_remove_from_list(&trlist, tr);
+	    netsnmp_transport_free(tr);
 	}
 	list = next;
     }
@@ -620,4 +620,4 @@ netsnmp_clear_callback_list(void)
 
 }
 
-#endif /* SNMP_TRANSPORT_CALLBACK_DOMAIN */
+#endif /* NETSNMP_TRANSPORT_CALLBACK_DOMAIN */
