@@ -2,21 +2,6 @@
 
 #ifdef NETSNMP_TRANSPORT_TCPIPV6_DOMAIN
 
-/*
- * hack-o-matic for Cygwin to use winsock2
-*/
-#if defined(cygwin)
-#undef HAVE_UNISTD_H
-#undef HAVE_NETINET_IN_H
-#undef HAVE_ARPA_INET_H
-#undef HAVE_NET_IF_H
-#undef HAVE_NETDB_H
-#undef HAVE_SYS_PARAM_H
-#undef HAVE_SYS_SELECT_H
-#undef HAVE_SYS_SOCKET_H
-#undef HAVE_IN_ADDR_T
-#endif
-
 #include <stdio.h>
 #include <sys/types.h>
 #include <errno.h>
@@ -36,7 +21,7 @@
 #include <sys/socket.h>
 #endif
 
-#if defined(HAVE_WINSOCK_H) || defined(cygwin)
+#if defined(HAVE_WINSOCK_H)
     /*
      * Windows IPv6 support is part of WinSock2 only
      */
@@ -97,9 +82,9 @@ netsnmp_tcp6_fmtaddr(netsnmp_transport *t, void *data, int len)
         return strdup("TCP/IPv6: unknown");
     } else {
         char addr[INET6_ADDRSTRLEN];
-        char tmp[INET6_ADDRSTRLEN + 8];
+        char tmp[INET6_ADDRSTRLEN + 18];
 
-        sprintf(tmp, "TCP/IPv6: [%s]:%hd",
+        sprintf(tmp, "TCP/IPv6: [%s]:%hu",
                 inet_ntop(AF_INET6, (void *) &(to->sin6_addr), addr,
                           INET6_ADDRSTRLEN), ntohs(to->sin6_port));
         return strdup(tmp);
@@ -120,7 +105,7 @@ netsnmp_tcp6_recv(netsnmp_transport *t, void *buf, int size,
 
     if (t != NULL && t->sock >= 0) {
 	while (rc < 0) {
-	    rc = recv(t->sock, buf, size, 0);
+	    rc = recvfrom(t->sock, buf, size, 0, NULL, 0);
 	    if (rc < 0 && errno != EINTR) {
 		DEBUGMSGTL(("netsnmp_tcp6", "recv fd %d err %d (\"%s\")\n",
 			    t->sock, errno, strerror(errno)));
@@ -157,7 +142,7 @@ netsnmp_tcp6_send(netsnmp_transport *t, void *buf, int size,
 
     if (t != NULL && t->sock >= 0) {
 	while (rc < 0) {
-	    rc = send(t->sock, buf, size, 0);
+	    rc = sendto(t->sock, buf, size, 0, NULL, 0);
 	    if (rc < 0 && errno != EINTR) {
 		break;
 	    }
