@@ -2,21 +2,6 @@
 
 #ifdef NETSNMP_TRANSPORT_UDPIPV6_DOMAIN
 
-/*
- * hack-o-matic for Cygwin to use winsock2
-*/
-#if defined(cygwin)
-#undef HAVE_UNISTD_H
-#undef HAVE_NETINET_IN_H
-#undef HAVE_ARPA_INET_H
-#undef HAVE_NET_IF_H
-#undef HAVE_NETDB_H
-#undef HAVE_SYS_PARAM_H
-#undef HAVE_SYS_SELECT_H
-#undef HAVE_SYS_SOCKET_H
-#undef HAVE_IN_ADDR_T
-#endif
-
 #include <stdio.h>
 #include <sys/types.h>
 #include <ctype.h>
@@ -37,7 +22,7 @@
 #include <sys/socket.h>
 #endif
 
-#if defined(HAVE_WINSOCK_H) || defined(cygwin)
+#if defined(HAVE_WINSOCK_H)
     /*
      *  Windows IPv6 support is part of WinSock2 only
      */
@@ -110,7 +95,7 @@ netsnmp_udp6_fmtaddr(netsnmp_transport *t, void *data, int len)
         return strdup("UDP/IPv6: unknown");
     } else {
         char addr[INET6_ADDRSTRLEN];
-        char tmp[INET6_ADDRSTRLEN + 8];
+        char tmp[INET6_ADDRSTRLEN + 18];
 
         sprintf(tmp, "UDP/IPv6: [%s]:%hu",
                 inet_ntop(AF_INET6, (void *) &(to->sin6_addr), addr,
@@ -395,7 +380,7 @@ netsnmp_sockaddr_in6_2(struct sockaddr_in6 *addr,
             return 0;
         }
 
-        for (cp = peername; *cp && isdigit((int) *cp); cp++);
+        for (cp = peername; *cp && isdigit((unsigned char) *cp); cp++);
         if (!*cp && atoi(peername) != 0) {
             /*
              * Okay, it looks like JUST a port number.  
@@ -535,6 +520,13 @@ netsnmp_sockaddr_in6_2(struct sockaddr_in6 *addr,
              * Fall through.  
              */
         }
+
+        if (peername[0] == '\0') {
+          DEBUGMSGTL(("netsnmp_sockaddr_in6", "empty hostname\n"));
+          free(peername);
+          return 0;
+        }
+
 #if HAVE_GETADDRINFO
         memset(&hint, 0, sizeof hint);
         hint.ai_flags = 0;
@@ -545,10 +537,11 @@ netsnmp_sockaddr_in6_2(struct sockaddr_in6 *addr,
         err = getaddrinfo(peername, NULL, &hint, &addrs);
         if (err != 0) {
 #if HAVE_GAI_STRERROR
-            snmp_log(LOG_ERR, "getaddrinfo: %s %s\n", peername,
+            snmp_log(LOG_ERR, "getaddrinfo(\"%s\", NULL, ...): %s\n", peername,
                      gai_strerror(err));
 #else
-            snmp_log(LOG_ERR, "getaddrinfo: %s (error %d)\n", peername, err);
+            snmp_log(LOG_ERR, "getaddrinfo(\"%s\", NULL, ...): (error %d)\n",
+                     peername, err);
 #endif
             free(peername);
             return 0;
@@ -682,7 +675,11 @@ inet_make_mask_addr(int pf, void *dst, int masklength)
             mask |= maskbit;
             maskbit >>= 1;
         }
-        (*(u_char *) (&((struct in6_addr *) dst)->s6_addr[j])) = mask;
+
+	if (j < sizeof (((struct in6_addr *) dst)->s6_addr)){
+	    (*(u_char *) (&((struct in6_addr *) dst)->s6_addr[j])) = mask;
+	}
+
         break;
     default:
         return -1;              /* unsupported protocol family */
@@ -1058,39 +1055,39 @@ netsnmp_udp6_parse_security(const char *token, char *param)
      * Deal with the network part first.  
      */
     if ((strcmp(source, "default") == 0) || (strcmp(source, "::") == 0)) {
-        strnetwork = strdup("0::0");
-        strmask = strdup("0::0");
-
-        inet_pton(AF_INET6, strnetwork, &net.sin6_addr);
-        inet_pton(AF_INET6, strmask, &mask.sin6_addr);
-
-        e = (com2Sec6Entry *) malloc(sizeof(com2Sec6Entry));
-        if (e == NULL) {
-            config_perror("memory error");
-            return;
-        }
-        /*
-         * Everything is okay.  Copy the parameters to the structure allocated
-         * above and add it to END of the list.  
-         */
-        if (strmask != NULL && strnetwork != NULL) {
-            DEBUGMSGTL(("netsnmp_udp6_parse_security",
-                        "<\"%s\", %s/%s> => \"%s\"\n", community,
-                        strnetwork, strmask, secName));
-            free(strmask);
-            free(strnetwork);
-        } else {
-            DEBUGMSGTL(("netsnmp_udp6_parse_security",
-                        "Couldn't allocate enough memory\n"));
-        }
-        memmove_com2Sec6Entry(e, secName, community, net, mask, contextName);
-        if (com2Sec6ListLast != NULL) {
-            com2Sec6ListLast->next = e;
-            com2Sec6ListLast = e;
-        } else {
-            com2Sec6ListLast = com2Sec6List = e;
-        }
-
+        if ((strnetwork = strdup("0::0")) != NULL)
+	{
+	    if ((strmask = strdup("0::0")) != NULL)
+	    {
+	
+		inet_pton(AF_INET6, strnetwork, &net.sin6_addr);
+		inet_pton(AF_INET6, strmask, &mask.sin6_addr);
+		
+		e = (com2Sec6Entry *) malloc(sizeof(com2Sec6Entry));
+		if (e != NULL) {
+		    memmove_com2Sec6Entry(e, secName, community, net, mask, contextName);
+		    if (com2Sec6ListLast != NULL) {
+			com2Sec6ListLast->next = e;
+			com2Sec6ListLast = e;
+		    } else {
+			com2Sec6ListLast = com2Sec6List = e;
+		    }
+		}
+		else {
+		    config_perror ("memory error");
+		}
+		free (strmask);
+	    }
+	    else {
+		DEBUGMSGTL(("netsnmp_udp6_parse_security",
+			    "Couldn't allocate enough memory\n"));
+	    }
+	    free (strnetwork);
+	}
+	else {
+	    DEBUGMSGTL(("netsnmp_udp6_parse_security",
+			"Couldn't allocate enough memory\n"));
+	}
     } else {
         /*
          * Try interpreting as IPv6 address.  
@@ -1175,8 +1172,8 @@ netsnmp_udp6_parse_security(const char *token, char *param)
                      0, NI_NUMERICHOST)) {
                     config_perror("getnameinfo failed");
                 }
-                memmove(ai->ai_addr, &net, sizeof(struct sockaddr_in6));
-                inet_make_mask_addr(AF_INET6, &mask.sin6_addr, 127);
+                memmove(&net, ai->ai_addr, sizeof(struct sockaddr_in6));
+                inet_make_mask_addr(PF_INET6, &mask.sin6_addr, 128);
 
                 e = (com2Sec6Entry *) malloc(sizeof(com2Sec6Entry));
                 if (e == NULL) {
@@ -1294,7 +1291,7 @@ netsnmp_udp6_getSecName(void *opaque, int olength,
 
     for (c = com2Sec6List; c != NULL; c = c->next) {
         DEBUGMSGTL(("netsnmp_udp6_getSecName",
-                    "compare <\"%s\", 0x%032/0x%032x>", c->community,
+                    "compare <\"%s\", 0x%032x/0x%032x>", c->community,
                     c->network, c->mask));
 
         if ((community_len == (int)strlen(c->community)) &&
