@@ -32,6 +32,7 @@
 #include <net-snmp/config_api.h>
 
 #include <net-snmp/library/snmp_transport.h>
+#include <net-snmp/library/snmpUDPDomain.h>
 #include <net-snmp/library/snmpUnixDomain.h>
 
 
@@ -133,7 +134,7 @@ netsnmp_unix_recv(netsnmp_transport *t, void *buf, int size,
             return -1;
         };
         while (rc < 0) {
-            rc = recv(t->sock, buf, size, 0);
+            rc = recvfrom(t->sock, buf, size, 0, NULL, 0);
             if (rc < 0 && errno != EINTR) {
                 DEBUGMSGTL(("netsnmp_unix", "recv fd %d err %d (\"%s\")\n",
                             t->sock, errno, strerror(errno)));
@@ -159,7 +160,7 @@ netsnmp_unix_send(netsnmp_transport *t, void *buf, int size,
         DEBUGMSGTL(("netsnmp_unix", "send %d bytes to %p on fd %d\n",
                     size, buf, t->sock));
         while (rc < 0) {
-            rc = send(t->sock, buf, size, 0);
+            rc = sendto(t->sock, buf, size, 0, NULL, 0);
             if (rc < 0 && errno != EINTR) {
                 break;
             }
@@ -242,6 +243,8 @@ netsnmp_unix_accept(netsnmp_transport *t)
                     farend, farendlen));
         t->data = farend;
         t->data_length = sizeof(struct sockaddr_un);
+       netsnmp_sock_buffer_set(newsock, SO_SNDBUF, 1, 0);
+       netsnmp_sock_buffer_set(newsock, SO_RCVBUF, 1, 0);
         return newsock;
     } else {
         free(farend);
@@ -264,7 +267,7 @@ netsnmp_unix_transport(struct sockaddr_un *addr, int local)
     netsnmp_transport *t = NULL;
     sockaddr_un_pair *sup = NULL;
     int             rc = 0;
-    char           *str = NULL;
+    char           *string = NULL;
 
     if (addr == NULL || addr->sun_family != AF_UNIX) {
         return NULL;
@@ -275,11 +278,11 @@ netsnmp_unix_transport(struct sockaddr_un *addr, int local)
         return NULL;
     }
 
-    str = netsnmp_unix_fmtaddr(NULL, (void *)addr,
+    string = netsnmp_unix_fmtaddr(NULL, (void *)addr,
                                   sizeof(struct sockaddr_un));
     DEBUGMSGTL(("netsnmp_unix", "open %s %s\n", local ? "local" : "remote",
-                str));
-    free(str);
+                string));
+    free(string);
 
     memset(t, 0, sizeof(netsnmp_transport));
 
@@ -305,7 +308,7 @@ netsnmp_unix_transport(struct sockaddr_un *addr, int local)
     t->flags = NETSNMP_TRANSPORT_FLAG_STREAM;
 
     if (local) {
-        t->local = malloc(strlen(addr->sun_path));
+      t->local = (u_char *)malloc(strlen(addr->sun_path));
         if (t->local == NULL) {
             netsnmp_transport_free(t);
             return NULL;
@@ -355,7 +358,7 @@ netsnmp_unix_transport(struct sockaddr_un *addr, int local)
         }
 
     } else {
-        t->remote = malloc(strlen(addr->sun_path));
+      t->remote = (u_char *)malloc(strlen(addr->sun_path));
         if (t->remote == NULL) {
             netsnmp_transport_free(t);
             return NULL;
@@ -382,6 +385,8 @@ netsnmp_unix_transport(struct sockaddr_un *addr, int local)
         sup->server.sun_family = AF_UNIX;
         strcpy(sup->server.sun_path, addr->sun_path);
         sup->local = 0;
+       netsnmp_sock_buffer_set(t->sock, SO_SNDBUF, local, 0);
+       netsnmp_sock_buffer_set(t->sock, SO_RCVBUF, local, 0);
     }
 
     /*
@@ -400,17 +405,24 @@ netsnmp_unix_transport(struct sockaddr_un *addr, int local)
 }
 
 netsnmp_transport *
-netsnmp_unix_create_tstring(const char *str, int local)
+netsnmp_unix_create_tstring(const char *string, int local,
+			    const char *default_target)
 {
     struct sockaddr_un addr;
 
-    if ((str != NULL) && (strlen(str) < sizeof(addr.sun_path))) {
+    if (string && *string != '\0') {
+    } else if (default_target && *default_target != '\0') {
+      string = default_target;
+    }
+
+    if ((string != NULL && *string != '\0') &&
+	(strlen(string) < sizeof(addr.sun_path))) {
         addr.sun_family = AF_UNIX;
         memset(addr.sun_path, 0, sizeof(addr.sun_path));
-        strncpy(addr.sun_path, str, sizeof(addr.sun_path) - 1);
+        strncpy(addr.sun_path, string, sizeof(addr.sun_path) - 1);
         return netsnmp_unix_transport(&addr, local);
     } else {
-        if (str != NULL) {
+        if (string != NULL && *string != '\0') {
             snmp_log(LOG_ERR, "Path too long for Unix domain transport\n");
         }
         return NULL;
@@ -427,7 +439,7 @@ netsnmp_unix_create_ostring(const u_char * o, size_t o_len, int local)
     if (o_len > 0 && o_len < (sizeof(addr.sun_path) - 1)) {
         addr.sun_family = AF_UNIX;
         memset(addr.sun_path, 0, sizeof(addr.sun_path));
-        strncpy(addr.sun_path, o, o_len);
+        strncpy(addr.sun_path, (const char *)o, o_len);
         return netsnmp_unix_transport(&addr, local);
     } else {
         if (o_len > 0) {
@@ -444,21 +456,21 @@ netsnmp_unix_ctor(void)
 {
     unixDomain.name = netsnmp_UnixDomain;
     unixDomain.name_length = sizeof(netsnmp_UnixDomain) / sizeof(oid);
-    unixDomain.prefix = calloc(2, sizeof(char *));
+    unixDomain.prefix = (const char**)calloc(2, sizeof(char *));
     unixDomain.prefix[0] = "unix";
 
-    unixDomain.f_create_from_tstring = netsnmp_unix_create_tstring;
+    unixDomain.f_create_from_tstring_new = netsnmp_unix_create_tstring;
     unixDomain.f_create_from_ostring = netsnmp_unix_create_ostring;
 
     netsnmp_tdomain_register(&unixDomain);
 }
 
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
 /* support for SNMPv1 and SNMPv2c on unix domain*/
 
 #define EXAMPLE_COMMUNITY "COMMUNITY"
 typedef struct _com2SecUnixEntry {
-    char            community[VACMSTRINGLEN];
+    char            community[COMMUNITY_MAX_LEN];
     char            sockpath[sizeof(struct sockaddr_un)];
     unsigned long   pathlen;
     char            secName[VACMSTRINGLEN];
@@ -543,7 +555,7 @@ netsnmp_unix_getSecName(void *opaque, int olength,
 void
 netsnmp_unix_parse_security(const char *token, char *param)
 {
-    char              secName[VACMSTRINGLEN + 1], community[VACMSTRINGLEN + 1];
+    char              secName[VACMSTRINGLEN + 1], community[COMMUNITY_MAX_LEN + 1];
     char              contextName[VACMSTRINGLEN + 1];
     char              sockpath[sizeof(struct sockaddr_un) + 1];
     com2SecUnixEntry *e = NULL;
@@ -551,12 +563,12 @@ netsnmp_unix_parse_security(const char *token, char *param)
 
     param = copy_nword(param, secName, VACMSTRINGLEN);
     if (strcmp(secName, "-Cn") == 0) {
-        if (!secName) {
+        param = copy_nword( param, contextName, sizeof(contextName));
+        param = copy_nword( param, secName, sizeof(secName));
+        if (contextName[0] == '\0') {
             config_perror("missing CONTEXT_NAME parameter");
             return;
         }
-        param = copy_nword( param, contextName, sizeof(contextName));
-        param = copy_nword( param, secName, sizeof(secName));
     } else {
         contextName[0] = '\0';
     }
@@ -581,7 +593,7 @@ netsnmp_unix_parse_security(const char *token, char *param)
         sockpath[0] = 0;
     }
 
-    param = copy_nword(param, community, VACMSTRINGLEN);
+    param = copy_nword(param, community, COMMUNITY_MAX_LEN);
     if (community[0] == '\0') {
         config_perror("missing COMMUNITY parameter\n");
         return;
@@ -590,7 +602,7 @@ netsnmp_unix_parse_security(const char *token, char *param)
                == 0) {
         config_perror("example config COMMUNITY not properly configured");
         return;
-    } else if (strlen(community) > (VACMSTRINGLEN - 1)) {
+    } else if (strlen(community) > (COMMUNITY_MAX_LEN - 1)) {
         config_perror("community name too long");
         return;
     }
@@ -635,7 +647,7 @@ netsnmp_unix_com2SecList_free(void)
 void
 netsnmp_unix_agent_config_tokens_register(void)
 {
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
     register_app_config_handler("com2secunix", netsnmp_unix_parse_security,
                                 netsnmp_unix_com2SecList_free,
                                 "[-Cn CONTEXT] secName sockpath community");
