@@ -89,12 +89,6 @@ init_snmp_alarm(void)
 void
 sa_update_entry(struct snmp_alarm *a)
 {
-    if (a->t.tv_sec == 0 && a->t.tv_usec == 0) {
-        DEBUGMSGTL(("snmp_alarm",
-                    "update_entry: illegal interval specified\n"));
-        return;
-    }
-
     if (a->t_last.tv_sec == 0 && a->t_last.tv_usec == 0) {
         struct timeval  t_now;
         /*
@@ -117,6 +111,13 @@ sa_update_entry(struct snmp_alarm *a)
          * We've been called but not reset for the next call.  
          */
         if (a->flags & SA_REPEAT) {
+            if (a->t.tv_sec == 0 && a->t.tv_usec == 0) {
+                DEBUGMSGTL(("snmp_alarm",
+                            "update_entry: illegal interval specified\n"));
+                snmp_alarm_unregister(a->clientreg);
+                return;
+            }
+
             a->t_next.tv_sec = a->t_last.tv_sec + a->t.tv_sec;
             a->t_next.tv_usec = a->t_last.tv_usec + a->t.tv_usec;
 
@@ -196,8 +197,18 @@ struct snmp_alarm *
 sa_find_next(void)
 {
     struct snmp_alarm *a, *lowest = NULL;
+    struct timeval  t_now;
+
+    gettimeofday(&t_now, NULL);
 
     for (a = thealarms; a != NULL; a = a->next) {
+        /* check for time delta skew */
+        if ((a->t_next.tv_sec - t_now.tv_sec) > a->t.tv_sec)
+        {
+            DEBUGMSGTL(("time_skew", "Time delta too big (%d seconds), should be %d seconds - fixing\n", (a->t_next.tv_sec - t_now.tv_sec), a->t.tv_sec));
+            a->t_next.tv_sec = t_now.tv_sec + a->t.tv_sec;
+            a->t_next.tv_usec = t_now.tv_usec + a->t.tv_usec;
+        }
         if (lowest == NULL) {
             lowest = a;
         } else if (a->t_next.tv_sec == lowest->t_next.tv_sec) {
@@ -369,7 +380,7 @@ set_an_alarm(void)
 
 
 /**
- * This function registers function callbacks to occur at a speciifc time
+ * This function registers function callbacks to occur at a specific time
  * in the future.
  *
  * @param when is an unsigned integer specifying when the callback function
@@ -415,8 +426,13 @@ snmp_alarm_register(unsigned int when, unsigned int flags,
     if (*sa_pptr == NULL)
         return 0;
 
-    (*sa_pptr)->t.tv_sec = when;
-    (*sa_pptr)->t.tv_usec = 0;
+    if (0 == when) {
+        (*sa_pptr)->t.tv_sec = 0;
+        (*sa_pptr)->t.tv_usec = 1;
+    } else {
+        (*sa_pptr)->t.tv_sec = when;
+        (*sa_pptr)->t.tv_usec = 0;
+    }
     (*sa_pptr)->flags = flags;
     (*sa_pptr)->clientarg = clientarg;
     (*sa_pptr)->thecallback = thecallback;
