@@ -580,7 +580,8 @@ snmp_split_pdu(netsnmp_pdu *pdu, int skip_count, int copy_count)
  * The command is set to the input command and the reqid, errstat, and
  * errindex are set to default values.
  * If the error status didn't indicate an error, the error index didn't
- * indicate a variable, the pdu wasn't a get response message, or there
+ * indicate a variable, the pdu wasn't a get response message, the
+ * marked variable was not present in the initial request, or there
  * would be no remaining variables, this function will return 0.
  * If everything was successful, a pointer to the fixed cloned pdu will
  * be returned.
@@ -593,6 +594,7 @@ snmp_fix_pdu(netsnmp_pdu *pdu, int command)
     if ((pdu->command != SNMP_MSG_RESPONSE)
         || (pdu->errstat == SNMP_ERR_NOERROR)
         || (0 == pdu->variables)
+        || (pdu->errindex > snmp_varbind_len(pdu))
         || (pdu->errindex <= 0)) {
         return 0;               /* pre-condition tests fail */
     }
@@ -642,8 +644,7 @@ snmp_set_var_objid(netsnmp_variable_list * vp,
 {
     size_t          len = sizeof(oid) * name_length;
 
-    if (vp->name != vp->name_loc && vp->name != NULL &&
-        vp->name_length > (sizeof(vp->name_loc) / sizeof(oid))) {
+    if (vp->name != vp->name_loc && vp->name != NULL) {
         /*
          * Probably previously-allocated "big storage".  Better free it
          * else memory leaks possible.  
@@ -1014,7 +1015,7 @@ snmp_synch_response_cb(netsnmp_session * ss,
         count = select(numfds, &fdset, 0, 0, tvp);
         if (count > 0) {
             snmp_read(&fdset);
-        } else
+        } else {
             switch (count) {
             case 0:
                 snmp_timeout();
@@ -1040,6 +1041,14 @@ snmp_synch_response_cb(netsnmp_session * ss,
                 state->status = STAT_ERROR;
                 state->waiting = 0;
             }
+        }
+
+        if ( ss->flags & SNMP_FLAGS_RESP_CALLBACK ) {
+            void (*cb)(void);
+            cb = ss->myvoid;
+            cb();        /* Used to invoke 'netsnmp_check_outstanding_agent_requests();'
+                            on internal AgentX queries.  */
+        }
     }
     *response = state->pdu;
     ss->callback = cbsav;
@@ -1127,31 +1136,31 @@ snmp_sess_synch_response(void *sessp,
 }
 
 
-const char     *error_string[19] = {
-    "(noError) No Error",
-    "(tooBig) Response message would have been too large.",
-    "(noSuchName) There is no such variable name in this MIB.",
-    "(badValue) The value given has the wrong type or length.",
-    "(readOnly) The two parties used do not have access to use the specified SNMP PDU.",
-    "(genError) A general failure occured",
-    "noAccess",
-    "wrongType (The set datatype does not match the data type the agent expects)",
-    "wrongLength (The set value has an illegal length from what the agent expects)",
-    "wrongEncoding",
-    "wrongValue (The set value is illegal or unsupported in some way)",
-    "noCreation (That table does not support row creation or that object can not ever be created)",
-    "inconsistentValue (The set value is illegal or unsupported in some way)",
-    "resourceUnavailable (This is likely a out-of-memory failure within the agent)",
-    "commitFailed",
-    "undoFailed",
-    "authorizationError (access denied to that object)",
-    "notWritable (That object does not support modification)",
-    "inconsistentName (That object can not currently be created)"
-};
-
 const char     *
 snmp_errstring(int errstat)
 {
+    const char * const error_string[19] = {
+        "(noError) No Error",
+        "(tooBig) Response message would have been too large.",
+        "(noSuchName) There is no such variable name in this MIB.",
+        "(badValue) The value given has the wrong type or length.",
+        "(readOnly) The two parties used do not have access to use the specified SNMP PDU.",
+        "(genError) A general failure occured",
+        "noAccess",
+        "wrongType (The set datatype does not match the data type the agent expects)",
+        "wrongLength (The set value has an illegal length from what the agent expects)",
+        "wrongEncoding",
+        "wrongValue (The set value is illegal or unsupported in some way)",
+        "noCreation (That table does not support row creation or that object can not ever be created)",
+        "inconsistentValue (The set value is illegal or unsupported in some way)",
+        "resourceUnavailable (This is likely a out-of-memory failure within the agent)",
+        "commitFailed",
+        "undoFailed",
+        "authorizationError (access denied to that object)",
+        "notWritable (That object does not support modification)",
+        "inconsistentName (That object can not currently be created)"
+    };
+
     if (errstat <= MAX_SNMP_ERR && errstat >= SNMP_ERR_NOERROR) {
         return error_string[errstat];
     } else {
