@@ -86,6 +86,17 @@
 #include <net-snmp/library/callback.h>
 #define LOGLENGTH 1024
 
+#ifdef va_copy
+#define NEED_VA_END_AFTER_VA_COPY
+#else
+#ifdef __vacopy
+#define vacopy __vacopy
+#define NEED_VA_END_AFTER_VA_COPY
+#else
+#define va_copy(dest, src) memcpy (&dest, &src, sizeof (va_list))
+#endif
+#endif
+
 /*
  * logh_head:  A list of all log handlers, in increasing order of priority
  * logh_priorities:  'Indexes' into this list, by priority
@@ -115,6 +126,14 @@ init_snmp_logging(void)
 			 NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_LOG_TIMESTAMP);
 }
 
+void
+shutdown_snmp_logging(void)
+{
+   snmp_disable_log();
+   while(NULL != logh_head)
+      netsnmp_remove_loghandler( logh_head );
+}
+
 /*
  * These definitions handle 4.2 systems without additional syslog facilities.
  */
@@ -155,12 +174,21 @@ init_snmp_logging(void)
 #define LOG_USER	0
 #endif
 
+/*
+ * Decodes log priority.
+ * @param optarg - IN - priority to decode, "0" or "0-7"
+ *                 OUT - points to last character after the decoded priority
+ * @param pri_max - OUT - maximum priority (i.e. 0x7 from "0-7")
+ */
 int
-decode_priority( char *optarg, int *pri_max )
+decode_priority( char **optarg, int *pri_max )
 {
     int pri_low = LOG_DEBUG;
 
-    switch (*optarg) {
+    if (*optarg == NULL)
+        return -1;
+
+    switch (**optarg) {
         case '0': 
         case '!': 
             pri_low = LOG_EMERG;
@@ -201,13 +229,21 @@ decode_priority( char *optarg, int *pri_max )
             pri_low = LOG_DEBUG;
             break;
         default: 
-            fprintf(stderr, "invalid priority: %c\n",*optarg);
+            fprintf(stderr, "invalid priority: %c\n",**optarg);
             return -1;
     }
+    *optarg = *optarg+1;
 
-    if (pri_max && *(optarg+1)=='-') {
-        *pri_max = decode_priority( optarg+2, NULL );
+    if (pri_max && **optarg=='-') {
+        *optarg = *optarg + 1; /* skip '-' */
+        *pri_max = decode_priority( optarg, NULL );
         if (*pri_max == -1) return -1;
+        if (pri_low < *pri_max) { 
+            int tmp = pri_low; 
+            pri_low = *pri_max; 
+            *pri_max = tmp; 
+        }
+
     }
     return pri_low;
 }
@@ -303,7 +339,7 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
      * Log to Standard Error
      */
     case 'E':
-        priority = decode_priority( optarg, &pri_max );
+        priority = decode_priority( &optarg, &pri_max );
         if (priority == -1)  return -1;
         if (inc_optind)
             optind++;
@@ -320,7 +356,7 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
      * Log to Standard Output
      */
     case 'O':
-        priority = decode_priority( optarg, &pri_max );
+        priority = decode_priority( &optarg, &pri_max );
         if (priority == -1)  return -1;
         if (inc_optind)
             optind++;
@@ -338,8 +374,8 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
      * Log to a named file
      */
     case 'F':
-        priority = decode_priority( optarg, &pri_max );
-        if (priority == -1)  return -1;
+        priority = decode_priority( &optarg, &pri_max );
+        if (priority == -1 || !argv)  return -1;
         optarg = argv[++optind];
         /* Fallthrough */
     case 'f':
@@ -363,9 +399,15 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
      * Log to syslog
      */
     case 'S':
-        priority = decode_priority( optarg, &pri_max );
-        if (priority == -1)  return -1;
-        optarg = argv[++optind];
+        priority = decode_priority( &optarg, &pri_max );
+        if (priority == -1 || !argv)  return -1;
+        if (!optarg[0]) {
+            /* The command line argument with priority does not contain log
+             * facility. The facility must be in next argument then. */
+            optind++;
+            if (optind < argc)
+                optarg = argv[optind];
+        }
         /* Fallthrough */
     case 's':
         if (inc_optind)
@@ -380,7 +422,7 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
             if (facility == -1)  return -1;
             logh->pri_max = pri_max;
             logh->token   = strdup(snmp_log_syslogname(0));
-            logh->magic   = (void *)facility;
+            logh->magic   = (void *)(intptr_t)facility;
 	    snmp_enable_syslog_ident(snmp_log_syslogname(0), facility);
 	}
         break;
@@ -389,7 +431,7 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
      * Don't log 
      */
     case 'N':
-        priority = decode_priority( optarg, &pri_max );
+        priority = decode_priority( &optarg, &pri_max );
         if (priority == -1)  return -1;
         if (inc_optind)
             optind++;
@@ -600,7 +642,7 @@ netsnmp_logging_restart(void)
             continue;
         if (logh->type == NETSNMP_LOGHANDLER_SYSLOG) {
             snmp_disable_syslog_entry(logh);
-            snmp_enable_syslog_ident(logh->token,(int)logh->magic);
+            snmp_enable_syslog_ident(logh->token,(int)(intptr_t)logh->magic);
         }
         else if (logh->type == NETSNMP_LOGHANDLER_FILE) {
             snmp_disable_filelog_entry(logh);
@@ -755,8 +797,8 @@ snmp_enable_calllog(void)	/* XXX - or take a callback routine ??? */
     netsnmp_log_handler *logh;
     int                  found = 0;
 
-    for (logh = logh_head; logh; logh = logh->next) {
-        if (logh->type == NETSNMP_LOGHANDLER_CALLBACK)
+    for (logh = logh_head; logh; logh = logh->next)
+        if (logh->type == NETSNMP_LOGHANDLER_CALLBACK) {
             logh->enabled = 1;
             found         = 1;
 	}
@@ -907,6 +949,7 @@ netsnmp_disable_loghandler( const char *token )
 int
 netsnmp_remove_loghandler( netsnmp_log_handler *logh )
 {
+    int i;
     if (!logh)
         return 0;
 
@@ -917,6 +960,12 @@ netsnmp_remove_loghandler( netsnmp_log_handler *logh )
 
     if (logh->next)
         logh->next->prev = logh->prev;
+
+    for (i=LOG_EMERG; i<=logh->priority; i++)
+        logh_priorities[i] = NULL;
+    SNMP_FREE(logh->token);
+    SNMP_FREE(logh);
+
     return 1;
 }
 
@@ -1016,7 +1065,7 @@ log_handler_syslog(  netsnmp_log_handler* logh, int pri, const char *str)
 	 */
     if (!(logh->imagic)) {
         const char *ident    = logh->token;
-        int   facility = (int)logh->magic;
+        int   facility = (int)(intptr_t)logh->magic;
         if (!ident)
             ident = netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID,
                                           NETSNMP_DS_LIB_APPTYPE);
@@ -1167,19 +1216,32 @@ snmp_vlog(int priority, const char *format, va_list ap)
     char            buffer[LOGLENGTH];
     int             length;
     char           *dynamic;
+    va_list         aq;
 
+    va_copy(aq, ap);
     length = vsnprintf(buffer, LOGLENGTH, format, ap);
+    va_end(ap);
 
-    if (length == 0)
+    if (length == 0) {
+#ifdef NEED_VA_END_AFTER_VA_COPY
+        va_end(aq);
+#endif
         return (0);             /* Empty string */
+    }
 
     if (length == -1) {
         snmp_log_string(LOG_ERR, "Could not format log-string\n");
+#ifdef NEED_VA_END_AFTER_VA_COPY
+        va_end(aq);
+#endif
         return (-1);
     }
 
     if (length < LOGLENGTH) {
         snmp_log_string(priority, buffer);
+#ifdef NEED_VA_END_AFTER_VA_COPY
+        va_end(aq);
+#endif
         return (0);
     }
 
@@ -1188,12 +1250,16 @@ snmp_vlog(int priority, const char *format, va_list ap)
         snmp_log_string(LOG_ERR,
                         "Could not allocate memory for log-message\n");
         snmp_log_string(priority, buffer);
+#ifdef NEED_VA_END_AFTER_VA_COPY
+        va_end(aq);
+#endif
         return (-2);
     }
 
-    vsnprintf(dynamic, length + 1, format, ap);
+    vsnprintf(dynamic, length + 1, format, aq);
     snmp_log_string(priority, dynamic);
     free(dynamic);
+    va_end(aq);
     return 0;
 }
 
