@@ -276,6 +276,7 @@ snmp_parse_args(int argc,
             break;
 
         case 'M':
+            netsnmp_get_mib_directory(); /* prepare the default directories */
             netsnmp_set_mib_directory(optarg);
             break;
 #endif /* NETSNMP_DISABLE_MIB_LOADING */
@@ -446,6 +447,11 @@ snmp_parse_args(int argc,
                     free(ebuf);
                     return (-1);
                 }
+                if ((eout_len < 5) || (eout_len > 32)) {
+                    fprintf(stderr, "Invalid engine ID value after -e flag.\n");
+                    free(ebuf);
+                    return (-1);
+                }
                 session->securityEngineID = ebuf;
                 session->securityEngineIDLen = eout_len;
                 break;
@@ -465,6 +471,11 @@ snmp_parse_args(int argc,
                     free(ebuf);
                     return (-1);
                 }
+                if ((eout_len < 5) || (eout_len > 32)) {
+                    fprintf(stderr, "Invalid engine ID value after -E flag.\n");
+                    free(ebuf);
+                    return (-1);
+                }
                 session->contextEngineID = ebuf;
                 session->contextEngineIDLen = eout_len;
                 break;
