@@ -3,6 +3,7 @@
  */
 
 #include <net-snmp/net-snmp-config.h>
+#include <errno.h>
 #ifdef HAVE_LIMITS_H
 #include <limits.h>
 #endif
@@ -118,7 +119,7 @@ static int      getHwAddress(const char *networkDevice, char *addressOut);
 void
 snmpv3_authtype_conf(const char *word, char *cptr)
 {
-#ifndef DISABLE_MD5
+#ifndef NETSNMP_DISABLE_MD5
     if (strcasecmp(cptr, "MD5") == 0)
         defaultAuthType = usmHMACMD5AuthProtocol;
     else
@@ -148,7 +149,7 @@ snmpv3_privtype_conf(const char *word, char *cptr)
 {
     int testcase = 0;
 
-#ifndef DISABLE_DES
+#ifndef NETSNMP_DISABLE_DES
     if (strcasecmp(cptr, "DES") == 0) {
         testcase = 1;
         defaultPrivType = usmDESPrivProtocol;
@@ -173,7 +174,7 @@ const oid      *
 get_default_privtype(size_t * len)
 {
     if (defaultPrivType == NULL) {
-#ifndef DISABLE_DES
+#ifndef NETSNMP_DISABLE_DES
         defaultPrivType = usmDESPrivProtocol;
 #else
         defaultPrivType = usmAESPrivProtocol;
@@ -195,23 +196,32 @@ get_default_privtype(size_t * len)
  * Line syntax:
  *	defSecurityLevel "noAuthNoPriv" | "authNoPriv" | "authPriv"
  */
+
+int
+parse_secLevel_conf(const char *word, char *cptr) {
+    if (strcasecmp(cptr, "noAuthNoPriv") == 0 || strcmp(cptr, "1") == 0 ||
+	strcasecmp(cptr, "nanp") == 0) {
+        return SNMP_SEC_LEVEL_NOAUTH;
+    } else if (strcasecmp(cptr, "authNoPriv") == 0 || strcmp(cptr, "2") == 0 ||
+	       strcasecmp(cptr, "anp") == 0) {
+        return SNMP_SEC_LEVEL_AUTHNOPRIV;
+    } else if (strcasecmp(cptr, "authPriv") == 0 || strcmp(cptr, "3") == 0 ||
+	       strcasecmp(cptr, "ap") == 0) {
+        return SNMP_SEC_LEVEL_AUTHPRIV;
+    } else {
+        return -1;
+    }
+}
+
 void
 snmpv3_secLevel_conf(const char *word, char *cptr)
 {
     char            buf[1024];
+    int             secLevel;
 
-    if (strcasecmp(cptr, "noAuthNoPriv") == 0 || strcmp(cptr, "1") == 0 ||
-	strcasecmp(cptr, "nanp") == 0) {
+    if ((secLevel = parse_secLevel_conf( word, cptr )) >= 0 ) {
         netsnmp_ds_set_int(NETSNMP_DS_LIBRARY_ID, 
-			   NETSNMP_DS_LIB_SECLEVEL, SNMP_SEC_LEVEL_NOAUTH);
-    } else if (strcasecmp(cptr, "authNoPriv") == 0 || strcmp(cptr, "2") == 0 ||
-	       strcasecmp(cptr, "anp") == 0) {
-        netsnmp_ds_set_int(NETSNMP_DS_LIBRARY_ID, 
-			   NETSNMP_DS_LIB_SECLEVEL, SNMP_SEC_LEVEL_AUTHNOPRIV);
-    } else if (strcasecmp(cptr, "authPriv") == 0 || strcmp(cptr, "3") == 0 ||
-	       strcasecmp(cptr, "ap") == 0) {
-        netsnmp_ds_set_int(NETSNMP_DS_LIBRARY_ID, 
-			   NETSNMP_DS_LIB_SECLEVEL, SNMP_SEC_LEVEL_AUTHPRIV);
+			   NETSNMP_DS_LIB_SECLEVEL, secLevel);
     } else {
         snprintf(buf, sizeof(buf), "Unknown security level: %s", cptr);
         buf[ sizeof(buf)-1 ] = 0;
@@ -262,16 +272,22 @@ snmpv3_options(char *optarg, netsnmp_session * session, char **Apsz,
     switch (*cp) {
 
     case 'Z':
-        session->engineBoots = strtoul(optarg, NULL, 10);
-        if (session->engineBoots == 0 || !isdigit(optarg[0])) {
+        errno=0;
+        session->engineBoots = strtoul(optarg, &cp, 10);
+        if (errno || cp == optarg) {
             fprintf(stderr, "Need engine boots value after -3Z flag.\n");
             return (-1);
         }
-        cp = strchr(optarg, ',');
-        if (cp && *(++cp) && isdigit(*cp))
-            session->engineTime = strtoul(cp, NULL, 10);
-        else {
-            fprintf(stderr, "Need engine time value after -3Z flag.\n");
+        if (*cp == ',') {
+            char *endptr;
+            cp++;
+            session->engineTime = strtoul(cp, &endptr, 10);
+            if (errno || cp == endptr) {
+                fprintf(stderr, "Need engine time after \"-3Z engineBoot,\".\n");
+                return (-1);
+            }
+        } else {
+            fprintf(stderr, "Need engine time after \"-3Z engineBoot,\".\n");
             return (-1);
         }
         break;
@@ -344,7 +360,7 @@ snmpv3_options(char *optarg, netsnmp_session * session, char **Apsz,
         break;
 
     case 'a':
-#ifndef DISABLE_MD5
+#ifndef NETSNMP_DISABLE_MD5
         if (!strcasecmp(optarg, "MD5")) {
             session->securityAuthProto = usmHMACMD5AuthProtocol;
             session->securityAuthProtoLen = USM_AUTH_PROTO_MD5_LEN;
@@ -363,7 +379,7 @@ snmpv3_options(char *optarg, netsnmp_session * session, char **Apsz,
 
     case 'x':
         testcase = 0;
-#ifndef DISABLE_DES
+#ifndef NETSNMP_DISABLE_DES
         if (!strcasecmp(optarg, "DES")) {
             session->securityPrivProto = usmDESPrivProtocol;
             session->securityPrivProtoLen = USM_PRIV_PROTO_DES_LEN;
@@ -489,7 +505,7 @@ snmpv3_options(char *optarg, netsnmp_session * session, char **Apsz,
 int
 setup_engineID(u_char ** eidp, const char *text)
 {
-    int             enterpriseid = htonl(ENTERPRISE_OID),
+    int             enterpriseid = htonl(NETSNMP_ENTERPRISE_OID),
         netsnmpoid = htonl(NETSNMP_OID),
         localsetup = (eidp) ? 0 : 1;
 
@@ -560,6 +576,11 @@ setup_engineID(u_char ** eidp, const char *text)
     len = 5;                    /* always have 5 leading bytes */
     switch (localEngineIDType) {
     case ENGINEID_TYPE_TEXT:
+        if (NULL == text) {
+            snmp_log(LOG_ERR,
+                     "Can't set up engineID of type text from an empty string.\n");
+            return -1;
+        }
         len += strlen(text);    /* 5 leading bytes+text. No NULL char */
         break;
 #if defined(IFHWADDRLEN) && defined(SIOCGIFHWADDR)
@@ -662,9 +683,9 @@ setup_engineID(u_char ** eidp, const char *text)
              * use default NIC if none provided 
              */
             if (NULL == engineIDNic) {
-                x = getHwAddress(DEFAULT_NIC, &bufp[5]);
+	      x = getHwAddress(DEFAULT_NIC, (char *)&bufp[5]);
             } else {
-                x = getHwAddress(engineIDNic, &bufp[5]);
+	      x = getHwAddress((char *)engineIDNic, (char *)&bufp[5]);
             }
             if (0 != x)
                 /*
@@ -726,6 +747,7 @@ free_engineID(int majorid, int minorid, void *serverarg,
     SNMP_FREE(engineID);
     SNMP_FREE(engineIDNic);
     SNMP_FREE(oldEngineID);
+    engineIDIsSet = 0;
     return 0;
 }
 
@@ -804,7 +826,7 @@ usm_parse_create_usmUser(const char *token, char *line)
     /*
      * READ: Authentication Type 
      */
-#ifndef DISABLE_MD5
+#ifndef NETSNMP_DISABLE_MD5
     if (strncmp(cp, "MD5", 3) == 0) {
         memcpy(newuser->authProtocol, usmHMACMD5AuthProtocol,
                sizeof(usmHMACMD5AuthProtocol));
@@ -861,6 +883,7 @@ usm_parse_create_usmUser(const char *token, char *line)
                                newuser->authProtocolLen);
     if (ret2 <= 0) {
         config_perror("Could not get proper authentication protocol key length");
+	usm_free_user(newuser);
         return;
     }
     newuser->authKey = (u_char *) malloc(ret2);
@@ -884,9 +907,9 @@ usm_parse_create_usmUser(const char *token, char *line)
     } else {
         newuser->authKeyLen = ret2;
         ret2 = generate_kul(newuser->authProtocol, newuser->authProtocolLen,
-                            newuser->engineID, newuser->engineIDLen,
-                            userKey, userKeyLen,
-                            newuser->authKey, &newuser->authKeyLen);
+                           newuser->engineID, newuser->engineIDLen,
+                           userKey, userKeyLen,
+                           newuser->authKey, &newuser->authKeyLen);
         if (ret2 != SNMPERR_SUCCESS) {
             config_perror("could not generate localized authentication key "
                           "(Kul) from the master key (Ku).");
@@ -902,7 +925,7 @@ usm_parse_create_usmUser(const char *token, char *line)
      * READ: Privacy Type 
      */
     testcase = 0;
-#ifndef DISABLE_DES
+#ifndef NETSNMP_DISABLE_DES
     if (strncmp(cp, "DES", 3) == 0) {
         memcpy(newuser->privProtocol, usmDESPrivProtocol,
                sizeof(usmDESPrivProtocol));
@@ -974,7 +997,7 @@ usm_parse_create_usmUser(const char *token, char *line)
             usm_free_user(newuser);
             return;
         }
-        newuser->privKey = (u_char *) malloc(ret);
+        newuser->privKey = (u_char *) malloc(ret2);
 
         if (strcmp(buf,"-l") == 0) {
             /* a local key is directly specified */
@@ -990,9 +1013,9 @@ usm_parse_create_usmUser(const char *token, char *line)
         } else {
             newuser->privKeyLen = ret2;
             ret2 = generate_kul(newuser->authProtocol, newuser->authProtocolLen,
-                                newuser->engineID, newuser->engineIDLen,
-                                userKey, userKeyLen,
-                                newuser->privKey, &newuser->privKeyLen);
+                               newuser->engineID, newuser->engineIDLen,
+                               userKey, userKeyLen,
+                               newuser->privKey, &newuser->privKeyLen);
             if (ret2 != SNMPERR_SUCCESS) {
                 config_perror("could not generate localized privacy key "
                               "(Kul) from the master key (Ku).");
@@ -1142,7 +1165,7 @@ void
 version_conf(const char *word, char *cptr)
 {
     int valid = 0;
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
     if ((strcmp(cptr,  "1") == 0) ||
         (strcmp(cptr, "v1") == 0)) {
         netsnmp_ds_set_int(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_SNMPVERSION, 
@@ -1150,7 +1173,7 @@ version_conf(const char *word, char *cptr)
         valid = 1;
     }
 #endif
-#ifndef DISABLE_SNMPV2C
+#ifndef NETSNMP_DISABLE_SNMPV2C
     if ((strcasecmp(cptr,  "2c") == 0) ||
                (strcasecmp(cptr, "v2c") == 0)) {
         netsnmp_ds_set_int(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_SNMPVERSION, 
@@ -1511,7 +1534,7 @@ snmpv3_clone_engineID(u_char ** dest, size_t * destlen, u_char * src,
  *
  * Generates a malloced copy of our engineID.
  *
- * 'length' is set to the length of engineID  -OR-  0 on failure.
+ * 'length' is set to the length of engineID  -OR-  < 0 on failure.
  */
 u_char         *
 snmpv3_generate_engineID(size_t * length)
@@ -1634,7 +1657,7 @@ getHwAddress(const char *networkDevice, /* e.g. "eth0", "eth1" */
 }
 #endif
 
-#ifdef SNMP_TESTING_CODE
+#ifdef NETSNMP_ENABLE_TESTING_CODE
 /*
  * snmpv3_set_engineBootsAndTime(): this function does not exist.  Go away. 
  */
