@@ -40,6 +40,12 @@ SOFTWARE.
 #include <ctype.h>
 #include <errno.h>
 
+#if HAVE_IO_H
+#include <io.h>
+#endif
+#if HAVE_DIRECT_H
+#include <direct.h>
+#endif
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -752,7 +758,7 @@ get_boottime(void)
 long
 get_uptime(void)
 {
-#if !defined(solaris2) && !defined(linux) && !defined(cygwin) && !defined(aix4) && !defined(aix5)
+#if !defined(solaris2) && !defined(linux) && !defined(cygwin) && !defined(aix4) && !defined(aix5) && !defined(aix6)
     struct timeval  now;
     long            boottime_csecs, nowtime_csecs;
 
@@ -765,7 +771,7 @@ get_uptime(void)
     return (nowtime_csecs - boottime_csecs);
 #endif
 
-#if defined(aix4) || defined(aix5)
+#if defined(aix4) || defined(aix5) || defined(aix6)
     struct nlist nl;
     int kmem;
     time_t lbolt;
@@ -1096,14 +1102,13 @@ netsnmp_mktemp(void)
 #else
     if (mktemp(name)) {
 # ifndef WIN32
-        fd = open(name, O_CREAT | O_EXCL | O_WRONLY);
+        fd = open(name, O_CREAT | O_EXCL | O_WRONLY, S_IRUSR | S_IWUSR);
 # else
         /*
          * Win32 needs _S_IREAD | _S_IWRITE to set permissions on file
          * after closing
          */
-        fd = _open(name, _O_CREAT,
-                   _S_IREAD | _S_IWRITE | _O_EXCL | _O_WRONLY);
+        fd = _open(name, _O_CREAT | _O_EXCL | _O_WRONLY, _S_IREAD | _S_IWRITE);
 # endif
     }
 #endif
@@ -1178,7 +1183,7 @@ netsnmp_os_kernel_width(void)
   } else {
     return -1;
   }
-#elif defined(aix4) || defined(aix5)
+#elif defined(aix4) || defined(aix5) || defined(aix6)
   return (__KERNEL_32() ? 32 : (__KERNEL_64() ? 64 : -1));
 #elif defined(osf4) || defined(osf5) || defined(__alpha)
   return 64; /* Alpha is always 64bit */
